unit FileView;

{$I GX_CondDefine.inc}
{.$DEFINE GX_COMP_PACKAGE}
{.$UNDEF SYNTAXMEMO}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, StdCtrls
  {$IFDEF SYNTAXMEMO}
  , SyntaxEd, SynParse
  {$ENDIF SYNTAXMEMO}
  {$IFDEF HAVEJPEG}
  , JPEG
  {$ENDIF HAVEJPEG}
  ;
type
  TFileViewer = class(TCustomPanel)
  private
    FImage: TImage;
    FRichEdit: TRichEdit;
    {$IFDEF SYNTAXMEMO}
    FSyntaxEd: TSyntaxMemo;
    {$ENDIF SYNTAXMEMO}
  protected
  public
    LoadedFile: string;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadFromFile(FileName: string);
    procedure Clear;
  published
    property Align;
  end;

procedure Register;

implementation

{$IFNDEF GX_COMP_PACKAGE}
uses
  GX_uGenFunc;
{$ENDIF}

constructor TFileViewer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Caption := '';
  BevelInner := bvNone;
  BevelOuter := bvLowered;
  Color := clWindow;
  if not (csDesigning in ComponentState) then
  begin
    {Image Viewer}
    FImage := TImage.Create(nil);
    FImage.Parent := Self;
    FImage.Align := alClient;
    FImage.Visible := False;

    {Text Viewer}
    FRichEdit := TRichEdit.Create(nil);

    FRichEdit.Parent := Self;
    FRichEdit.Align := alClient;
    FRichEdit.WordWrap := False;
    FRichEdit.ScrollBars := ssBoth;
    FRichEdit.ReadOnly := True;
    FRichEdit.Visible := False;
    FRichEdit.BorderStyle := bsNone;

    {$IFDEF SYNTAXMEMO}
    {Syntax Memo}
    FSyntaxEd := TSyntaxMemo.Create(nil);
    with FSyntaxEd do
    begin
      Parent := Self;
      Align := alClient;
      Visible := False;
      BorderStyle := bsNone;
      // ReadOnly := True;
      Gutter := 0;
      //SetupSyntaxParsers(FSyntaxEd, [gxpPAS, gxpCPP, gxpHTML, gxpSQL]);
      Options := Options - [smoShowGutter];
    end;
    {$ENDIF SYNTAXMEMO}
  end;
end;

destructor TFileViewer.Destroy;
begin
  if not (csDesigning in ComponentState) then
  begin
    FImage.Free;
    FImage := nil;

    FRichEdit.Free;
    FRichEdit := nil;

    {$IFDEF SYNTAXMEMO}
    FSyntaxEd.Free;
    FSyntaxEd := nil;
    {$ENDIF SYNTAXMEMO}
  end;

  inherited Destroy;
end;

procedure TFileViewer.Clear;
begin
  FImage.Visible := False;
  FRichEdit.Visible := False;
  {$IFDEF SYNTAXMEMO}
  FSyntaxEd.Visible := False;
  {$ENDIF SYNTAXMEMO}
end;

procedure TFileViewer.LoadFromFile(FileName: string);

  {$IFDEF SYNTAXMEMO}
  procedure AssignParser(Parser: GXSyntaxParser);
  begin
    {$IFNDEF GX_COMP_PACKAGE}
    FSyntaxEd.Parser2 := GetSyntaxParser(Parser);
    {$ENDIF}
    FSyntaxEd.ActiveParser := 2;
    FSyntaxEd.Gutter := 0;
    FRichEdit.Visible := False;
    FImage.Visible := False;
    FSyntaxEd.LoadFromFile(FileName);
    (FSyntaxEd.Parser2 as TSyntaxMemoParser).UpdateEditors;
    FSyntaxEd.Visible := True;
  end;
  {$ENDIF SYNTAXMEMO}

var
  Ext: string;
begin
  LoadedFile := FileName;
  {$IFDEF SYNTAXMEMO}
  FSyntaxEd.Lines.BeginUpdate;
  try
    if FSyntaxEd.Gutter > 0 then
      FSyntaxEd.Gutter := 0;
  {$ENDIF SYNTAXMEMO}

    {$IFNDEF GX_COMP_PACKAGE}
    Ext := ExtractUpperFileExt(FileName);
    {$ENDIF}

    if {$IFDEF HAVEJPEG} (Ext = '.JPG') or {$ENDIF HAVEJPEG}
       (Ext = '.BMP') or (Ext = '.ICO') then
    begin
      FRichEdit.Visible := False;
      {$IFDEF SYNTAXMEMO}
      FSyntaxEd.Visible := False;
      {$ENDIF SYNTAXMEMO}
      FImage.Picture.LoadFromFile(Filename);
      FImage.Visible := True;
    end
    else
    if (Ext = '.RTF')
    {$IFNDEF SYNTAXMEMO}
        or (Ext = '.TXT') or (Ext = '.ASC') or (Ext = '.ME')
        or (Ext = '.PAS') or (Ext = '.DPR') or (Ext = '.INC')
        or (Ext = '.HTML') or (Ext = '.HTM') or (Ext = '.DIZ')
        or (Ext = '.C') or (Ext = '.CPP') or (Ext = '.H') or (Ext = '.HPP')
        or (Ext = '.SQL')
    {$ENDIF SYNTAXMEMO}
    then
    begin
      FRichEdit.PlainText := not(Ext = '.RTF');
      FImage.Visible := False;
      {$IFDEF SYNTAXMEMO}
      FSyntaxEd.Visible := False;
      {$ENDIF SYNTAXMEMO}
      FRichEdit.Lines.LoadFromFile(Filename);
      FRichEdit.Visible := True;
    end
    {$IFDEF SYNTAXMEMO}
    else
    if (Ext = '.TXT') or (Ext = '.ASC') or (Ext = '.ME') or (Ext = '.DIZ') then
    begin
      FSyntaxEd.ActiveParser := 1;
      FRichEdit.Visible := False;
      FImage.Visible := False;
      FSyntaxEd.LoadFromFile(FileName);
      FSyntaxEd.Visible := True;
    end
    else
    if (Ext = '.PAS') or (Ext = '.DPR') or (Ext = '.INC') then
      AssignParser(gxpPAS)
    else
    if (Ext = '.HTML') or (Ext = '.HTM') then
      AssignParser(gxpHTML)
    else
    if (Ext = '.C') or (Ext = '.CPP') or (Ext = '.H') or (Ext = '.HPP') then
      AssignParser(gxpCPP)
    else
    if (Ext = '.SQL') then
      AssignParser(gxpSQL)
    {$ENDIF SYNTAXMEMO}
    else
      Clear;

  {$IFDEF SYNTAXMEMO}
    FSyntaxEd.Options := FSyntaxEd.Options - [smoShowGutter];
    FSyntaxEd.Gutter := 0;
  finally
    FSyntaxEd.Lines.EndUpdate;
  end;
  {$ENDIF SYNTAXMEMO}
end;

procedure Register;
begin
  RegisterComponents('User', [TFileViewer]);
end;

end.

