unit GX_eFindDelimiter;

interface

//! StH: This unit has been completely prepared for localization
//! StH: This unit is *not* compatible with C++ Builder
    // it exclusively scans Pascal source code

uses
  mwPasParser,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Menus,
  Dialogs, ExptIntf, EditIntf, GX_EditorExpert, StdCtrls, Registry;

type
  TBaseDelimiterExpert = class(TEditorExpert)
  public
    procedure DoDelimiterAction( EditIntf: TIEditorInterface;
                                 Parser: TmPasParser;
                                 SChar, EChar: TCharPos); virtual; abstract;
    procedure Execute; override;
  end;

  TLocateDelimiter = class(TBaseDelimiterExpert)
  public
    constructor Create; override;
    procedure DoDelimiterAction( EditIntf: TIEditorInterface;
                                 Parser: TmPasParser;
                                 SChar, EChar: TCharPos); override;
    procedure SaveSettings; override;
    procedure LoadSettings; override;
    procedure GetHelpString(List: TStrings); override;
  end;

type
  TMoveToDelimiter = class(TBaseDelimiterExpert)
  public
    constructor Create; override;
    procedure DoDelimiterAction( EditIntf: TIEditorInterface;
                                 Parser: TmPasParser;
                                 SChar, EChar: TCharPos); override;
    procedure SaveSettings; override;
    procedure LoadSettings; override;
    procedure GetHelpString(List: TStrings); override;
  end;

implementation

uses
  ExpertUtil, GX_EditRead, GX_uGenFunc;

resourcestring
  SDelimiterMoveToMessage =
    '  This expert enables you to quickly move to a matching '+
    'delimiter for begin, try, case, repeat, "(" and "[".';

  SDelimiterLocateMessage =
    '  This expert enables you to quickly locate a matching '+
    'delimiter for begin, try, case, repeat, "(" and "[".';

  SDelimiterUsage =
    #13#10+
    '  To use this expert, place the cursor directly on '+
    'the item to match.  For example, if you are searching for an '+
    '"end" matching a "begin", place the cursor in front of the "b" in "begin" and '+
    'execute this expert using the keyboard shortcut.';

resourcestring
  SNotValidIdentifier = '%s is not a valid delimiter.';
  SNoMatchingEndFound = 'No matching closing delimiter was found.';

{ TBaseDelimiterExpert }

procedure TBaseDelimiterExpert.Execute;
var
  EditRead: TEditReader;
  MemStream: TMemoryStream;
  C: Integer;

  StartToken: TTokenKind;
  EndToken: TTokenKind;
  SPos: Integer;
  EPos: Integer;
  SChar: TCharPos;
  EChar: TCharPos;
  ModIntf: TIModuleInterface;
  EditIntf: TIEditorInterface;
  EList: TEditorStrings;
  Parser: TmPasParser;

  EditorLine: string;
  ScanPos: Integer;
begin
  MemStream := TMemoryStream.Create;
  try
    // Since this edit reader is destroyed almost
    // almost immediately, do not call FreeFileData
    EditRead := TEditReader.Create(ToolServices.GetCurrentFile);
    try
      EditorLine := EditRead.GetPrecedingCharacters(256);
      ScanPos := Length(EditorLine);
      while ScanPos > 0 do
      begin
        // none of the token starts with a numeric char,
        // so we scan for alpha chars only.
        if not (EditorLine[ScanPos] in ['a'..'z', 'A'..'Z']) then
          Break;
        Dec(ScanPos);
      end;
      Delete(EditorLine, 1, ScanPos);
      MemStream.Write(PChar(EditorLine)^, Length(EditorLine));

      SPos := EditRead.GetCurrentBufferPos - Length(EditorLine);
      EditRead.SaveToStreamFromPos(MemStream);
    finally
      EditRead.Free;
    end;

    MemStream.Position := 0;

    Parser := TmPasParser.Create;
    try
      Parser.Origin := MemStream.Memory;
      Parser.NextToken;
      StartToken := Parser.Token.ID;
      case Parser.Token.ID of
        tkBegin, tkTry, tkCase,
        tkFinally, tkExcept:    EndToken := tkEnd;
        tkRoundOpen:            EndToken := tkRoundClose;
        tkSquareOpen:           EndToken := tkSquareClose;
        tkRepeat:               EndToken := tkUntil;
      else
        MessageDlg(Format(SNotValidIdentifier, [Parser.Token.Data]), mtError, [mbOK], 0);
        Exit;
      end;

      c := 1;

      while (c > 0) and (Parser.Token.ID <> tkNull) do
      begin
        Parser.NextToken;
        case StartToken of
          tkBegin, tkTry, tkCase:
            begin
              if Parser.Token.Id in [tkBegin, tkTry, tkCase] then
                Inc(c);
            end;

          tkRoundOpen, tkSquareOpen, tkRepeat:
            if Parser.Token.ID = StartToken then
              Inc(c);
        end;
        if Parser.Token.ID = EndToken then
          Dec(c);
      end;

      if c <> 0 then
      begin
        MessageDlg(SNoMatchingEndFound, mtInformation, [mbOK], 0);
        Exit;
      end;

      // Matching terminating delimiter found
      EPos := SPos + Parser.RunPos;
      GetInterfaces(ModIntf, EditIntf);
      try
        if EditIntf <> nil then
        begin
          EList := TEditorStrings.Create(EditIntf);
          try
            // FIXME Unknown AV here sometimes with very simple units?
            SChar := EList.PosToCharPos(SPos);
            EChar := EList.PosToCharPos(EPos);

            DoDelimiterAction(EditIntf, Parser, SChar, EChar);
          finally
            EList.Free;
          end;
        end;
      finally
        EditIntf.Free;
        ModIntf.Free;
      end;
    finally
      Parser.Free;
    end;
  finally
    MemStream.Free;
  end;
end;

{ TMoveToDelimiter }

constructor TMoveToDelimiter.Create;
resourcestring
  SMoveToExpertName = 'Move to Matching Delimiter';
begin
  inherited Create;
  ShortCut := scCtrl + scAlt + Ord('V');
  FName := SMoveToExpertName;
  FButtonNo := 62;
  FHasConfigOptions := False;
end;

procedure TMoveToDelimiter.GetHelpString(List: TStrings);
begin
  List.Text := SDelimiterMoveToMessage + SDelimiterUsage;
end;

procedure TMoveToDelimiter.SaveSettings;
begin
  with TRegIniFile.Create(BaseRegistryKey) do
  try
    WriteInteger('MoveToDelimiter', 'ShortCut', ShortCut); // do not localize
  finally
    Free;
  end;
end;

procedure TMoveToDelimiter.LoadSettings;
begin
  with TRegIniFile.Create(BaseRegistryKey) do
  try
    ShortCut := ReadInteger('MoveToDelimiter', 'ShortCut', ShortCut);  // do not localize
  finally
    Free;
  end;
end;

procedure TMoveToDelimiter.DoDelimiterAction(EditIntf: TIEditorInterface;
  Parser: TmPasParser; SChar, EChar: TCharPos);
var
  CurrentView: Integer;
  EditView: TIEditView;
  EditPos: TEditPos;
  Line: Integer;
begin
  CurrentView := GetCurrentEditView(EditIntf);
  Assert( (CurrentView >= 0) and (EditIntf.GetViewCount > 0) );
  EditView := EditIntf.GetView(CurrentView);
  try
    EditView.ConvertPos(False, EditPos, EChar);
    EditPos.Col := EditPos.Col - Length(Parser.Token.Data);
    if EditPos.Col < 1 then EditPos.Col := 1;
    EditView.CursorPos := EditPos;
    if EditView.CursorPos.Line > EditView.TopPos.Line + EditView.ViewSize.cy then
    begin
      Line := EditView.CursorPos.Line - Editview.ViewSize.cy + 2;
      if Line <= 0 then Line := 1;
      EditPos.Line := Line;
      EditPos.Col := EditView.TopPos.Col;
      EditView.TopPos := EditPos;
    end;
  finally
    EditView.Free;
  end;
end;


//****************** TLocateDelimiter ************************

constructor TLocateDelimiter.Create;
resourcestring
  SLocateDelimiterName = 'Locate Matching Delimiter';
begin
  inherited Create;
  ShortCut := scCtrl + scAlt + Ord('O');
  FName := SLocateDelimiterName;
  FButtonNo := 61;
  FHasConfigOptions := False;
end;

procedure TLocateDelimiter.GetHelpString(List: TStrings);
begin
  List.Text := SDelimiterLocateMessage + SDelimiterUsage;
end;

procedure TLocateDelimiter.SaveSettings;
begin
  with TRegIniFile.Create(BaseRegistryKey) do
  try
    WriteInteger('LocateDelimiter', 'ShortCut', ShortCut);  // do not localize
  finally
    Free;
  end;
end;

procedure TLocateDelimiter.LoadSettings;
begin
  with TRegIniFile.Create(BaseRegistryKey) do
  try
    ShortCut := ReadInteger('LocateDelimiter', 'ShortCut', ShortCut);  // do not localize
  finally
    Free;
  end;
end;

procedure TLocateDelimiter.DoDelimiterAction(EditIntf: TIEditorInterface;
  Parser: TmPasParser; SChar, EChar: TCharPos);
var
  EditView: TIEditView;
  EditPos: TEditPos;
  CurrentView: Integer;
begin
  CurrentView := GetCurrentEditView(EditIntf);
  Assert( (CurrentView >= 0) and (EditIntf.GetViewCount > 0) );
  EditView := EditIntf.GetView(CurrentView);
  try
    EditView.ConvertPos(False, EditPos, SChar);
    EditView.CursorPos := EditPos;
  finally
    EditView.Free;
  end;
  SelectBlock(EditIntf, SChar, EChar);
end;

initialization
  RegisterEditorExpert(TLocateDelimiter);
  RegisterEditorExpert(TMoveToDelimiter);
end.

