unit GX_CompCopyCreate;

{$I GX_CondDefine.inc}

// Copy Component Creation Code to Clipboard
//
// Original author: Primoz Gabrijelcic <primoz.gabrijelcic@altavista.net>
// Last change    : 1999-06-25
// Creation date  : 1999-06-20
// Change history :
//   1999-06-20: First alpha version created by copying and mangling
//               GX_CompReplace.
//   1999-06-21: First beta release.
//   1999-06-25: 'Name' property is set to the component name.
//               Events are exported.
//               Binary properties can be skipped, exported commented or
//               exported uncommented.
//               Simple configuration form aded. Currently, user can only select
//               what will happen to binary properties.
//               Expert now works correctly when nothing is selected.
//
// Tested with: D4, D3, CB4
//
// Thanks to John Hansen, Stefan Hoffmeister for help and suggestions.
//
// Notes:
//  (to-do) When exporting whole form, created code still needs some tweaking.
//  (to-do) Language should be set according to source file but currently Pascal
//    is always generated.
//    - C is PITA, sometimes.
//  (to-do) Config - Remove component (default: no). Make a backup copy. Store
//          it somewhere in memory, make a button on config "Restore component".
//  (to-do) There is no real icon, just a combination of Clip and Grid - feel
//    free to create a better icon.
//  (feature) Special processing works only for TTabSheet and descendants, not
//    for other TabSheet equivalents (TTab95Sheet for example) or other strange
//    components (TTeeChart, for example).

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, DsgnIntf, ToolIntf, ExptIntf, RplWizInfo, EditIntf,
  GX_uExperts, ExtCtrls;

type
  TfmCompCopyCreate = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    rgBinProps: TRadioGroup;
  private
  public
  end;

  TBinProps = (bpSkip, bpComment, bpUncomment);

  TCompCopyCreateExpert = class(TGX_Expert)
  public
    constructor Create; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    procedure Configure; override;
    procedure LoadSettings; override;
    procedure SaveSettings; override;
    function IconFileName: string; override;
    {$IFDEF GX_UseNativeToolsApi}
    procedure Click(Sender: TObject); override;
    {$ELSE}
    procedure Click(Sender: TIMenuItemIntf); override;
    {$ENDIF GX_UseNativeToolsApi}
  private
    ccBinProps: TBinProps;
    procedure DoCopyCreationCode;
  end;

implementation

{$R *.DFM}

uses
{$IFOPT D+}
  GX_DbugIntf,
{$ENDIF D+}
  GX_uConfigurationInfo,
  GX_GExperts,
  GX_uGenFunc,
  Clipbrd,
  Registry;

type
  TCCOptions = (ccBinaryRemove, ccBinaryComment, ccBinaryUncomment);
  TCCOptionSet = set of TCCOptions;

  TComponentCreate = class
  private
    ccDecl    : TStringList;
    ccCrea    : TStringList;
    ccImpl    : TStringList;
    ccDumped  : TStringList;
    ccCompDef : TStringList;
    ccInpPos  : integer;
    ccLn      : string;
    ccULn     : string;
    ccIsDirty : boolean;
    ccForm    : string;
    ccOptions : TCCOptionSet;
    function  Eof: boolean;
    procedure Readln;
    procedure DumpComponent(comp: TComponent; out decl, crea, impl: TStringList);
    procedure StreamAndParse(comp: TComponent; out decl, crea, impl, sub: TStringList);
    procedure ParseComponent(comp: TComponent; out decl, crea, impl, sub: TStringList);
    function  GetDumped: TStringList;
  public
    constructor Create;
    destructor  Destroy; override;
    procedure   Dump(comp: TComponent; options: TCCOptionSet);
    property    Dumped: TStringList read GetDumped;
  end;

procedure AppendStringList(source, additional: TStrings);
var
  i: integer;
begin
  for i := 0 to additional.Count - 1 do
    source.Add(additional[i]);
end; { AppendStringList }

function FindChild(cntr: TWinControl; const ControlName: string): TControl;
var
  i: integer;
begin
  Result := nil;
  with cntr do
    for i := 0 to ControlCount-1 do
      if CompareText(Controls[I].Name, ControlName) = 0 then begin
        Result := Controls[I];
        Exit;
      end;
end; { FindChild }

{ TCompCopyCreateExpert }

constructor TCompCopyCreateExpert.Create;
begin
  inherited Create;
  HasConfigOptions := True;
  HasMenuItem := True;
end; { TCompCopyCreateExpert.Create }

function TCompCopyCreateExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = 'Cop&y Component Creation';
begin
  Result := SMenuCaption;
end; { TCompCopyCreateExpert.GetMenuCaption }

function TCompCopyCreateExpert.GetMenuName: string;
begin
  Result := 'GX_CompCopyCreate';
end; { TCompCopyCreateExpert.GetMenuName }

function TCompCopyCreateExpert.GetMenuMask: string;
begin
  Result := '';
end; { TCompCopyCreateExpert.GetMenuMask }

function TCompCopyCreateExpert.GetName: string;
begin
  Result := 'Component_Copy_Create';
end; { TCompCopyCreateExpert.GetName }

function TCompCopyCreateExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'Copy Component Creation to Clipboard';
begin
  Result := SDisplayName;
end; { TCompCopyCreateExpert.GetDisplayName }

{$IFDEF GX_UseNativeToolsApi}
procedure TCompCopyCreateExpert.Click(Sender: TObject);
{$ELSE}
procedure TCompCopyCreateExpert.Click(Sender: TIMenuItemIntf);
{$ENDIF GX_UseNativeToolsApi}
begin
  DoCopyCreationCode;
end; { TCompCopyCreateExpert.Click }

procedure TCompCopyCreateExpert.LoadSettings;
var
  RegIni: TRegIniFile;
begin
  inherited LoadSettings;

  // do not localize any of the below items
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    ccBinProps := TBinProps(RegIni.ReadInteger('Copy Component Creation', 'Binary properties', 1));
  finally
    RegIni.Free;
  end;
end; { TCompCopyCreateExpert.LoadSettings }

procedure TCompCopyCreateExpert.SaveSettings;
var
  RegIni: TRegIniFile;
begin
  inherited SaveSettings;

  // do not localize any of the below items
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    RegIni.WriteInteger('Copy Component Creation', 'Binary properties', Ord(ccBinProps));
  finally
    RegIni.Free;
  end;
end; { TCompCopyCreateExpert.SaveSettings }

procedure TCompCopyCreateExpert.Configure;
var
  Dlg: TfmCompCopyCreate;
begin
  Dlg := TfmCompCopyCreate.Create(nil);
  try
    Dlg.rgBinProps.ItemIndex := Ord(ccBinProps);
    if Dlg.ShowModal = mrOK then begin
      ccBinProps := TBinProps(Dlg.rgBinProps.ItemIndex);
    end;
  finally Dlg.Free; end;
end; { TCompCopyCreateExpert.Configure }

function TCompCopyCreateExpert.IconFileName: string;
begin
  Result := 'CopyCreate';
end; { TCompCopyCreateExpert.IconFileName }

procedure TCompCopyCreateExpert.DoCopyCreationCode;
var
  FormIntf       : TIFormInterface;
  ModIntf        : TIModuleInterface;
  CompIntf       : TIComponentInterface;
  CompIntf2      : TIComponentInterface;
  i,j            : integer;
  CurrentFileName: string;
  compConv       : TComponentCreate;
  comp           : TComponent;
  compParent     : TControl;
  shouldDump     : boolean;
  dumpOptions    : TCCOptionSet;
begin
  ModIntf := nil;
  FormIntf := nil;
  CompIntf := nil;
  // first assume that we have a Pascal source file
  CurrentFileName := UpperCase(ChangeFileExt(ToolServices.GetCurrentFile, '.PAS'));
  try
    ModIntf := ToolServices.GetModuleInterface(CurrentFileName);
    if ModIntf = nil then begin
      // try again with C++ source file
      CurrentFileName := UpperCase(ChangeFileExt(ToolServices.GetCurrentFile, '.CPP'));
      ModIntf := ToolServices.GetModuleInterface(CurrentFileName);
    end;
    if ModIntf <> nil then begin
      FormIntf := ModIntf.GetFormInterface;
      if FormIntf <> nil then begin
        if FormIntf.GetSelCount > 0 then begin
          compConv := TComponentCreate.Create;
          try
            for i := 0 to FormIntf.GetSelCount-1 do begin
              CompIntf := FormIntf.GetSelComponent(i);
              try
                if CompIntf <>  nil then begin
                  // Only export component if it is not a child of another
                  // component also selected for export.
                  // Primarily used to remove duplicates when user selects
                  // Edit.Select All, GExpert.Copy Component Creation.
                  comp := CompIntf.GetComponentHandle;
                  shouldDump := true;
                  if (comp is TWinControl) or (comp is TGraphicControl) then begin
                    if comp is TWinControl
                      then compParent := (comp as TWinControl).Parent
                      else compParent := (comp as TGraphicControl).Parent;
                    for j := 0 to FormIntf.GetSelCount-1 do begin
                      if i <> j then begin
                        CompIntf2 := FormIntf.GetSelComponent(j);
                        if CompIntf2 <> nil then begin
                          try
                            if compParent = TComponent(CompIntf2.GetComponentHandle) then begin
                              shouldDump := false;
                              break; //for
                            end;
                          finally
                            CompIntf2.Free;
                            CompIntf2 := nil;
                          end;
                        end;
                      end;
                    end; //for
                  end;
                  if shouldDump then begin
                    dumpOptions := [];
                    case ccBinProps of
                      bpSkip: Include(dumpOptions,ccBinaryRemove);
                      bpComment: Include(dumpOptions,ccBinaryComment);
                      bpUncomment: Include(dumpOptions,ccBinaryUncomment);
                    end; //case
                    compConv.Dump(comp,dumpOptions);
                  end;
                end;
              finally
                CompIntf.Free;
                CompIntf := nil;
              end;
            end; //for
            Clipboard.AsText := compConv.Dumped.Text;
          finally compConv.Free; end;
        end;
      end;
    end;
  finally
    if ModIntf <> nil then ModIntf.Free;
    if FormIntf <> nil then FormIntf.Free;
    if CompIntf <> nil then CompIntf.Free;
  end;
end; { TCompCopyCreateExpert.DoCopyCreationCode }

{ TComponentCreate }

resourcestring
  CFmtVar         = 'var';
  CFmtDeclaration = '  %s: %s;';
  CFmtCreation    = '  %s := %s.Create(self);';
  CFmtWith        = '  with %s do begin';
  CFmtParent      = '    Parent := %s;';
  CFmtName        = '    Name := ''%s'';';
  CFmtPageControl = '    PageControl := %s;';
  CFmtBinaryCmt   = '//  %s := // please assign';
  CFmtBinaryUncmt = '    %s := // please assign';
  CFmtAdd         = '    %s.Add(%s);';
  CFmtAssign      = '    %s := %s;';
  CFmtEnd         = '  end;';
  CFmtActivePage  = '  %s.%s := %s;';
  CFmtWithAdd     = '  with %s.Add do begin ';

constructor TComponentCreate.Create;
begin
  inherited Create;
  ccDumped := TStringList.Create;
  ccDecl := TStringList.Create;
  ccCrea := TStringList.Create;
  ccImpl := TStringList.Create;
end; { TComponentCreate.Create }

destructor TComponentCreate.Destroy;
begin
  ccDumped.Free;
  ccDecl.Free;
  ccCrea.Free;
  ccImpl.Free;
  inherited Destroy;
end; { TComponentCreate.Destroy }

procedure TComponentCreate.Dump(comp: TComponent; options: TCCOptionSet);
begin
  ccOptions := options;
  DumpComponent(comp,ccDecl,ccCrea,ccImpl);
  ccIsDirty := true;
end; { TComponentCreate.Dump }

procedure TComponentCreate.DumpComponent(comp: TComponent; 
  out decl, crea, impl: TStringList);

  procedure _DumpComponent(comp: TComponent; out decl, crea, impl: TStringList);
  var
    i  : integer;
    sub: TStringList;
  begin
    sub := TStringList.Create;
    try
      StreamAndParse(comp,decl,crea,impl,sub);
      if comp is TWinControl then with comp as TWinControl do begin
        for i := 0 to ControlCount-1 do
          if sub.IndexOf(Controls[i].Name) < 0
            then _DumpComponent(Controls[i],decl,crea,impl);
      end; //with
    finally sub.Free; end;
  end; { _DumpComponent }

begin
  if comp.Owner <> nil
    then ccForm := UpperCase(comp.Owner.Name)
    else ccForm := UpperCase(comp.Name);
  _DumpComponent(comp,decl,crea,impl);
end; { TComponentCreate.DumpComponent }

function TComponentCreate.Eof: boolean;
begin
  Result := (ccInpPos >= ccCompDef.Count);
end; { TComponentCreate.Eof }

function TComponentCreate.GetDumped: TStringList;
begin
  if ccIsDirty then begin
    ccDumped.Clear;
    ccDumped.Add(CFmtVar);
    AppendStringList(ccDumped,ccDecl);
    ccDumped.Add('');
    AppendStringList(ccDumped,ccCrea);
    AppendStringList(ccDumped,ccImpl);
    ccIsDirty := false;
  end;
  Result := ccDumped;
end; { TComponentCreate.GetDumped }

procedure TComponentCreate.ParseComponent(comp: TComponent;
  out decl, crea, impl, sub: TStringList);
var
  p        : integer;
  compSub  : TStringList;
  compName : string;
  compClass: string;
  cName    : string;
  propVal  : string;
  propName : string;
  last     : boolean;
  childComp: TComponent;
  parent   : string;
  tmps     : string;

  procedure Log(list: TStringList; fmt: string; values: array of const);
  begin
    list.Add(Format(fmt,values));
  end; { Log }

  procedure ProcGlyph(propName: string);
  begin
    if ([ccBinaryComment,ccBinaryUnComment] * ccOptions) <> [] then begin
      p := LastDelimiter('.',propName);
      if p > 0
        then propName := Copy(propName,1,p-1);
      if ccBinaryComment in ccOptions
        then Log(impl,CFmtBinaryCmt,[propName])
        else Log(impl,CFmtBinaryUnCmt,[propName])
    end;
    while not Eof do begin
      Readln;
      if ccLn[Length(ccLn)] = '}' then break;
    end; //while
  end; { ProcGlyph }

  procedure ProcSL(propName: string);
  var
    p: integer;
  begin
    p := LastDelimiter('.',propName);
    if p > 0 then begin
      propName := Copy(propName,1,p-1);
      last := false;
      while not (Eof or last) do begin
        Readln;
        if ccLn[Length(ccLn)] = ')' then begin
          ccLn := Copy(ccLn,1,Length(ccLn)-1);
          last := true;
        end;
        Log(impl,CFmtAdd,[propName,ccLn]);
      end; //while
    end;
  end; { ProcSL }

  procedure ProcItems(propName: string);
  var
    pVal : string;
    pName: string;
    last : boolean;
    p    : integer;
  begin
    last := false;
    while not (Eof or last) do begin
      Readln;
      if ccULn = 'ITEM' then begin
        Log(impl,'  '+CFmtWithAdd,[propName]);
        while not (Eof or last) do begin
          Readln;
          if (ccULn = 'END') or (ccUln = 'END>') then begin
            Log(impl,'  '+CFmtEnd,[nil]);
            if ccLn[Length(ccLn)] = '>' then last := true;
            break;
          end
          else begin
            p := Pos('=',ccLn);
            if p > 0 then begin
              pName := TrimRight(Copy(ccLn,1,p-1));
              pVal  := TrimLeft(Copy(ccLn,p+1,Length(ccLn)-p));
              Log(impl,'  '+CFmtAssign,[pName,pVal]);
            end;
          end;
        end; //while
      end;
    end; //while
  end; { ProcItems }

begin { TComponentCreate.ParseComponent }
  compSub := TStringList.Create;
  try
    ccLn := Copy(ccLn,Length('object')+2,Length(ccLn)-Length('object')-1);
    p := Pos(':',ccLn);
    if p > 0 then begin
      compName  := TrimRight(Copy(ccLn,1,p-1));
      compClass := TrimLeft(Copy(ccLn,p+1,Length(ccLn)-p));
      Log(decl,CFmtDeclaration,[compName,compClass]);
      Log(crea,CFmtCreation,[compName,compClass]);
      Log(impl,CFmtWith,[compName]);
      if comp is TControl then begin
        if ((comp as TControl).Parent = nil) or
           ((comp as TControl).Parent is TCustomForm)
          then parent := 'self'
          else parent := (comp as TControl).Parent.Name;
      end
      else parent := ''; // not really needed, just to keep Delphi from issuing warnings
      Log(impl,CFmtName,[compName]);
      if comp is TControl
        then Log(impl,CFmtParent,[parent]);
      if comp is TTabSheet
        then Log(impl,CFmtPageControl,[parent]);
      while not Eof do begin
        Readln;
        if ccULn = 'END' then break
        else if Copy(ccULn,1,Length('object')) = 'OBJECT' then begin
          tmps := Copy(ccLn,Length('object')+2,Length(ccLn)-Length('object')-1);
          p := Pos(':',tmps);
          if p > 0 then begin
            cName := TrimRight(Copy(tmps,1,p-1));
            sub.Add(cName);
            childComp := comp;
            if comp is TWinControl then begin
              childComp := FindChild(comp as TWinControl,cName);
              if not assigned(childComp)
                then childComp := comp;
            end;
            ParseComponent(childComp,decl,crea,compSub,sub);
          end;
        end
        else begin
          p := Pos('=',ccLn);
          if p > 0 then begin
            propName := TrimRight(Copy(ccLn,1,p-1));
            propVal  := TrimLeft(Copy(ccLn,p+1,Length(ccLn)-p));
            p := Pos('.',propVal);
            if p > 0 then begin
              if UpperCase(Copy(propVal,1,p-1)) = ccForm
                then propVal := Copy(propVal,p+1,Length(propVal)-p);
            end;
            if propVal = '{' // glyphs etc - skip
              then ProcGlyph(propName)
            else if propVal = '(' // string lists
              then ProcSL(propName)
            else if propVal = '<' // ListView columns etc
              then ProcItems(propName)
            else
              Log(impl,CFmtAssign,[propName,propVal]);
          end; //if p > 0
        end; //else if not skip
      end; //while not Eof
      Log(impl,CFmtEnd,[nil]);
    end;
    AppendStringList(impl,compSub);
  finally compSub.Free; end;
end; { TComponentCreate.ParseComponent }

procedure TComponentCreate.Readln;
begin
  ccLn := Trim(ccCompDef[ccInpPos]);
  ccUln := UpperCase(ccLn);
  Inc(ccInpPos);
end; { TComponentCreate.Readln }

procedure TComponentCreate.StreamAndParse(comp: TComponent; 
  out decl, crea, impl, sub: TStringList);
var
  tmpWriter: TWriter;
  iStream  : TMemoryStream;
  oStream  : TMemoryStream;
begin
  iStream := TMemoryStream.Create;
  try
//  iStream.WriteComponent(comp);
    //<Suggested by John Hansen:
    tmpWriter := TWriter.Create(iStream, 4096);
    try
      tmpWriter.Root := comp.Owner;
      tmpWriter.WriteSignature;
      tmpWriter.WriteComponent(comp);
      tmpWriter.WriteListEnd;
      tmpWriter.WriteListEnd;
    finally tmpWriter.Free; end;
    //>
    iStream.Position := 0;
    oStream := TMemoryStream.Create;
    try
      ObjectBinaryToText(iStream,oStream);
      ccCompDef := TStringList.Create;
      try
        oStream.Position := 0;
        ccCompDef.LoadFromStream(oStream);
        ccInpPos := 0;
        while not Eof do begin
          Readln;
          if Copy(ccULn,1,Length('object')) = 'OBJECT'
            then ParseComponent(comp,decl,crea,impl,sub);
        end; //while
      finally ccCompDef.Free; end;
    finally oStream.Free; end;
  finally iStream.Free; end;
end; { TComponentCreate.StreamAndParse }

initialization
  RegisterGX_Expert(TCompCopyCreateExpert);
end.
