unit GX_KibitzComp;

{$I GX_CondDefine.inc}

interface

uses
  Classes;

procedure GetKibitzSymbols(EditControl: TObject; XPos, YPos: Integer; Strings: TStrings);

var
  KibitzEnabled: Boolean;

implementation

{$IFDEF GX_VER120_up}
uses
  Windows, SysUtils, GX_VerDepConst;

type
  PObject = ^TObject;
  TSymbols = array[0..0] of Integer;
  PSymbols = ^TSymbols;
  TUnknowns = packed array [0..0] of Byte;
  PUnknowns = ^TUnknowns;
  TKibitzResult = array[0..81] of Integer;

const
  // dphideXX.bpl
  // Codcmplt.TCodeCompletionManger.GetKibitzInfo(XPos, YPos: Integer; var KibitzResult: TKibitzResult);
  GetKibitzInfoName = '@Codcmplt@TCodeCompletionManager@GetKibitzInfo$qqriir22Comtypes@TKibitzResult';
  // Codcmplt.CodeCompletionManager: TCodeCompletionManager;
  CodeCompletionManagerName = '@Codcmplt@CodeCompletionManager';
  // dccXX.dll
  // KibitzGetValidSymbols(var KibitzResult: TKibitzResult; Symbols: PSymbols; Unknowns: PUnknowns; SymbolCount: Integer): Integer; stdcall;
  KibitzGetValidSymbolsName = 'KibitzGetValidSymbols';
  // corideXX.bpl
  // Comdebug.CompGetSymbolText(Symbol: Integer; var S: string; Unknown: Integer); stdcall;
  CompGetSymbolTextName = '@Comdebug@CompGetSymbolText$qqsp16Comtypes@TSymbolr17System@AnsiStringus';

type
  TGetKibitzInfoProc = procedure(Self: TObject; XPos, YPos: Integer; var KibitzResult: TKibitzResult);
  TKibitzGetValidSymbolsProc = function (var KibitzResult: TKibitzResult; Symbols: PSymbols;
        Unknowns: PUnknowns; SymbolCount: Integer): Integer; stdcall;
  TCompGetSymbolTextProc = procedure (Symbol: Integer; var S: string; Unknown: Integer); stdcall;

var
  GetKibitzInfo: TGetKibitzInfoProc;
  KibitzGetValidSymbols: TKibitzGetValidSymbolsProc;
  CompGetSymbolText: TCompGetSymbolTextProc;
  CodeCompletionManager: ^TObject;

// XPos starts a 0 YPos at 1.
procedure GetKibitzSymbols(EditControl: TObject; XPos, YPos: Integer; Strings: TStrings);
var
  KibitzResult: TKibitzResult;
  SymbolCount: Integer;
  Unknowns: PUnknowns;
  Symbols: PSymbols;
  I: Integer;
  S: string;
  OldEditControl: TObject;
begin
  // exit if kibitz is not enabled.
  if not KibitzEnabled then exit;

  // Save the old edit control (I don't known if it is necessary but I won't harm).
  OldEditControl := PObject(PChar(CodeCompletionManager^) + 4)^;
  PObject(PChar(CodeCompletionManager^) + 4)^ := EditControl;

  // Get general kibitzinfo.
  GetKibitzInfo(CodeCompletionManager^, XPos, YPos, KibitzResult);
  case Byte(KibitzResult[0]) of
    $0B, $08, $09: exit;
   else
     // Get valid symbol count.
     SymbolCount := KibitzGetValidSymbols(KibitzResult, nil, nil, $7FFFFFFF);
     // allocmem for symbols.
     Symbols := AllocMem(SymbolCount * 4);
     try
       Unknowns := AllocMem(SymbolCount);
       try
         // Get symbols.
         KibitzGetValidSymbols(KibitzResult, Symbols, Unknowns, SymbolCount);
         for I := 0 to SymbolCount -1 do
         begin
           // Get the name of the symbol.
           CompGetSymbolText(Symbols[I], S, 2);
           // add the string to the list.
           Strings.Add(S);
         end;
       finally
         FreeMem(Unknowns, SymbolCount);
       end;
     finally
       FreeMem(Symbols, SymbolCount * 4);
     end;
  end;
  // restore the old edit control
  PObject(PChar(CodeCompletionManager^) + 4)^ := OldEditControl;
end;

var
  DphIdeModule: HModule;
  dccModule: HModule;
  CorIdeModule: HModule;

procedure Initialize;
begin
  dphideModule := LoadPackage(DphIdeLibName);
  if dphideModule = 0 then exit;
  CodeCompletionManager := GetProcAddress(DphIdeModule, CodeCompletionManagerName);
  if not Assigned(CodeCompletionManager) then exit;
  GetKibitzInfo := GetProcAddress(dphideModule, GetKibitzInfoName);
  if not Assigned(GetKibitzInfo) then exit;

  dccModule := LoadLibrary(dccLibName);
  KibitzGetValidSymbols := GetProcAddress(dccModule, KibitzGetValidSymbolsName);
  if not Assigned(KibitzGetValidSymbols) then exit;

  corideModule := LoadPackage(CorIdeLibName);
  if corideModule = 0 then exit;
  CompGetSymbolText := GetProcAddress(corideModule, CompGetSymbolTextName);
  if not Assigned(CompGetSymbolText) then exit;

  // if everything succeeded set KibitzEnabled to True.
  KibitzEnabled := True;
end;

procedure Finalize;
begin
  if corideModule <> 0 then
    UnLoadPackage(corideModule);
  if dccModule <> 0 then
    FreeLibrary(dccModule);
  if dphideModule <> 0 then
    UnLoadPackage(dphideModule);
end;

initialization
  Initialize;

finalization
  Finalize;
  
{$ELSE} // D3 or BCB3.
procedure GetKibitzSymbols(EditControl: TObject; XPos, YPos: Integer; Strings: TStrings);
begin
end;
{$ENDIF GX_VER120_up}

end.
