unit GX_uSampleExpert;

{$I GX_CondDefine.inc}

interface

uses
  ToolIntf, ExptIntf,
  GX_uExperts,
  Windows, Messages, SysUtils, Classes, Controls, Forms, Dialogs, StdCtrls;

type
  TfmGxSampleExpertForm = class(TForm)
    btOk: TButton;
    lNote: TLabel;
  end;

  TGxSampleExpert = class(TGX_Expert)
  private
    FSomeData: Integer;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    procedure Configure; override;
    procedure SaveSettings; override;
    procedure LoadSettings; override;
    function IconFileName: string; override;
    {$IFDEF GX_UseNativeToolsApi}
    procedure Click(Sender: TObject); override;
    {$ELSE}
    procedure Click(Sender: TIMenuItemIntf); override;
    {$ENDIF GX_UseNativeToolsApi}
  end;


implementation

{$R *.DFM}

uses
{$IFOPT D+}
  GX_DbugIntf,
{$ENDIF D+}
  Registry, GX_GExperts, GX_uConfigurationInfo;


{ TGxSampleExpert }

constructor TGxSampleExpert.Create;
begin
  inherited Create;

  HasConfigOptions := True;
  HasMenuItem := True;

  // Assign a default to your data
  FSomeData := 42;
end;

destructor TGxSampleExpert.Destroy;
begin
  inherited Destroy;
end;

function TGxSampleExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = 'Sample Expert menu item';
begin
  Result := SMenuCaption;
end;

function TGxSampleExpert.GetMenuName: string;
begin
  Result := 'GX_TheUniqueNameOfTheMenu';
end;

function TGxSampleExpert.GetMenuMask: string;
begin
  // File extensions for which the menu item is available;
  // an empty string means always available
  Result := '';
end;

function TGxSampleExpert.GetName: string;
begin
  Result := 'TheUniqueNameOfTheExpert';
end;

function TGxSampleExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'Name of the expert as displayed';
begin
  Result := SDisplayName;
end;

procedure TGxSampleExpert.Configure;
resourcestring
  SYouClickedConfigure = 'You clicked the Configuration button :-)';
begin
  MessageDlg(SYouClickedConfigure, mtInformation, [mbOK], 0);
end;

procedure TGxSampleExpert.SaveSettings;
var
  RegIni: TRegIniFile;
begin
  inherited SaveSettings;

  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    // write your settings here, for example
    RegIni.WriteInteger('SampleExpert', 'TestSetting', FSomeData);
  finally
    RegIni.Free;
  end;

end;

procedure TGxSampleExpert.LoadSettings;
var
  RegIni: TRegIniFile;
begin
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    // read your settings here, for example
    FSomeData := RegIni.ReadInteger('SampleExpert', 'TestSetting', FSomeData);
  finally
    RegIni.Free;
  end;

  inherited LoadSettings;
end;

{$IFDEF GX_UseNativeToolsApi}
procedure TGxSampleExpert.Click(Sender: TObject);
{$ELSE}
procedure TGxSampleExpert.Click(Sender: TIMenuItemIntf);
{$ENDIF GX_UseNativeToolsApi}
begin
  with TfmGxSampleExpertForm.Create(nil) do
  try
    ShowModal;
  finally
    Release;
  end;
end;

function TGxSampleExpert.IconFileName: string;
begin
  Result := 'FileNameWithoutExtension';
end;

initialization
  RegisterGX_Expert(TGxSampleExpert);
end.

