unit IdeDock;

{$I GX_CondDefine.inc}

interface

uses
  Windows, SysUtils, Classes, IniFiles, Forms, Controls,
  MenuBar, Menus, Messages,
  DummyIdeDock,
  // You must include DsnIdeXX.dcp as a runtime package to compile
  DockForm;

type
{$IFDEF EnableIdeDockingSupport}
  TDummyPopupMenu = class(TPopupMenu)
  private
    OwnerMenu: TMenu;
  public
    function IsShortCut(var Message: TWMKey): Boolean; override;
  end;
{$ENDIF EnableIdeDockingSupport}

{$UNDEF TrickTheIdeAncestorForm}  // this must always be undefined, so that
{$IFDEF TrickTheIdeAncestorForm}  // <--- this define is always false
  TIDEDockForm = class(TDummyIdeDockForm);
{$ELSE}
  TIDEDockForm = class(TDockableForm)
{$ENDIF TrickTheIdeAncestorForm}
  protected
    FMenuBar: TMenuBar;
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    {$IFDEF EnableIdeDockingSupport}
    procedure LoadWindowState(Desktop: TMemIniFile); override;
    procedure SaveWindowState(Desktop: TMemIniFile; IsProject: Boolean); override;
    {$ENDIF EnableIdeDockingSupport}
  end;

type
  TIdeDockFormClass = class of TIdeDockForm;

type
  EIdeDockError = class(Exception);

  TIdeDockManager = class(TObject)
  private
    {$IFDEF EnableIdeDockingSupport}
    FDockForms: TStringList;
    {$ENDIF EnableIdeDockingSupport}
    function GetDockForms: TStringList;
  public
    constructor Create; virtual;
    destructor Destroy; override;

    procedure RegisterDockableForm(IdeDockFormClass: TIdeDockFormClass;
      var IdeDockFormVar; const IdeDockFormName: string); virtual;
    procedure UnRegisterDockableForm(var IdeDockFormVar; const IdeDockFormName: string); virtual;
    procedure ShowForm(Form: TForm);
    property DockForms: TStringList read GetDockForms;
  public
    // Override de/allocator functions to implement Singleton behaviour.
    class function NewInstance: TObject; override;
    procedure FreeInstance; override;
  end;

function IdeDockManager: TIdeDockManager;

implementation

uses
  {$IFOPT D+} DbugIntf,{$ENDIF D+}
  DeskForm, DeskUtil,
  GX_VerDepConst, GX_uGenFunc;

{$R *.DFM}

{ TIdeDockManager }

constructor TIdeDockManager.Create;
begin
  inherited Create;
end;

destructor TIdeDockManager.Destroy;
begin
  inherited Destroy;
end;

function TIdeDockManager.GetDockForms: TStringList;
{$IFDEF EnableIdeDockingSupport}
resourcestring
  SBorlandIdeObjectNotFound = 'BorlandIde object not found';
  SListOfFormsNotFound = 'List of forms not found in BorlandIde object';
  SErrorGettingFormList = 'Error "%s" while getting list of forms';
  SCorIdeLoadingError = 'Cannot load IDE core %s';
const
  BorlandIDEName = '@Ideintf@BorlandIDE';
  // Ideintf.BorlandIDE
type
  PPointer = ^Pointer;
var
  p1: PPointer;
  CorIdeLib: HINST;
  Check_this_code_in_every_release___DeliberateHint___DoNotDelete: Integer;
begin
  if IsStandAlone then
  begin
    Result := nil;
    Exit;
  end;
  if FDockForms = nil then
  begin
    CorIdeLib := GetModuleHandle(CorIdeLibName);
    if CorIdeLib = 0 then
      raise EIdeDockError.CreateFmt(SCorIdeLoadingError, [CorIdeLibName]);

    p1 := GetProcAddress(CorIdeLib, BorlandIDEName);
    if p1 = nil then
      raise EIdeDockError.Create(SBorlandIdeObjectNotFound);

    try
      { TODO -oStefan -Check: The $8 is highly version dependent. DO CHECK THIS. }
      Check_this_code_in_every_release___DeliberateHint___DoNotDelete := 0;
      FDockForms := TStringList(PPointer(Integer(p1^) - $8)^);

      if FDockForms.ClassType <> TStringList then
        raise EIdeDockError.Create(SListOfFormsNotFound);

    except
      on E: EAccessViolation do
        raise EIdeDockError.CreateFmt(SErrorGettingFormList, [E.Message]);
    end;
  end;

  Result := FDockForms;
end;
{$ELSE}
begin
  Result := nil; // dummy return to kill compiler warning
end;
{$ENDIF EnableIdeDockingSupport}

procedure TIdeDockManager.ShowForm(Form: TForm);
begin
  {$IFDEF EnableIdeDockingSupport}
  with Form as TDockableForm do
  begin
    if not Floating then
    begin
      ForceShow;
      FocusWindow(Form);
    end
    else
      Show;
  end;
  {$ELSE}
    Form.Show;
  {$ENDIF EnableIdeDockingSupport}
end;

procedure TIdeDockManager.RegisterDockableForm(IdeDockFormClass: TIdeDockFormClass;
       var IdeDockFormVar; const IdeDockFormName: string);
begin
  {$IFDEF EnableIdeDockingSupport}
  if IsStandAlone then
    Exit;
  Assert(DockForms <> nil);

  DockForms.AddObject(IdeDockFormName, TObject(@IdeDockFormVar));

  RegisterDesktopFormClass(IdeDockFormClass, IdeDockFormName, IdeDockFormName);
  {$ENDIF EnableIdeDockingSupport}
end;

procedure TIdeDockManager.UnRegisterDockableForm(var IdeDockFormVar; const IdeDockFormName: string);
{$IFDEF EnableIdeDockingSupport}
resourcestring
  SIdeDockFormNotFound = 'Ide dock form "%s" not found.';
var
  I: Integer;
{$ENDIF EnableIdeDockingSupport}
begin
  {$IFDEF EnableIdeDockingSupport}
  if IsStandAlone then
    Exit;
  Assert(DockForms <> nil);
  I := DockForms.IndexOf(IdeDockFormName);

  if I = -1 then
    raise EIdeDockError.CreateFmt(SIdeDockFormNotFound, [IdeDockFormName]);

  DockForms.Delete(I);
  {$ENDIF EnableIdeDockingSupport}
end;

var
  PrivateIdeDockManager: TIdeDockManager = nil;
  CanFreeIdeDockManager: Boolean = False;

function IdeDockManager: TIdeDockManager;
begin
  Result := PrivateIdeDockManager;
end;

class function TIdeDockManager.NewInstance: TObject;
resourcestring
  SIdeDockManagerCannotBeCreatedManually = 'The IDE dock manager cannot be created manually.';
begin
  if PrivateIdeDockManager = nil then
    Result := inherited NewInstance
  else
    raise EIdeDockError.Create(SIdeDockManagerCannotBeCreatedManually);
end;

procedure TIdeDockManager.FreeInstance;
begin
  if CanFreeIdeDockManager then
    inherited FreeInstance;
end;

{ TIdeDockForm }

constructor TIdeDockForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  //OutputDebugString(PChar(AnsiString(ClassName + ' - ' + Name + ' - ' + DeskSection)));
  {$IFDEF EnableIdeDockingSupport}
  if Menu <> nil then
  begin
    FMenuBar := TMenuBar.Create(Self);
    FMenuBar.Parent := Self;
    FMenuBar.Menu := Menu;
    FMenuBar.Height := GetSystemMetrics(SM_CYMENU) + 2;
    Menu := nil;
  end;
  if (PopupMenu = nil) and (FMenuBar <> nil) then
  begin
    PopupMenu := TDummyPopupMenu.Create(Self);
    TDummyPopupMenu(PopupMenu).OwnerMenu := FMenuBar.Menu;
  end;

  DeskSection := Name;
  AutoSave := True;
  SaveStateNecessary := True;
  {$ENDIF EnableIdeDockingSupport}
end;

destructor TIDEDockForm.Destroy;
begin
  {$IFDEF EnableIdeDockingSupport}
  SaveStateNecessary := True;
  {$ENDIF EnableIdeDockingSupport}
  inherited;
end;

procedure TIDEDockForm.Loaded;
begin
  inherited;
end;

{$IFDEF EnableIdeDockingSupport}
procedure TIDEDockForm.LoadWindowState(Desktop: TMemIniFile);
begin
  inherited;
end;

procedure TIDEDockForm.SaveWindowState(Desktop: TMemIniFile; IsProject: Boolean);
begin
  inherited;
end;

{ TDummyPopupMenu }

function TDummyPopupMenu.IsShortCut(var Message: TWMKey): Boolean;
begin
  // Call the form's IsShortCut so docked forms can use main menu shortcuts
  Result := (OwnerMenu <> nil) and OwnerMenu.IsShortCut(Message);
end;
{$ENDIF EnableIdeDockingSupport}

initialization
  PrivateIdeDockManager := TIdeDockManager.Create;

finalization
  CanFreeIdeDockManager := True;
  FreeAndNil(PrivateIdeDockManager);

end.
