// Original Author: John Hansen <John_Hansen@tcsmgmt.com>

unit GX_uProjOptionSets;

{$I GX_CondDefine.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, CheckLst, GX_uExperts, ActiveX, IdeDock, E_StgStr,
  TypInfo, Menus, ToolsApi, ComCtrls;

type
  TfmProjOptionSets = class;

  TProjOptionSetsExpert = class(TGX_Expert)
  private
    function GetStorageFile: string;
  protected
    FStoragePath: string;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    function IconFileName: string; override;
    procedure Click(Sender: TObject); override;
    procedure LoadSettings; override;
    procedure SaveSettings; override;
    procedure Configure; override;
    property StorageFile: string read GetStorageFile;
  end;

  TfmProjOptionSets = class(TIdeDockForm)
    pnlLeft: TPanel;
    grpSets: TGroupBox;
    pnlOSInner: TPanel;
    edtSetName: TEdit;
    lstSets: TListBox;
    pmuPrjOptions: TPopupMenu;
    mniPrjClearAll: TMenuItem;
    mniPrjCheckAll: TMenuItem;
    splLeftRight: TSplitter;
    mniPrjSortByName: TMenuItem;
    mniPrjSortByCheckmark: TMenuItem;
    mniPrjAscending: TMenuItem;
    mniPrjDescending: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    barInformation: TStatusBar;
    N3: TMenuItem;
    mniModifyPrjOptionValues: TMenuItem;
    pnlRight: TPanel;
    pcSettings: TPageControl;
    tsProjectSettings: TTabSheet;
    tsEnvironmentSettings: TTabSheet;
    lblProjectSettings: TLabel;
    pnlFilterComboHost: TPanel;
    cbFilter: TComboBox;
    pnlCheckListHost: TPanel;
    pmuEnvOptions: TPopupMenu;
    mniModifyEnvOptionValues: TMenuItem;
    lblCheckListNot: TLabel;
    btnAdd: TButton;
    btnDelete: TButton;
    btnSave: TButton;
    btnApply: TButton;
    btnHelp: TButton;
    procedure lstSetsClick(Sender: TObject);
    procedure edtSetNameChange(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure mniPrjClearAllClick(Sender: TObject);
    procedure mniPrjCheckAllClick(Sender: TObject);
    procedure mniPrjSortByCheckmarkClick(Sender: TObject);
    procedure mniPrjDescendingClick(Sender: TObject);
    procedure pmuPrjOptionsPopup(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure cbFilterChange(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure splLeftRightMoved(Sender: TObject);
    procedure mniModifyEnvOptionValuesClick(Sender: TObject);
    procedure splLeftRightCanResize(Sender: TObject; var NewSize: Integer;
      var Accept: Boolean);
    procedure pnlOSInnerResize(Sender: TObject);
  private
    lstPrjOptions: TCheckListBox;
    lstEnvOptions: TCheckListBox;
    FSetOptions: TStringList;
    FSetChanged: Boolean;
    FLastSetIndex: Integer;
    FStorage: TStructuredStorage;
    FPrjOptions: IOTAProjectOptions;
    FEnvOptions: IOTAEnvironmentOptions;
    SplitterRatio: Double;
    function ProjOptsExpert: TProjOptionSetsExpert;
    procedure lstGenericClickCheck(Sender: TObject);
    procedure HandleOnGetHintPrj(Sender: TObject; const CursorPos: TPoint; var HintStr: string);
    procedure HandleOnGetHintEnv(Sender: TObject; const CursorPos: TPoint; var HintStr: string);
    procedure SetupListOptionsControls;
    procedure FillFilterBox;
    procedure UpdateButtonState;
    procedure GenericClearOptionList(AList: TCheckListBox);
    procedure ClearEnvOptionList;
    procedure ClearPrjOptionList;
    procedure SetAllPrjOptionChecks(ACheckFlag: Boolean);
    procedure LoadPrjOptionList;
    procedure LoadEnvOptionList;
    procedure RefreshSetsList;
    procedure LoadSetOptions;
    procedure SaveSetOptions;
    procedure ApplySetOptions;
    procedure DeleteSetFromStorage;
    procedure UpdateSetListInStorage;
    function GetCheckedPrjOptionCount: Integer;
    function GetValueAsString(AValue: Variant; AKind: TTypeKind): string;
    function GetPrjOptionValue(const AOption: string): string;
    procedure SetOptionValue(const AOption, AValue: string);
    function GetProjectOptions: IOTAProjectOptions;
    function GetEnvironmentOptions: IOTAEnvironmentOptions;
    procedure LoadSettings;
    procedure SaveSettings;
    procedure RefreshCheckmarks;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  fmProjOptionSets: TfmProjOptionSets;
  PrjOptSetsExpert: TProjOptionSetsExpert;

implementation

{$R *.DFM}

uses
  GX_VerDepConst,
  {$IFOPT D+}
  GX_DbugIntf,
  {$ENDIF D+}
  ActnList, Registry, Clipbrd,
  GX_GExperts, Gx_uConfigurationInfo,
  GX_uProjOptOptions, GX_uGenFunc,
  GX_uProjOptMap;

type
  TKindObject = class(TObject)
  private
    FOptionKind: TTypeKind;
  public
    property OptionKind: TTypeKind read FOptionKind write FOptionKind;
  end;

  TJCHListSortCompare = function(Item1, Item2: integer): integer of object;
  TGetHintEvent = procedure(Sender: TObject; const CursorPos: TPoint; var HintStr: string) of object;

  TCheckListBoxWithHints = class(TCheckListBox)
  private
    FOnGetHint: TGetHintEvent;
    FSortAscend: Boolean;
    FSortByString: Boolean;
    procedure CMHintShow(var Message: TMessage); message CM_HINTSHOW;
    procedure SetSortAscend(const Value: Boolean);
    procedure SetSortByString(const Value: Boolean);
    procedure QuickSort(L, R: Integer; SCompare: TJCHListSortCompare);
  protected
    procedure DoOnGetHint(const CursorPos: TPoint; var HintStr: string); // virtual;
    function  CompareByStringAscending(Item1, Item2: Integer): Integer;
    function  CompareByStringDescending(Item1, Item2: Integer): Integer;
    function  CompareByCheckAscending(Item1, Item2: Integer): Integer;
    function  CompareByCheckDescending(Item1, Item2: Integer): Integer;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SortList(Compare: TJCHListSortCompare);
    procedure Resort;
    property SortAscending: Boolean read FSortAscend write SetSortAscend;
    property SortByString: Boolean read FSortByString write SetSortByString;
    property OnGetHint: TGetHintEvent read FOnGetHint write FOnGetHint;
  end;

function BoolToStr(PValue: Boolean): string;
const
  BoolChar: array[Boolean] of Char = ('F', 'T');
begin
  Result := BoolChar[PValue];
end;

function StrToBool(const AString: string): Boolean;
begin
  Result := (AString = 'T');
end;

{ TCheckListBoxWithHints }

procedure TCheckListBoxWithHints.QuickSort(L, R: Integer; SCompare: TJCHListSortCompare);
var
  I, J, P: Integer;
  tmpObj : TObject;
  tmpStr : String;
  tmpChecked : Boolean;
begin
  repeat
    I := L;
    J := R;
    P := (L + R) shr 1;
    repeat
      while SCompare(I, P) < 0 do Inc(I);
      while SCompare(J, P) > 0 do Dec(J);
      if I <= J then
      begin
        // exchange I and J
        tmpStr           := Items[I];
        tmpObj           := Items.Objects[I];
        tmpChecked       := Self.Checked[I];

        Items[I]         := Items[J];
        Items.Objects[I] := Items.Objects[J];
        Self.Checked[I]  := Self.Checked[J];

        Items[J]         := tmpStr;
        Items.Objects[J] := tmpObj;
        Self.Checked[J]  := tmpChecked;

        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then QuickSort(L, J, SCompare);
    L := I;
  until I >= R;
end;

procedure TCheckListBoxWithHints.CMHintShow(var Message: TMessage);
var
  NewHintStr: string;
begin
  NewHintStr := TCMHintShow(Message).HintInfo^.HintStr;
  DoOnGetHint(TCMHintShow(Message).HintInfo^.CursorPos, NewHintStr);
  TCMHintShow(Message).HintInfo^.HintStr := NewHintStr;
  inherited;
end;

constructor TCheckListBoxWithHints.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSortAscend := True;
  FSortByString := True;
end;

procedure TCheckListBoxWithHints.DoOnGetHint(const CursorPos: TPoint; var HintStr: string);
begin
  if Assigned(FOnGetHint) then
    FOnGetHint(Self, CursorPos, HintStr);
end;

procedure TCheckListBoxWithHints.SetSortAscend(const Value: Boolean);
begin
  if Sorted then
    Exit;
  FSortAscend := Value;
  Resort;
end;

procedure TCheckListBoxWithHints.SetSortByString(const Value: Boolean);
begin
  if Sorted then
    Exit;
  FSortByString := Value;
  Resort;
end;

procedure TCheckListBoxWithHints.SortList(Compare: TJCHListSortCompare);
begin
  if Items.Count = 0 then Exit;
  Items.BeginUpdate;
  try
    QuickSort(0, Items.Count - 1, Compare);
  finally
    Items.EndUpdate;
  end;
end;

procedure TCheckListBoxWithHints.Resort;
begin
  if SortByString then
  begin
    if SortAscending then
      SortList(CompareByStringAscending)
    else
      SortList(CompareByStringDescending);
  end
  else
  begin
    if SortAscending then
      SortList(CompareByCheckAscending)
    else
      SortList(CompareByCheckDescending);
  end;
end;

function TCheckListBoxWithHints.CompareByCheckAscending(Item1, Item2: Integer): Integer;
begin
  Result := 0;
  if Self.Checked[Item1] and not Self.Checked[Item2] then
    Result := -1
  else if Self.Checked[Item1] and Self.Checked[Item2] then
    Result := CompareText(Items[Item1], Items[Item2])
  else if not Self.Checked[Item1] and not Self.Checked[Item2] then
    Result := CompareText(Items[Item1], Items[Item2])
  else if not Self.Checked[Item1] and Self.Checked[Item2] then
    Result := 1;
end;

function TCheckListBoxWithHints.CompareByCheckDescending(Item1, Item2: Integer): Integer;
begin
  Result := 0;
  if Self.Checked[Item1] and not Self.Checked[Item2] then
    Result := 1
  else if Self.Checked[Item1] and Self.Checked[Item2] then
    Result := CompareText(Items[Item2], Items[Item1])
  else if not Self.Checked[Item1] and not Self.Checked[Item2] then
    Result := CompareText(Items[Item2], Items[Item1])
  else if not Self.Checked[Item1] and Self.Checked[Item2] then
    Result := -1;
end;

function TCheckListBoxWithHints.CompareByStringAscending(Item1, Item2: Integer): Integer;
begin
  Result := CompareText(Items[Item1], Items[Item2]);
end;

function TCheckListBoxWithHints.CompareByStringDescending(Item1, Item2: Integer): Integer;
begin
  Result := CompareText(Items[Item2], Items[Item1]);
end;

{ TProjOptionSetsExpert }

procedure TProjOptionSetsExpert.Click(Sender: TObject);
begin
  // If the form doesn't exist, create it.
  if fmProjOptionSets = nil then
    fmProjOptionSets := TfmProjOptionSets.Create(nil);
  // Show the form using IdeDockManager.ShowForm.
  IdeDockManager.ShowForm(fmProjOptionSets);
end;

procedure TProjOptionSetsExpert.Configure;
begin
  with TfmProjOptOptions.Create(nil) do
  try
    Path := FStoragePath;
    if ShowModal = mrOK then
      FStoragePath := Path;
  finally
    Free;
  end;
end;

constructor TProjOptionSetsExpert.Create;
begin
  inherited Create;
  FStoragePath := ConfigInfo.ConfigPath;
  HasMenuItem := True; // set to true if you want a menu item
  HasConfigOptions := True; // set to true if you want a "Configure..." button

  if Assigned(PrjOptSetsExpert) then
  begin
    PrjOptSetsExpert.Free;
    PrjOptSetsExpert := nil;
  end;
  PrjOptSetsExpert := Self;
end;

destructor TProjOptionSetsExpert.Destroy;
begin
  fmProjOptionSets.Free;
  fmProjOptionSets := nil;
  PrjOptSetsExpert := nil;
  inherited Destroy;
end;

function TProjOptionSetsExpert.GetDisplayName: string;
resourcestring
  SProjOptionsDisplayName = 'Project Option Sets';
begin
  Result := SProjOptionsDisplayName;
end;

function TProjOptionSetsExpert.GetMenuCaption: string;
resourcestring
  SProjOptionsMenuCaption = 'Project &Option Sets';
begin
  Result := SProjOptionsMenuCaption;
end;

function TProjOptionSetsExpert.GetMenuMask: string;
begin
  Result := '';
end;

function TProjOptionSetsExpert.GetMenuName: string;
begin
  Result := 'GX_ProjOptionSets';
end;

function TProjOptionSetsExpert.GetName: string;
begin
  Result := 'GXProjOptionSets_Expert';
end;

function TProjOptionSetsExpert.GetStorageFile: string;
begin
  Result := FStoragePath + ProjectOptionsSetsFileName;
end;

function TProjOptionSetsExpert.IconFileName: string;
begin
  Result := 'ProjOpts';
end;

procedure TProjOptionSetsExpert.LoadSettings;
begin
  inherited LoadSettings;
  // load your expert-specific settings here
  with TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts') do
  try
    FStoragePath := ReadString('ProjOptionSets', 'StoragePath', FStoragePath);
  finally
    Free;
  end;
  // This procedure is only called once so it is safe to register the form here.
  if Active then
    IdeDockManager.RegisterDockableForm(TfmProjOptionSets,
      fmProjOptionSets, 'fmProjOptionSets');
end;

procedure TProjOptionSetsExpert.SaveSettings;
begin
  inherited SaveSettings;
  // save you expert-specific settings here
  with TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts') do
  try
    WriteString('ProjOptionSets', 'StoragePath', FStoragePath);
  finally
    Free;
  end;
end;

{ TfmProjOptionSets }

constructor TfmProjOptionSets.Create(AOwner: TComponent);
resourcestring
  SModifyOptionValuesMenuCaption = '&Modify Option Values...';
var
  MainForm: TCustomForm;
  ProjectOptions: TComponent;
begin
  inherited Create(AOwner);

  FLastSetIndex := -1;
  SetupListOptionsControls;
  UpdateButtonState;
  FillFilterBox;

  LoadPrjOptionList;
  LoadEnvOptionList;

  TCheckListBoxWithHints(lstPrjOptions).SortByString := True;
  TCheckListBoxWithHints(lstEnvOptions).SortByString := True;

  FStorage := TStructuredStorage.Create;
  FSetOptions := TStringList.Create;
  SplitterRatio := 0.50;
  LoadSettings;

  // Note: optimally, we would go through
  //       IOTAOptions.EditOptions on our
  //       IOTAProjectOptions, but this
  //       fails when no project is loaded
  //       The TAction will work always,
  //       in particular for default project
  //       options, when there is no project
  //       loaded in the IDE.
  MainForm := GX_uGenFunc.GetIdeMainForm;
  if Assigned(MainForm) then
  begin
    ProjectOptions := MainForm.FindComponent('ProjectOptionsCommand');
    if (ProjectOptions <> nil) and (ProjectOptions is TAction) then
    begin
      mniModifyPrjOptionValues.Action:= TAction(ProjectOptions);
      mniModifyPrjOptionValues.Caption := SModifyOptionValuesMenuCaption;
    end;
  end;
  if mniModifyPrjOptionValues.Action = nil then
    mniModifyPrjOptionValues.Enabled := False;
end;

destructor TfmProjOptionSets.Destroy;
begin
  SaveSettings;
  ClearPrjOptionList;

  // before we free our structured storage obect
  // try to defragment our storage file
  if Assigned(FStorage) then
    FStorage.DefragmentStorageFile(ProjOptsExpert.StorageFile);

  // now free our object
  FStorage.Free;
  FStorage := nil;

  FSetOptions.Free;
  FSetOptions := nil;

  fmProjOptionSets := nil;

  inherited Destroy;
end;

procedure TfmProjOptionSets.UpdateButtonState;
var
  HaveSetName: Boolean;
  SetNameIsInList: Boolean;
  IsListItemSelected: Boolean;
begin
  HaveSetName := (edtSetName.Text <> '');
  SetNameIsInList := (lstSets.Items.IndexOf(edtSetName.Text) > -1);
  IsListItemSelected := (lstSets.ItemIndex > -1);

  btnAdd.Enabled := HaveSetName and not SetNameIsInList;
  btnDelete.Enabled := IsListItemSelected;
  // allow save even if the set has not changed (maybe the value
  // of an option has changed...)
  btnSave.Enabled := HaveSetName and IsListItemSelected and
    SetNameIsInList { and FSetChanged};
  btnApply.Enabled := HaveSetName and IsListItemSelected and
    SetNameIsInList and not FSetChanged;
  lstPrjOptions.Enabled := IsListItemSelected;
  lstEnvOptions.Enabled := IsListItemSelected;
  pcSettings.Enabled := IsListItemSelected;
  cbFilter.Enabled := IsListItemSelected;
end;

procedure TfmProjOptionSets.lstSetsClick(Sender: TObject);
begin
  if lstSets.ItemIndex > -1 then
  begin
    if lstSets.ItemIndex <> FLastSetIndex then
    begin
      edtSetName.Text := lstSets.Items[lstSets.ItemIndex];
      // load a options from the selected "file" in structured storage
      LoadSetOptions;
      // store the item index
      FLastSetIndex := lstSets.ItemIndex;
    end;
  end;
  FSetChanged := False;
  UpdateButtonState;
end;

procedure TfmProjOptionSets.edtSetNameChange(Sender: TObject);
begin
  UpdateButtonState;
end;

procedure TfmProjOptionSets.btnAddClick(Sender: TObject);
begin
  if edtSetName.Text <> '' then
  begin
    lstSets.Items.Add(edtSetName.Text);
    // update our set list
    UpdateSetListInStorage;
    lstSets.ItemIndex := lstSets.Items.IndexOf(edtSetName.Text);
  end;
  UpdateButtonState;
end;

procedure TfmProjOptionSets.btnDeleteClick(Sender: TObject);
resourcestring
  sDeleteMsg = 'Are you sure you want to delete this option set?';
begin
  if lstSets.ItemIndex <> -1 then
  begin
    if MessageDlg(sDeleteMsg, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    begin
      // delete a file from our structured storage - this uses
      // the name in edtSetName so we wait to clear the edit
      // box until we have deleted it from storage
      DeleteSetFromStorage;
      edtSetName.Text := '';
      // delete the item from our list
      lstSets.Items.Delete(lstSets.ItemIndex);
      // now update our storage list of sets - this method
      // uses lstSets to re-write the set list in our storage
      UpdateSetListInStorage;
      // clear checks from option list
      SetAllPrjOptionChecks(False);
      UpdateButtonState;
    end;
  end;
end;

procedure TfmProjOptionSets.btnSaveClick(Sender: TObject);
begin
  if edtSetName.Text <> '' then
  begin
    // overwrite the file named edtSetName.Text with
    // current project options information
    SaveSetOptions;
    // now reload set options
    LoadSetOptions;
  end;
  UpdateButtonState;
end;

procedure TfmProjOptionSets.btnApplyClick(Sender: TObject);
begin
  if edtSetName.Text <> '' then
  begin
    // apply the settings in edtSetName.Text to the Project Options
    ApplySetOptions;
  end;
  UpdateButtonState;
end;

resourcestring
  SAllOptions = 'All options';

procedure TfmProjOptionSets.LoadPrjOptionList;
var
  j: Integer;
  OptionNames: TOTAOptionNameArray;
  tmpObj: TKindObject;
{$UNDEF CheckProjectOptionMap}
{.$DEFINE CheckProjectOptionMap}
{$IFDEF CheckProjectOptionMap}
  i: Integer;
  MissingEntries: string;
{$ENDIF CheckProjectOptionMap}
begin
  OptionNames := nil;
  ClearPrjOptionList;
  FPrjOptions := GetProjectOptions;
  if Assigned(FPrjOptions) then
  try
      OptionNames := FPrjOptions.GetOptionNames;
      for j := Low(OptionNames) to High(OptionNames) do
      begin
      {$IFDEF CheckProjectOptionMap}
        i := High(GxOptionsMap);
        while i >= Low(GxOptionsMap) do
          begin
            if CompareText(GxOptionsMap[i].Name, OptionNames[j].Name) = 0 then
              Break;
            Dec(i);
          end;

        if i < Low(GxOptionsMap) then
        begin
          // Add missing project option to list
          MissingEntries := MissingEntries + OptionNames[j].Name;
          MissingEntries := MissingEntries + '(' + GetEnumName(TypeInfo(TTypeKind), Integer(OptionNames[j].Kind)) + '), ';
        end
        else
        begin
          // Sanity check: do we handle the IDE's option
          // with the right type ourselves?
          Assert(GxOptionsMap[i].AssumedTypeKind = OptionNames[j].Kind);
        end;
      {$ENDIF CheckProjectOptionMap}

        // load options honoring filter selection
        if (cbFilter.Text = sAllOptions) or
           (CategoryTextToCategory(cbFilter.Text) in
             GetOptionCategories(OptionNames[j].Name)) then
        begin
          tmpObj := TKindObject.Create;
          try
            tmpObj.OptionKind := OptionNames[j].Kind;
          lstPrjOptions.Items.AddObject(OptionNames[j].Name, tmpObj);
          except
            on E: Exception do
            begin
              tmpObj.Free;
              raise;
            end;
          end;
        end;
        // done loading option list
      end;

    {$IFDEF CheckProjectOptionMap}
      if MissingEntries <> '' then
      begin
        ClipBoard.AsText := MissingEntries;
        // do not localize; this is just test code
        MessageDlg(MissingEntries, mtWarning, [mbOK], 0);
    end;
    {$ENDIF CheckProjectOptionMap}
  finally
    FPrjOptions := nil;
  end;
end;

procedure TfmProjOptionSets.LoadEnvOptionList;
var
  OptionNames: TOTAOptionNameArray;

    procedure AddEnvOptionItem(const OptionName: string);
    var
      i: Integer;
      tmpObj: TKindObject;
      EnvOption: TOTAOptionName;
    begin
      i := High(OptionNames);
      while i >= Low(OptionNames) do
      begin
        if OptionNames[i].Name = OptionName then  // case-sensitive comparison
          Break;
        Dec(i);
      end;

      // Do not localize - this is just for debugging
      Assert(i >= Low(OptionNames), 'Could not find environment option ' + OptionName);

      tmpObj := TKindObject.Create;
      try
        tmpObj.OptionKind := EnvOption.Kind;
        lstEnvOptions.Items.AddObject(OptionNames[i].Name, tmpObj);
      except
        on E: Exception do
        begin
          tmpObj.Free;
          raise;
        end;
      end;
    end;

begin
  ClearEnvOptionList;

  FEnvOptions := GetEnvironmentOptions;
  if Assigned(FEnvOptions) then
  try
    OptionNames := FEnvOptions.GetOptionNames;
    Assert(OptionNames <> nil);

    AddEnvOptionItem('LibraryPath');
    AddEnvOptionItem('BrowsingPath');
    AddEnvOptionItem('RepositoryDir');

    // PackageDPLOutput	tkLString
    // PackageDCPOutput	tkLString

  finally
    FEnvOptions := nil;
  end;
end;

procedure TfmProjOptionSets.FormShow(Sender: TObject);
begin
  // Load project options in case they have not
  // been available yet; this takes care of the
  // case where the form is shown when no project
  // is open. Without these two lines, after doing
  // the above, it would be impossible to use the
  // expert, as no options would ever be re-retrieved.
  if not Assigned(FPrjOptions) then
    LoadPrjOptionList;

  if not Assigned(FEnvOptions) then
    LoadEnvOptionList;

  // Make sure the set name is empty
  edtSetName.Text := '';
  // Clear and reload the list of sets
  RefreshSetsList;
  // Set item index in lstSets
  if lstSets.Items.Count > 0 then
    lstSets.ItemIndex := 0;
  lstSetsClick(lstSets);
  UpdateButtonState;

  // FIXME!
  { TODO -oStefan -cFeature : These are the path settings sheets }
  // tsEnvironmentSettings.TabVisible := False;

  // If we (un)dock, sort order is lost and we need to resort
  if Assigned(lstPrjOptions) then
    TCheckListBoxWithHints(lstPrjOptions).Resort;

  if Assigned(lstEnvOptions) then
    TCheckListBoxWithHints(lstEnvOptions).Resort;
end;

procedure TfmProjOptionSets.lstGenericClickCheck(Sender: TObject);
begin
  FSetChanged := True;
  UpdateButtonState;
end;

procedure TfmProjOptionSets.RefreshSetsList;
var
  Success: Boolean;
  Stream: IStream;
  NumSets, i: Integer;
  SetName: string;
begin
  lstSets.Clear;
  // Read structured storage (ProjOptsExpert.StorageFile) for list of "files"
  // and add them to lstSets
  Success := FStorage.OpenStorageFile(ProjOptsExpert.StorageFile,
    STGM_DIRECT or STGM_READ or
    STGM_SHARE_DENY_WRITE);
  if Success then
  try
    if FStorage.OpenStream(FStorage.StorageFile, 'SETS',
      STGM_DIRECT or STGM_READ or
      STGM_SHARE_EXCLUSIVE, Stream) then
    begin
      Success := FStorage.ReadInt(Stream, NumSets);
      if Success then
      begin
        for i := 0 to NumSets - 1 do
        begin
          Success := FStorage.ReadString(Stream, SetName);
          if Success then
            lstSets.Items.Add(SetName);
        end;
      end;
    end;
  finally
    Stream := nil; // Closes the stream ?
    FStorage.CloseStorageFile;
  end;
  // Add a default, empty set if none exist
  if lstSets.Items.Count = 0 then
  begin
    edtSetName.Text := 'Default';
    btnAdd.Click;
  end;
end;

procedure TfmProjOptionSets.LoadSetOptions;
var
  Success: Boolean;
  Stream: IStream;
  NumOptions, i: Integer;
  Option: string;
begin
  FSetOptions.Clear;
  Success := FStorage.OpenStorageFile(ProjOptsExpert.StorageFile,
    STGM_DIRECT or STGM_READ or
    STGM_SHARE_DENY_WRITE);
  if Success then
  try
    if FStorage.OpenStream(FStorage.StorageFile, edtSetName.Text,
      STGM_DIRECT or STGM_READ or
      STGM_SHARE_EXCLUSIVE, Stream) then
    begin
      Success := FStorage.ReadInt(Stream, NumOptions);
      if Success then
      begin
        for i := 0 to NumOptions - 1 do
        begin
          Success := FStorage.ReadString(Stream, Option);
          if Success then
            FSetOptions.Add(Option);
        end;
      end;
    end;
  finally
    Stream := nil; // Closes the stream ?
    FStorage.CloseStorageFile;
  end;
  RefreshCheckmarks;
  FSetChanged := False;
end;

procedure TfmProjOptionSets.ApplySetOptions;
var
  j: Integer;
  OptionName: string;
begin
  // This method uses FSetOptions and updates the ProjectOptions
  FPrjOptions := GetProjectOptions;
  try
    if Assigned(FPrjOptions) then
    begin
      for j := 0 to FSetOptions.Count - 1 do
      begin
        OptionName := FSetOptions.Names[j];
        SetOptionValue(OptionName, FSetOptions.Values[OptionName]);
      end;
    end;
  finally
    FPrjOptions := nil;
  end;
end;

procedure TfmProjOptionSets.SaveSetOptions;
var
  Success: Boolean;
  NumOptions, j: Integer;
  OptionText: string;
  Stream: IStream;
begin
  if FileExists(ProjOptsExpert.StorageFile) then
    Success := FStorage.OpenStorageFile(ProjOptsExpert.StorageFile,
      STGM_DIRECT or STGM_READWRITE or
      STGM_SHARE_EXCLUSIVE)
  else
    Success := FStorage.CreateStorageFile(ProjOptsExpert.StorageFile,
      STGM_DIRECT or STGM_READWRITE or
      STGM_SHARE_EXCLUSIVE);
  if Success then
  try
    // Delete the existing "file"
    FStorage.DestroyElement(FStorage.StorageFile, edtSetName.Text);
    // Create a new "file"
    Success := FStorage.CreateStream(FStorage.StorageFile, edtSetName.Text,
      STGM_DIRECT or STGM_READWRITE or
      STGM_SHARE_EXCLUSIVE, Stream);
    if Success then
    begin
      NumOptions := GetCheckedPrjOptionCount;
      FStorage.WriteInt(Stream, NumOptions);
      FPrjOptions := GetProjectOptions;
      try
        if Assigned(FPrjOptions) then
        begin
          for j := 0 to lstPrjOptions.Items.Count - 1 do
          begin
            if lstPrjOptions.Checked[j] then
            begin
              OptionText := lstPrjOptions.Items[j] + '=' +
                GetPrjOptionValue(lstPrjOptions.Items[j]);
              FStorage.WriteString(Stream, OptionText);
            end;
          end;
        end;
      finally
        FPrjOptions := nil;
      end;
    end;
  finally
    Stream := nil;
    FStorage.CloseStorageFile;
  end;
end;

procedure TfmProjOptionSets.DeleteSetFromStorage;
var
  Success: Boolean;
begin
  Success := FStorage.OpenStorageFile(ProjOptsExpert.StorageFile,
    STGM_DIRECT or STGM_READWRITE or
    STGM_SHARE_EXCLUSIVE);
  if Success then
  try
    FStorage.DestroyElement(FStorage.StorageFile, edtSetName.Text);
  finally
    FStorage.CloseStorageFile;
  end;
end;

procedure TfmProjOptionSets.UpdateSetListInStorage;
var
  Success: Boolean;
  Stream: IStream;
  i: Integer;
begin
  if FileExists(ProjOptsExpert.StorageFile) then
    Success := FStorage.OpenStorageFile(ProjOptsExpert.StorageFile,
      STGM_DIRECT or STGM_READWRITE or
      STGM_SHARE_EXCLUSIVE)
  else
    Success := FStorage.CreateStorageFile(ProjOptsExpert.StorageFile,
      STGM_DIRECT or STGM_READWRITE or
      STGM_SHARE_EXCLUSIVE);
  if Success then
  try
    FStorage.DestroyElement(FStorage.StorageFile, 'SETS');
    Success := FStorage.CreateStream(FStorage.StorageFile, 'SETS',
      STGM_DIRECT or STGM_READWRITE or
      STGM_SHARE_EXCLUSIVE, Stream);
    if Success then
    begin
      FStorage.WriteInt(Stream, lstSets.Items.Count);
      for i := 0 to lstSets.Items.Count - 1 do
        FStorage.WriteString(Stream, lstSets.Items[i]);
    end;
  finally
    Stream := nil;
    FStorage.CloseStorageFile;
  end;
end;

function TfmProjOptionSets.GetCheckedPrjOptionCount: Integer;
var
  i: Integer;
begin
  Result := 0;
  for i := 0 to lstPrjOptions.Items.Count - 1 do
  begin
    if lstPrjOptions.Checked[i] then
      Inc(Result);
  end;
end;

function TfmProjOptionSets.GetValueAsString(AValue: Variant; AKind: TTypeKind): string;
resourcestring
  SValUnknown = 'unknown';
begin
  case AKind of
    tkUnknown,
    tkInteger,
    tkChar,
    tkEnumeration,
    tkFloat,
    tkString,
    tkSet,
    tkClass,
    tkMethod,
    tkWChar,
    tkLString,
    tkWString,
    tkVariant,
    tkArray,
    tkRecord,
    tkInterface,
    tkInt64,
    tkDynArray:  Result := VarAsType(AValue, varString);
  else
    Result := SValUnknown;
  end;
end;

procedure TfmProjOptionSets.GenericClearOptionList(AList: TCheckListBox);
var
  i: Integer;
begin
  if Assigned(AList) then
  begin
    for i := 0 to AList.Items.Count - 1 do
      AList.Items.Objects[i].Free;

    AList.Items.Clear;
  end;
end;

procedure TfmProjOptionSets.ClearPrjOptionList;
begin
  GenericClearOptionList(lstPrjOptions);
end;

procedure TfmProjOptionSets.ClearEnvOptionList;
begin
  GenericClearOptionList(lstEnvOptions);
end;

procedure TfmProjOptionSets.SetAllPrjOptionChecks(ACheckFlag: Boolean);
var
  i: Integer;
begin
  for i := 0 to lstPrjOptions.Items.Count - 1 do
    lstPrjOptions.Checked[i] := ACheckFlag;
end;

procedure TfmProjOptionSets.mniPrjClearAllClick(Sender: TObject);
begin
  SetAllPrjOptionChecks(False);
  FSetChanged := (FSetOptions.Count <> 0);
  UpdateButtonState;
end;

procedure TfmProjOptionSets.mniPrjCheckAllClick(Sender: TObject);
begin
  SetAllPrjOptionChecks(True);
  FSetChanged := (lstPrjOptions.Items.Count <> FSetOptions.Count);
  UpdateButtonState;
end;

procedure TfmProjOptionSets.SetupListOptionsControls;
begin
  lstPrjOptions := TCheckListBoxWithHints.Create(Self);
  lstPrjOptions.Align := alClient;
  lstPrjOptions.Parent := pnlCheckListHost;
  lstPrjOptions.PopupMenu := pmuPrjOptions;
  lstPrjOptions.OnClickCheck := lstGenericClickCheck;
  lstPrjOptions.ShowHint := True;
  TCheckListBoxWithHints(lstPrjOptions).OnGetHint := HandleOnGetHintPrj;
  // Activate this to update the hint as the mouse moves, without
  // having to select the listbox item under the cursor
  //TCheckListBoxWithHints(lstPrjOptions).OnMouseMove := lstOptionsMouseMove;

  lstEnvOptions := TCheckListBoxWithHints.Create(Self);
  lstEnvOptions.Align := alClient;
  lstEnvOptions.Parent := tsEnvironmentSettings;
  lstEnvOptions.PopupMenu := pmuEnvOptions;
  lstEnvOptions.OnClickCheck := lstGenericClickCheck;
  lstEnvOptions.ShowHint := True;
  TCheckListBoxWithHints(lstEnvOptions).OnGetHint := HandleOnGetHintEnv;
  // Activate this to update the hint as the mouse moves, without
  // having to select the listbox item under the cursor
  //TCheckListBoxWithHints(lstEnvOptions).OnMouseMove := lstOptionsMouseMove;
end;

procedure TfmProjOptionSets.FillFilterBox;
var
  i: TGxOptionCategory;
begin
  cbFilter.Items.Add(SAllOptions);
  cbFilter.ItemIndex := 0;

  for i := Low(TGxOptionCategory) to High(TGxOptionCategory) do
    cbFilter.Items.AddObject(GxOptionsCategoryText[i], Pointer(i));
end;

function TfmProjOptionSets.GetPrjOptionValue(const AOption: string): string;
var
  idx: Integer;
  tmpObj: TKindObject;
begin
  Result := '';
  if Assigned(FPrjOptions) then
  begin
    idx := lstPrjOptions.Items.IndexOf(AOption);
    if idx <> -1 then
    begin
      tmpObj := TKindObject(lstPrjOptions.Items.Objects[idx]);
      Result := GetValueAsString(FPrjOptions.Values[AOption], tmpObj.OptionKind);
    end;
  end;
end;

procedure TfmProjOptionSets.SetOptionValue(const AOption, AValue: string);
begin
  if Assigned(FPrjOptions) then
    FPrjOptions.Values[AOption] := AValue;
end;

function TfmProjOptionSets.GetProjectOptions: IOTAProjectOptions;
var
  Modules: IOTAModuleServices;
  i: Integer;
  AModule: IOTAModule;
  AProject: IOTAProject;
begin
  Result := nil;
  Modules := (BorlandIDEServices as IOTAModuleServices);
  for i := 0 to Modules.ModuleCount - 1 do
  begin
    AModule := Modules.Modules[i];
    if (AProject = nil) and (AModule.QueryInterface(IOTAProject, AProject) = S_OK) then
    begin
      { TODO -oStefan -cC++Builder : C++Builder 4.0 and 4.01 crash on AProject.ProjectOptions.
                                     This is a C++Builder IDE bug }
      Result := AProject.ProjectOptions;
      Break;
    end;
  end;
end;

function TfmProjOptionSets.GetEnvironmentOptions: IOTAEnvironmentOptions;
var
  IServices: IOTAServices;
begin
  Result := nil;

  IServices := (BorlandIDEServices as IOTAServices);
  Result := IServices.GetEnvironmentOptions;
end;

procedure TfmProjOptionSets.HandleOnGetHintPrj(Sender: TObject; const CursorPos: TPoint; var HintStr: string);
resourcestring
  SOptValue = ' value';
  SOptSaved = 'saved: ';
  SOptCurrent = 'current: ';
const
  CRLF = #13#10;
var
  idx: Integer;
  OptionName: string;
begin
  HintStr := '';
  // HintStr := 'ItemIndex = ' + IntToStr(lstPrjOptions.ItemIndex);

  // By default display hint for that
  // item over which the cursor hovers.
  idx := lstPrjOptions.ItemAtPos(CursorPos, True);
  if not (idx >= 0) then
    idx := lstPrjOptions.ItemIndex;

  if idx <> -1 then
  begin
    OptionName := lstPrjOptions.Items[idx];
    HintStr := GetOptionDescription(OptionName);
    if HintStr <> '' then
      HintStr := HintStr + CRLF;
    HintStr := HintStr + OptionName + SOptValue;
    if lstPrjOptions.Checked[idx] and
      (FSetOptions.IndexOfName(OptionName) <> -1) then
    begin
      HintStr := HintStr + CRLF;
      // Get the saved value
      HintStr := HintStr + SOptSaved + TranslatedValue(OptionName, FSetOptions.Values[OptionName]);
    end;
    HintStr := HintStr + CRLF;
    // Get current value
    FPrjOptions := GetProjectOptions;
    try
      HintStr := HintStr + SOptCurrent + TranslatedValue(OptionName, GetPrjOptionValue(OptionName));
    finally
      FPrjOptions := nil;
    end;
  end;
end;

procedure TfmProjOptionSets.HandleOnGetHintEnv(Sender: TObject; const CursorPos: TPoint; var HintStr: string);
begin
  // TODO -oStefan: implement
end;

procedure TfmProjOptionSets.SaveSettings;
begin
  // Do not localize any of the below items
  with TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts') do
  try
    WriteInteger('ProjOptionSets', 'Height', Height);
    WriteInteger('ProjOptionSets', 'Width', Width);
    //WriteInteger('ProjOptionSets', 'LeftWidth', pnlLeft.Width);
    WriteInteger('ProjOptionSets', 'SplitterRatio', Round(SplitterRatio * 100));
    WriteString('ProjOptionSets', 'ActiveTab', pcSettings.ActivePage.Name);
  finally
    Free;
  end;
end;

procedure TfmProjOptionSets.LoadSettings;
var
  ActiveTabSheetName: string;
begin
  // Do not localize any of the below items
  with TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts') do
  try
    ActiveTabSheetName := ReadString('ProjOptionSets', 'ActiveTab', '');
    if ActiveTabSheetName = tsEnvironmentSettings.Name then
      pcSettings.ActivePage := tsEnvironmentSettings
    else
      pcSettings.ActivePage := tsProjectSettings;
    Width := ReadInteger('ProjOptionSets', 'Width', Width);
    Height := ReadInteger('ProjOptionSets', 'Height', Height);
    //pnlLeft.Width := ReadInteger('ProjOptionSets', 'LeftWidth', pnlLeft.Width);
    SplitterRatio := ReadInteger('ProjOptionSets', 'SplitterRatio', 50) / 100;
    FormResize(Self);
  finally
    Free;
  end;
end;

procedure TfmProjOptionSets.mniPrjSortByCheckmarkClick(Sender: TObject);
begin
  TCheckListBoxWithHints(lstPrjOptions).SortByString := (Sender = mniPrjSortByName);
end;

procedure TfmProjOptionSets.mniPrjDescendingClick(Sender: TObject);
begin
  TCheckListBoxWithHints(lstPrjOptions).SortAscending := (Sender = mniPrjAscending);
end;

procedure TfmProjOptionSets.pmuPrjOptionsPopup(Sender: TObject);
begin
  if TCheckListBoxWithHints(lstPrjOptions).SortAscending then
    mniPrjAscending.Checked := True
  else
    mniPrjDescending.Checked := True;

  if TCheckListBoxWithHints(lstPrjOptions).SortByString then
    mniPrjSortByName.Checked := True
  else
    mniPrjSortByCheckmark.Checked := True;
end;

function TfmProjOptionSets.ProjOptsExpert: TProjOptionSetsExpert;
begin
  Result := PrjOptSetsExpert;
end;

procedure TfmProjOptionSets.btnHelpClick(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 27);
end;

procedure TfmProjOptionSets.cbFilterChange(Sender: TObject);
begin
  lstPrjOptions.Items.BeginUpdate;
  try
    LoadPrjOptionList;
    RefreshCheckmarks;
  finally
    lstPrjOptions.Items.EndUpdate;
    UpdateButtonState;
  end;
end;

procedure TfmProjOptionSets.RefreshCheckmarks;
var
  i, j : integer;
begin
  if not Assigned(FSetOptions) then
    Exit;
  if lstPrjOptions.Items.Count = 0 then
    Exit;
  // Clear all checks in lstPrjOptions
  SetAllPrjOptionChecks(False);
  // Now iterate across FSetOptions and check any items in lstPrjOptions that match
  for i := 0 to FSetOptions.Count - 1 do
  begin
    j := lstPrjOptions.Items.IndexOf(FSetOptions.Names[i]);
    if j <> -1 then
      lstPrjOptions.Checked[j] := True;
  end;
  // Resort the list
  TCheckListBoxWithHints(lstPrjOptions).Resort;
end;

procedure TfmProjOptionSets.FormResize(Sender: TObject);
begin
  pnlLeft.Width := Trunc(SplitterRatio * ClientWidth); //(pcSettings.Width + pnlLeft.Width));
end;

procedure TfmProjOptionSets.splLeftRightMoved(Sender: TObject);
begin
  SplitterRatio := pnlLeft.Width / (pnlLeft.Width + pcSettings.Width);
  FormResize(Self);
end;

procedure TfmProjOptionSets.mniModifyEnvOptionValuesClick(Sender: TObject);
var
  IEnvironmentOptions: IOTAEnvironmentOptions;
begin
  IEnvironmentOptions := GetEnvironmentOptions;
  if Assigned(IEnvironmentOptions) then
    IEnvironmentOptions.EditOptions;
end;

procedure TfmProjOptionSets.splLeftRightCanResize(Sender: TObject;
  var NewSize: Integer; var Accept: Boolean);
var
  SizingSplitter: TSplitter;
begin
  SizingSplitter := (Sender as TSplitter);
  if NewSize < SizingSplitter.MinSize then
    NewSize := SizingSplitter.MinSize;
end;

procedure TfmProjOptionSets.pnlOSInnerResize(Sender: TObject);
begin
  lstSets.Width := pnlOSInner.Width - lstSets.Left - btnAdd.Width - 9;
  edtSetName.Width := lstSets.Width;
  lstSets.Height := pnlOSInner.ClientHeight - edtSetname.Top - edtSetName.Height - 10;
end;

initialization
  {$IFNDEF VER125}
    //! StH: C++Builder crashes on the call to GetProjectOptions.
    RegisterGX_Expert(TProjOptionSetsExpert);
  {$ENDIF VER125}
end.

 