unit GX_Dfm2Txt;

// Original Author: John Hansen <John_Hansen@tcsmgmt.com>

{$I GX_CondDefine.inc}

interface

uses
  ExpertUtil, EditIntf;

type
  TDfm2TxtHelper = class(TObject)
  private
    fProjNotifier: TRLProjectNotifier;
    fModNotifier: TModuleNotifier;
    fExtension: string;
    FActive: boolean;
    procedure SetActive(const Value: boolean);
  protected
    procedure ModAfterSave(Sender: TObject; ModuleInterface: TIModuleInterface);
  public
    constructor Create; virtual;
    destructor Destroy; override;
    property FileExtension: string read fExtension write fExtension;
    property Active: boolean read FActive write SetActive;
  end;

implementation

uses
  {$IFOPT D+}
  GX_DbugIntf,
  {$ENDIF D+}
  SysUtils, Classes;

const
  DEFAULT_EXTENSION = '.txt';

{ TDfm2TxtHelper }

constructor TDfm2TxtHelper.Create;
begin
  inherited Create;
  fActive := False;
  fExtension := DEFAULT_EXTENSION;
end;

destructor TDfm2TxtHelper.Destroy;
begin
  fModNotifier.Free;
  fModNotifier := nil;
  fProjNotifier.Free;
  fProjNotifier := nil;
  inherited Destroy;
end;

procedure TDfm2TxtHelper.ModAfterSave(Sender: TObject; ModuleInterface: TIModuleInterface);
var
  txtName: string;
  frmIntf: TIFormInterface;
  BinStream: TFileStream;
  txtStream: TFileStream;
begin
  // do nothing if we are not supposed to be active
  if not Active then Exit;
  try
    // does this module have a form?
    frmIntf := ModuleInterface.GetFormInterface;
    if Assigned(frmIntf) then
    begin
      txtName := ChangeFileExt(frmIntf.FileName, FileExtension);
      {$IFOPT D+}SendDebug('DFM2TXT in AfterSave: name = ' + txtName); {$ENDIF}
      BinStream := TFileStream.Create(frmIntf.FileName, fmOpenRead or fmShareDenyNone);
      try
        {$IFOPT D+}SendDebug('DFM2TXT in AfterSave: opened the DFM file for reading'); {$ENDIF}
        txtStream := TFileStream.Create(txtName, fmCreate);
        try
          {$IFOPT D+}SendDebug('DFM2TXT in AfterSave: opened the DXT file for writing'); {$ENDIF}
          ObjectResourceToText(BinStream, txtStream);
          {$IFOPT D+}SendDebug('DFM2TXT in AfterSave: wrote file'); {$ENDIF}
        finally
          txtStream.Free;
        end;
      finally
        BinStream.Free;
      end;
    end;
  except
    // swallow exceptions
    on E: Exception do
      {$IFOPT D+}SendDebug('DFM2TXT AfterSave Exception: ' + E.Message); {$ENDIF}
  end;
end;

procedure TDfm2TxtHelper.SetActive(const Value: Boolean);
begin
  {$IFDEF VER130}
  Exit; // Code in OwnerList unit causes shutdown problems in D5??
  {$ENDIF VER130}
  if Value = FActive then
    Exit;
  if Value then
  begin
    fProjNotifier := TRLProjectNotifier.Create(nil);
    fModNotifier := TModuleNotifier.Create(nil);
    fProjNotifier.ModuleNotifier := fModNotifier;
    fModNotifier.OnAfterSave := ModAfterSave;
  end
  else
  begin
    fProjNotifier.Free;
    fProjNotifier := nil;
    fModNotifier.Free;
    fModNotifier := nil;
  end;
  FActive := Value;
end;

end.

