unit GX_ExeExplr;

{$I GX_CondDefine.inc}

interface

//! StH: This unit has been completely prepared for localization
//! StH: This unit is fully compatible with C++ Builder

{ TODO -oStefan -cFeature : Switch to virtual listbox for D4+ }

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, ExtCtrls, HeadList, GX_uPE, ComCtrls, Menus, Registry,
  {$IFDEF ACEREPORTER}
  ACEOut, ACEFile, GX_uPRFunc,
  {$ENDIF ACEREPORTER}
  GX_uExperts, ExptIntf, ToolIntf, IdeDock;

type
  TfmEXEView = class(TIdeDockForm)
    pnToolBar: TPanel;
    sbOpen: TSpeedButton;
    dlgOpen: TOpenDialog;
    pcMain: TPageControl;
    tshMSDOS: TTabSheet;
    tshImport: TTabSheet;
    tshExports: TTabSheet;
    tshPEHEader: TTabSheet;
    hlMSDOS: THeaderListbox;
    hlPEHeader: THeaderListbox;
    hlImports: THeaderListbox;
    Splitter1: TSplitter;
    hlImportFunctions: THeaderListbox;
    hlExportFunctions: THeaderListbox;
    tshPEOptional: TTabSheet;
    hlPEOptionalHeader: THeaderListbox;
    sbPrint: TSpeedButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Options1: TMenuItem;
    Decimal1: TMenuItem;
    Hex1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Print1: TMenuItem;
    N2: TMenuItem;
    PrinterSetup1: TMenuItem;
    PrinterSetupDialog1: TPrinterSetupDialog;
    Help2: TMenuItem;
    N3: TMenuItem;
    sbHelp: TSpeedButton;
    procedure sbOpenClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure hlImportsClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure SetNumberType(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure PrinterSetup1Click(Sender: TObject);
    procedure sbPrintClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Help2Click(Sender: TObject);
    procedure pcMainChange(Sender: TObject);
    procedure sbHelpClick(Sender: TObject);
  private
    { Private declarations }
    PEInfo: TPEFileInfo;
    FNumberType: TNumberType;
    FFileName: string;
    procedure LoadPEInfo(const FileName: string);
    procedure SaveSettings;
    procedure LoadSettings;
    {$IFDEF ACEREPORTER}
    procedure PreviewReport(AceFile: TAceFile);
    {$ENDIF ACEREPORTER}
  public
    { Public declarations }
    property FileName: string read FFileName;
    property NumberType: TNumberType read FNumberType write FNumberType;
  end;

  TPEExpert = class(TGX_EnhExpert)
  protected
    procedure SetActive(New: Boolean); override;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    function IconFileName: string; override;
    procedure Click(Sender: TObject); override;
  end;

implementation

{$R *.DFM}

uses
  GX_uConfigurationInfo, GX_GExperts,
{$IFOPT D+}
  GX_DbugIntf,
{$ENDIF D+}
  GX_uGenFunc
  {$IFDEF ACEREPORTER}
  , GX_uPreview
  {$ENDIF ACEREPORTER}
  ;

var
  fmEXEView: TfmEXEView = nil;
  PEExpert: TPEExpert = nil;

resourcestring
  SNoAceReporterHint = 'Printing is disabled, because ACE Reporter was not compiled into GExperts';

procedure TfmEXEView.sbOpenClick(Sender: TObject);
begin
  if dlgOpen.Execute then
    LoadPEInfo(dlgOpen.FileName);
end;

procedure TfmEXEView.LoadPEInfo(const FileName: string);
resourcestring
  SFormCaption = 'PE Information - ';
begin
  Screen.Cursor := crHourglass;
  try
    Self.FFileName := FileName;
    Caption := SFormCaption + ExtractFileName(FileName);
    pcMain.ActivePage := tshMSDOS;
    PEInfo.Free;
    PEInfo := nil;
    try
      PEInfo := TPEFileInfo.Create(FileName, NumberType);
      hlImportFunctions.Clear;
      hlMSDOS.Items.Assign(PEInfo.MSDOSHeader);
      hlPEHeader.Items.Assign(PEInfo.PEHEaderList);
      hlPEOptionalHeader.Items.Assign(PEInfo.PEOptionalHEaderList);
      hlExportFunctions.Items.Assign(PEInfo.ExportList);
      hlImports.Items.Assign(PEInfo.ImportList);
    except
      on E: Exception do
      begin
        ShowExceptionErrorMessage(E);
      end;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfmEXEView.FormCreate(Sender: TObject);
begin
  inherited;
  try
    {$IFNDEF ACEREPORTER}
    sbPrint.Enabled := False;
    sbPrint.Hint := SNoAceReporterHint;
    Print1.Enabled := False;
    PrinterSetup1.Enabled := False;
    {$IFOPT D+}SendDebug(SNoAceReporterHint);{$ENDIF}
    {$ENDIF ACEREPORTER}
    PEInfo := nil;
    pcMain.ActivePage := tshMSDOS;
    CenterForm(Self);
    LoadSettings;
    Decimal1.Checked := (NumberType = ntDecimal);
    Hex1.Checked := (NumberType = ntHex);
  except
    on E: Exception do
      ShowExceptionErrorMessage(E);
  end;
end;

procedure TfmEXEView.FormDestroy(Sender: TObject);
begin
  fmEXEView := nil;
  try
    SaveSettings;
    PEInfo.Free;
    PEInfo := nil;
  except
    on E: Exception do
      ShowExceptionErrorMessage(E);
  end;
  inherited;
end;

procedure TfmEXEView.hlImportsClick(Sender: TObject);
var
  ImpExp: TImportExport;
  i: Integer;
begin
  hlImportFunctions.Items.BeginUpdate;
  try
    try
      hlImportFunctions.Items.Clear;
      if hlImports.ItemIndex < 0 then
        Exit;
      ImpExp := TImportExport(PEInfo.ImportList.Objects[hlImports.ItemIndex]);
      for i := 0 to ImpExp.Count - 1 do
        hlImportFunctions.Items.Add(ImpExp.Items[i].FunctionName + #9 + PEInfo.IntToNum(ImpExp.Items[i].Ordinal));
    except
      on E: Exception do
        ShowExceptionErrorMessage(E);
    end;
  finally
    hlImportFunctions.Items.EndUpdate;
  end;
end;

procedure TfmEXEView.FormResize(Sender: TObject);
begin
  try
    with hlMSDOS do
      Sections[1].Width := Width - Sections[0].Width;
    with hlPEHeader do
      Sections[1].Width := Width - Sections[0].Width;
    with hlPEOptionalHeader do
      Sections[1].Width := Width - Sections[0].Width;
    with hlImportFunctions do
      Sections[0].Width := Width - Sections[1].Width;
    hlImports.Sections[0].Width := hlImports.Width;
    with hlExportFunctions do
      Sections[0].Width := Width - Sections[1].Width - Sections[2].Width;
  except
    on E: Exception do
    begin
      // ignore
    end;
  end;
end;

procedure TfmEXEView.SetNumberType(Sender: TObject);
begin
  FNumberType := TNumberType(TMenuItem(Sender).Tag);
  if PEInfo <> nil then
    LoadPEInfo(FFilename);
  Decimal1.Checked := (NumberType = ntDecimal);
  Hex1.Checked := (NumberType = ntHex);
end;

procedure TfmEXEView.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfmEXEView.PrinterSetup1Click(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
end;

procedure TfmEXEView.SaveSettings;
begin
  // do not localize any of the below lines
  //! StH: Add StandAlone conditional?
  with TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts') do
  try
    WriteInteger('PEDump', 'Left', Left);
    WriteInteger('PEDump', 'Top', Top);
    WriteInteger('PEDump', 'Width', Width);
    WriteInteger('PEDump', 'Height', Height);
    WriteInteger('PEDump', 'Numbers', Integer(NumberType));
    WriteString('PEDump', 'BinPath', ExtractFilePath(dlgOpen.FileName));
  finally
    Free;
  end;
end;

procedure TfmEXEView.LoadSettings;
begin
  // do not localize any of the below lines
  //! StH: Add StandAlone conditional?
  with TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts') do
  try
    Left := ReadInteger('PEDump', 'Left', Left);
    Top := ReadInteger('PEDump', 'Top', Top);
    Width := ReadInteger('PEDump', 'Width', Width);
    Height := ReadInteger('PEDump', 'Height', Height);
    NumberType := TNumberType(ReadInteger('PEDump', 'Numbers', Ord(ntHex)));
    dlgOpen.InitialDir := ReadString('PEDump', 'BinPath', '');
  finally
    Free;
  end;
end;

procedure TfmEXEView.sbPrintClick(Sender: TObject);
{$IFNDEF ACEREPORTER}
begin
  MessageDlg( SNoAceReporterHint, mtInformation, [mbOK], 0);
end;
{$ELSE}
resourcestring
  SFile = 'File: ';
  SMsDosHeader = 'MS-DOS Header';
  SPeHeader = 'PE Header';
  SPeOptionalHeader = 'PE Optional Header';
  SImports = 'Imports';
  SFunction = 'Function';
  SOrdinal = 'Ordinal';
  SExports = 'Exports';
var
  AOut: TAceOutPut;
  FontHeight: Integer;

  procedure PrintReport;
  var
    Row: Integer;
    ImpExp: TImportExport;
    i, j: Integer;

    procedure CheckHeader;
    begin
      if Row + ((FontHeight + 1) * 3) > AOut.AceFile.PageHeight then
      begin
        AOut.NewPage;
        Row := PR_OffsetY;
      end;
    end;

    procedure PrintHeader(LB: THeaderListBox; Header: string);
    var
      t: Integer;
    begin
      with AOut.AceCanvas do
      begin
        Row := Row + 10;
        Font.Style := [fsBold];
        TextOut(PR_OffsetX, Row, Header);
        Row := Row + FontHeight + 4;
        Font.Style := [];
        for t := 0 to LB.Items.Count - 1 do
        begin
          TextOut(PR_OffSetX + 20, Row, LB.Values[0, t]);
          TextOut(PR_OffSetX + 300, Row, ':');
          TextOut(PR_OffSetX + 320, Row, LB.Values[1, t]);
          Row := Row + FontHeight + 4;
          CheckHeader;
        end;
      end;
    end;

  begin
    Row := PR_OffsetY;
    { File Header }
    with AOut.AceCanvas do
    begin
      Font.Name := 'Arial';  // do not localize
      Font.Size := 12;
      Font.Style := [fsBold];
      TextOut(PR_OffsetX, Row, SFile + Filename);
      FontHeight := GetTextHeight(AOut.ACECanvas, 'W');
      Row := Row + FontHeight + 4;
    end;
    { MS-DOS Header }
    PrintHeader(hlMSDos, SMsDosHeader);
    { PE Header }
    PrintHeader(hlPEHeader, SPeHeader);
    { PE Optional Header }
    PrintHeader(hlPEOptionalHeader, SPeOptionalHeader);
    { Imports }
    with AOut.AceCanvas do
    begin
      Font.Style := [fsBold];
      Row := Row + 10;
      TextOut(PR_OffsetX, Row, SImports);
      Row := Row + FontHeight + 4;
      Font.Style := [];
      for j := 0 to hlImports.Items.Count - 1 do
      begin
        Font.Style := [fsUnderLine];
        TextOut(PR_OffsetX + 20, Row, hlImports.Items[j]);
        TextOut(PR_OffsetX + 120, Row, SFunction);
        TextOut(PR_OffsetX + 550, Row, SOrdinal);
        Row := Row + FontHeight + 4;
        Font.Style := [];
        ImpExp := TImportExport(PEInfo.ImportList.Objects[j]);
        for i := 0 to ImpExp.count - 1 do
        begin
          TextRect( Bounds(PR_OffSetX + 120, Row, PR_OffSetX + 385, FontHeight + 4),
                    PR_OffSetX + 120, Row, ImpExp.Items[i].FunctionName);
          TextOut(PR_OffSetX + 550, Row, IntToStr(ImpExp.Items[i].Ordinal));
          Row := Row + FontHeight + 4;
          CheckHeader;
        end;
      end;
    end;
    { Exports }
    with AOut.AceCanvas do
    begin
      Font.Style := [fsBold];
      Row := Row + 10;
      TextOut(PR_OffsetX, Row, SExports);
      Font.Style := [];
      Row := Row + FontHeight + 4;
      for i := 0 to hlExportFunctions.Items.Count - 1 do
      begin
        TextRect( Bounds(PR_OffSetX + 20, Row, PR_OffSetX + 335, FontHeight + 4),
                  PR_OffSetX + 20, Row, hlExportFunctions.Values[0, i]);
        TextOut(PR_OffSetX + 480, Row, hlExportFunctions.Values[1, i]);
        TextOut(PR_OffSetX + 580, Row, hlExportFunctions.Values[2, i]);
        Row := Row + FontHeight + 4;
        CheckHeader;
      end;
    end;
  end;

resourcestring
  SPeInfoFor = 'PE Information for ';
begin
  if PEInfo = nil then
    Exit;
  try
    Screen.Cursor := crHourglass;
    AOut := TAceOutPut.Create;
    try
      AOut.Destination := adAceFile;
      AOut.Description := SPeInfoFor + ExtractFileName(dlgOpen.FileName);
      AOut.BeginDoc;
      PrintReport;
      AOut.EndDoc;
      Screen.Cursor := crDefault; // this is OK
      PreviewReport(AOut.AceFile);
      { Set output.AceFile to nil so AcePreview file is not freed
        with output.AceFile }
      AOut.AceFile := nil;
    finally
      AOut.Free;
      Screen.Cursor := crDefault;
    end;
  except
    on E: Exception do
      ShowExceptionErrorMessage(E);
  end;
end;

procedure TfmEXEView.PreviewReport(AceFile: TAceFile);
begin
  with TfmPreview.Create(nil) do
  try
    AcePreview.LoadFromAceFile(AceFile);
    ShowModal;
  finally
    Free;
  end;
end;
{$ENDIF ACEREPORTER}

procedure TfmEXEView.About1Click(Sender: TObject);
begin
  ShowGXAboutForm;
end;

procedure TfmEXEView.Help2Click(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 16);
end;

procedure TfmEXEView.sbHelpClick(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 16);
end;

procedure TfmEXEView.pcMainChange(Sender: TObject);
begin
  // VCL 4.0 Bug workaround - columns in header list not visible (extent = 0)
  {$IFDEF GX_VER120_up}
  pcMain.ActivePage.Realign;
  FormResize(Self);
  {$ENDIF GX_VER120_up}
end;

//************************ TPEExpert ********************

constructor TPEExpert.Create;
begin
  inherited Create;
  PEExpert := Self;
  ShortCut := 0;
  HasConfigOptions := False;
  HasMenuItem := True;
end;

destructor TPEExpert.Destroy;
begin
  PEExpert := nil;
  inherited Destroy;
end;

procedure TPEExpert.SetActive(New: Boolean);
begin
  if New <> Active then
  begin
    inherited SetActive(New);
    if New then
      IdeDockManager.RegisterDockableForm(TfmEXEView, fmEXEView, 'fmEXEView')
    else
    begin
      fmEXEView.Free;
      // P: Already done in fmEXEView.FormDestroy
      //fmEXEView := nil;
      IdeDockManager.UnRegisterDockableForm(fmEXEView, 'fmEXEView');
    end;
  end;
end;

function TPEExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = 'P&E Information';
begin
  Result := SMenuCaption;
end;

function TPEExpert.GetMenuName: string;
begin
  Result := 'GX_PE';
end;

function TPEExpert.GetMenuMask: string;
begin
  Result := '';
end;

function TPEExpert.GetName: string;
begin
  Result := 'PE_DUMP';
end;

function TPEExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'PE Information';
begin
  Result := SDisplayName;
end;

procedure TPEExpert.Click(Sender: TObject);
begin
  if fmExeView = nil then
    fmExeView := TfmExeView.Create(nil);
  IdeDockManager.ShowForm(fmExeView);
end;

function TPEExpert.IconFileName: string;
begin
  Result := 'PE';
end;

initialization
  RegisterGX_Expert(TPEExpert);
end.

 
