unit GX_ExpMgr;

{$I GX_CondDefine.inc}

interface

//! StH: This unit has been completely prepared for localization
//! StH: This unit is fully compatible with C++ Builder

uses
  Windows, Messages, SysUtils, Classes, Graphics,
{$IFDEF GX_VER120_up}
  ImgList,
{$ENDIF GX_VER120_up}
  Controls, Forms, Dialogs,
  Menus, Buttons, HeadList, ExtCtrls, StdCtrls, Registry,
  GX_uExperts, ToolIntf, ExptIntf, ComCtrls;

type
  TExpertManagerExpert = class;

  TfmExperts = class(TForm)
    pnToolBar: TPanel;
    lbExperts: THeaderListbox;
    sbEnabled: TSpeedButton;
    sbDisabled: TSpeedButton;
    sbAdd: TSpeedButton;
    sbRemove: TSpeedButton;
    mnuMain: TMainMenu;
    File1: TMenuItem;
    AddExpert1: TMenuItem;
    RemoveExpert1: TMenuItem;
    Experts1: TMenuItem;
    EnabledExpert1: TMenuItem;
    DisableExpert1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    ilImages: TImageList;
    dlgAddExpert: TOpenDialog;
    sbHelp: TSpeedButton;
    Help2: TMenuItem;
    Contents1: TMenuItem;
    N2: TMenuItem;
    pmItems: TPopupMenu;
    EnablePopup: TMenuItem;
    DisablePopup: TMenuItem;
    N3: TMenuItem;
    RemovePopup: TMenuItem;
    StatusBar: TStatusBar;
    procedure About1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lbExpertsDrawCell(Sender: TObject; Col, Index: Integer;
      var Rect: TRect; var Text: string; State: TOwnerDrawState;
      var DoDefault: Boolean);
    procedure FormResize(Sender: TObject);
    procedure sbEnabledClick(Sender: TObject);
    procedure sbDisabledClick(Sender: TObject);
    procedure lbExpertsClick(Sender: TObject);
    procedure sbAddClick(Sender: TObject);
    procedure sbRemoveClick(Sender: TObject);
    procedure sbHelpClick(Sender: TObject);
    procedure Contents1Click(Sender: TObject);
    procedure pmItemsPopup(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FExpertManager: TExpertManagerExpert;
    procedure UpdateButtonState;
    procedure UpdateControlsState;
    procedure RefreshExpertState;
    function ConfirmIfGExperts(Index: Integer): Boolean;
    procedure LoadSettings;
    procedure SaveSettings;
  public
    constructor Create(AOwner: TComponent); override;
    constructor CreateWithManager(AOwner: TComponent; Manager: TExpertManagerExpert);
    destructor Destroy; override;
  end;

  TExpertManagerExpert = class(TGX_Expert)
  private
    FExpertsAndState: TStrings;
    FExpertFilenames: TStrings;
    procedure Load;
    procedure SaveDisabledExperts;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    function IconFileName: string; override;
    procedure Click(Sender: TObject); override;
    function AddExpert(const FileName: string): Boolean;
    procedure EnableExpert(Index: Integer);
    function DisableExpert(Index: Integer): Boolean;
    function RemoveExpert(Index: Integer): Boolean;

    procedure LoadSettings; override;
    procedure SaveSettings; override;

    property ExpertsAndState: TStrings read FExpertsAndState;
  end;

procedure ShowExpertManager; {$IFNDEF GX_BCB} export; {$ENDIF GX_BCB}
procedure CloseExpertManager; {$IFNDEF GX_BCB} export; {$ENDIF GX_BCB}

implementation

{$R *.DFM}
{$R BExpMgr.RES}

uses
{$IFOPT D+}
  GX_DbugIntf,
{$ENDIF D+}
  GX_GExperts,
{$IFDEF GX_VER120_up}
  ToolsApi,
{$ENDIF GX_VER120_up}
  GX_uConfigurationInfo;

const
  // do not localize!
  UnknownExpertName = 'Unknown Expert';
  UnknownExpertFileName = 'Unknown';
  UnknownExpert = UnknownExpertName + ',' + UnknownExpertFileName + ',0';

type
  TGxExpertState = ( gesCurrentlyDisabled, gesCurrentlyEnabled,
                     gesNextTimeDisabled, gesNextTimeEnabled,
                     gesInvalid);

constructor TfmExperts.Create(AOwner: TComponent);
begin
  Assert(False); // Call constructor CreateWithManger
end;

constructor TfmExperts.CreateWithManager(AOwner: TComponent; Manager: TExpertManagerExpert);
begin
  FExpertManager := Manager;
  inherited Create(AOwner);
end;

destructor TfmExperts.Destroy;
begin
  FExpertManager := nil;
  SaveSettings;

  inherited Destroy;
end;

procedure TfmExperts.About1Click(Sender: TObject);
begin
  ShowGXAboutForm;
end;

procedure TfmExperts.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfmExperts.RefreshExpertState;
var
  i: Integer;
  TempString: string;
begin
  lbExperts.Items.BeginUpdate;
  try
    lbExperts.Clear;

    with FExpertManager.ExpertsAndState do
      for i := 0 to Count-1 do
      begin
        TempString := Strings[i] + #9 + FExpertManager.FExpertFilenames[i];
        TempString := TempString + #9 + IntToStr(Integer(TGxExpertState(Objects[i])));
        lbExperts.Items.Add(TempString);
      end;
  finally
    lbExperts.Items.EndUpdate;
  end;
end;

procedure TfmExperts.FormCreate(Sender: TObject);
resourcestring
  SCouldNotLoadImages = 'Could not load expert DLL state images.';
begin
  ilImages.ResourceLoad(rtBitmap, 'EXPMGR_STATE', clOlive); // do not localize
  lbExperts.ListBox.ItemHeight := 18;
  LoadSettings;

  if ilImages.Count = 0 then
    MessageDlg(SCouldNotLoadImages, mtInformation, [mbOK], 0);

  RefreshExpertState;
end;

procedure TfmExperts.lbExpertsDrawCell(Sender: TObject; Col,
  Index: Integer; var Rect: TRect; var Text: string;
  State: TOwnerDrawState; var DoDefault: Boolean);

  function ExpertStateTextToImageListIndex(const ExpertStateText: string): Integer;
  var
    OldExceptionClass: TClass;
  begin
    OldExceptionClass := ExceptionClass;
    ExceptionClass := nil;
    try
      try
        case TGxExpertState(StrToInt(ExpertStateText)) of
          gesCurrentlyDisabled: Result := 2;
          gesCurrentlyEnabled:  Result := 0;
          gesNextTimeDisabled:  Result := 3;
          gesNextTimeEnabled:   Result := 1;
        else
          // gesInvalid
          Result := -1;
        end;
      except
        on E: Exception do
        begin
          Result := -1;
        end;
      end;
    finally
      ExceptionClass := OldExceptionClass;
    end;
  end;

var
  ImageListIndex: Integer;
begin
  if Col = 2 then
  begin
    DoDefault := False;

    if odSelected in State then
      Brush.Color := clHighLight
    else
      Brush.Color := clWindow;

    ImageListIndex := ExpertStateTextToImageListIndex(Text);
    if ImageListIndex = -1 then Exit;
    ilImages.Draw(lbExperts.ListBox.Canvas, Rect.Left + 1, Rect.Top + 1, ImageListIndex);
  end
  else
    DoDefault := True;
end;

procedure TfmExperts.FormResize(Sender: TObject);
begin
  with lbExperts do
    Sections[1].Width := Width - Sections[0].Width - Sections[2].Width;
end;

function TfmExperts.ConfirmIfGExperts(Index: Integer): Boolean;
resourcestring
  SConfirmGExpertsDisable = 'You are about to disable or remove the GExperts DLL '+
                            'that contains the expert you are currently using.'#13#13+
                            'If you proceed, you will not be able to use this Expert Manager '+
                            'the next time the IDE is started.'#13#13+
                            'Do you want to proceed?';
const
  MaxModuleLen = 32*1024;
var
  ModuleName: string;
begin
  Result := True;

  SetLength(ModuleName, MaxModuleLen);
  if GetModuleFileName(HINSTANCE, PChar(ModuleName), MaxModuleLen) <> 0 then
  begin
    SetLength(ModuleName, StrLen(PChar(ModuleName)));
    if CompareText(FExpertManager.FExpertFilenames[Index], ModuleName) = 0 then
    begin
      Result := (MessageDlg(SConfirmGExpertsDisable, mtConfirmation, [mbYes, mbNo], 0) = mrYes);
    end;
  end;
end;

procedure TfmExperts.sbEnabledClick(Sender: TObject);
var
  GxExpertState: TGxExpertState;
  n: Integer;
begin
  n := lbExperts.ItemIndex;
  if n < 0 then Exit;

  GxExpertState := TGxExpertState(lbExperts.Values[2, n]);
  if (GxExpertState in [gesCurrentlyEnabled, gesNextTimeEnabled]) then
    Exit;

  FExpertManager.EnableExpert(n);
  UpdateControlsState;
end;

procedure TfmExperts.sbDisabledClick(Sender: TObject);
var
  GxExpertState: TGxExpertState;
  n: Integer;
resourcestring
  SCouldNotDisable = 'The expert could not be disabled.';
begin
  n := lbExperts.ItemIndex;
  if n < 0 then Exit;

  GxExpertState := TGxExpertState(lbExperts.Values[2, n]);
  if (GxExpertState in [gesCurrentlyDisabled, gesNextTimeDisabled]) then
    Exit;

  if not ConfirmIfGExperts(n) then
    Exit;

  if FExpertManager.DisableExpert(n) then
    UpdateControlsState
  else
    MessageDlg(SCouldNotDisable, mtError, [mbOK], 0);
end;

procedure TfmExperts.lbExpertsClick(Sender: TObject);
begin
  UpdateButtonState;
end;

procedure TfmExperts.UpdateButtonState;

    function GetDescriptiveState(State: TGxExpertState): string;
    resourcestring
      SCurrentlyDisabled = 'Expert DLL is currently disabled and not loaded';
      SCurrentlyEnabled  = 'Expert DLL is currently enabled and active';
      SNextTimeEnabled   = 'Expert DLL will be active next time IDE is started';
      SNextTimeDisabled  = 'Expert DLL will be inactive next time IDE is started';
    begin
      case State of
        gesCurrentlyDisabled: Result := SCurrentlyDisabled;
        gesCurrentlyEnabled:  Result := SCurrentlyEnabled;
        gesNextTimeDisabled:  Result := SNextTimeDisabled;
        gesNextTimeEnabled:   Result := SNextTimeEnabled;
      else
        Result := '';
      end;
    end;

var
  CanRemoveExpert: Boolean;
  CanEnableExpert: Boolean;
  CanDisableExpert: Boolean;
  GxExpertState: TGxExpertState;
begin
  if lbExperts.ItemIndex < 0 then
  begin
    CanRemoveExpert := False;
    CanEnableExpert := False;
    CanDisableExpert := False;

    StatusBar.SimpleText := '';
  end
  else
  begin
    CanRemoveExpert := True;

    GxExpertState := TGxExpertState(lbExperts.Values[2, lbExperts.ItemIndex]);

    CanEnableExpert := (GxExpertState in [gesCurrentlyDisabled, gesNextTimeDisabled]);
    CanDisableExpert := (GxExpertState in [gesCurrentlyEnabled, gesNextTimeEnabled]);

    StatusBar.SimpleText := GetDescriptiveState(GxExpertState);
  end;

  sbRemove.Enabled := CanRemoveExpert;
  RemoveExpert1.Enabled := CanRemoveExpert;
  RemovePopup.Enabled := CanRemoveExpert;

  sbEnabled.Enabled := CanEnableExpert;
  EnabledExpert1.Enabled := CanEnableExpert;
  EnablePopup.Enabled := CanEnableExpert;

  sbDisabled.Enabled := CanDisableExpert;
  DisableExpert1.Enabled := CanDisableExpert;
  DisablePopup.Enabled := CanDisableExpert;
end;

procedure TfmExperts.UpdateControlsState;
var
  CurrentItemIndex: Integer;
begin
  CurrentItemIndex := lbExperts.ItemIndex;
  try
    RefreshExpertState;
  finally
    if CurrentItemIndex >= 0 then
      lbExperts.ItemIndex := CurrentItemIndex;
  end;

  UpdateButtonState;
end;

procedure TfmExperts.sbAddClick(Sender: TObject);
resourcestring
  SCouldNotAddExpert = 'The selected file could not be added as an expert.'#13#13+
                       'The module you chose probably is not a valid expert DLL or is '+
                       'already loaded as an expert DLL.';
begin
  if dlgAddExpert.Execute then
  begin
    if FExpertManager.AddExpert(dlgAddExpert.Filename) then
      UpdateControlsState
    else
      MessageDlg(SCouldNotAddExpert, mtError, [mbOK], 0);
  end;
end;

procedure TfmExperts.sbRemoveClick(Sender: TObject);
resourcestring
  SConfirmRemoval = 'Are you sure you want to remove the selected expert?';
  SRemovalFailed = 'Could not remove the expert (registry error?).';
var
  n: Integer;
begin
  n := lbExperts.ItemIndex;
  if n < 0 then Exit;

  if MessageDlg(SConfirmRemoval, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  begin
    if not ConfirmIfGExperts(n) then
      Exit;

    if FExpertManager.RemoveExpert(n) then
      UpdateControlsState
    else
      MessageDlg(SRemovalFailed, mtError, [mbOK], 0);
  end;
end;

procedure TfmExperts.sbHelpClick(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 7);
end;

procedure TfmExperts.Contents1Click(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTENTS, 0);
end;

{ TExpertManagerExpert }

constructor TExpertManagerExpert.Create;
begin
  inherited Create;

  FExpertsAndState := TStringList.Create;
  FExpertFilenames := TStringList.Create;
  HasConfigOptions := False;
  HasMenuItem := True;
end;

destructor TExpertManagerExpert.Destroy;
begin
  SaveSettings;

  FExpertsAndState.Free;
  FExpertsAndState := nil;

  FExpertFilenames.Free;
  FExpertFilenames := nil;

  inherited Destroy;
end;

procedure TExpertManagerExpert.LoadSettings;
begin
  inherited LoadSettings;

  Load;
end;

procedure TExpertManagerExpert.SaveSettings;
begin
  inherited SaveSettings;
  if Assigned(FExpertFilenames) and Assigned(FExpertsAndState) then
    SaveDisabledExperts;
end;

function TExpertManagerExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = '&Expert Manager...';
begin
  Result := SMenuCaption;
end;

function TExpertManagerExpert.GetMenuName: string;
begin
  Result := 'GX_ExpMgr';
end;

function TExpertManagerExpert.GetMenuMask: string;
begin
  Result := '';
end;

function TExpertManagerExpert.GetName: string;
begin
  Result := 'Expert_Manager';
end;

function TExpertManagerExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'Expert Manager';
begin
  Result := SDisplayName;
end;

procedure TExpertManagerExpert.Click(Sender: TObject);
begin
  with TfmExperts.CreateWithManager(nil, Self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

function TExpertManagerExpert.RemoveExpert(Index: Integer): Boolean;
begin
  Result := False;

  if TGxExpertState(FExpertsAndState.Objects[Index]) in [gesCurrentlyDisabled, gesNextTimeDisabled] then
    Result := True
  else
    with TRegistry.Create do
    try
      if OpenKey(ConfigInfo.RegKey + '\Experts', False) then  // do not localize
      begin
        Result := DeleteValue(FExpertsAndState.Strings[Index]);
        CloseKey;
      end;
    finally
      Free;
    end;

  if Result then
  begin
    FExpertsAndState.Delete(Index);
    FExpertFilenames.Delete(Index);
  end;
end;

function TExpertManagerExpert.AddExpert(const FileName: string): Boolean;
var
  ExpertName: string;
  DllHandle: THandle;
  i: Integer;
  RegIni: TRegIniFile;
begin
  {$IFOPT D+} SendDebug('Adding Expert: '+FileName); {$ENDIF}
  Result := True;
  // Make sure that this particular expert is not already loaded.
  for i := 0 to FExpertFilenames.Count-1 do
  begin
    if CompareText(FExpertFilenames[i], FileName) = 0 then
    begin
      Result := False;
      Exit;
    end;
  end;

{$DEFINE LoadAndCheckEntryPoint}
{$IFDEF LoadAndCheckEntryPoint}

  // Check that the DLL *really* is a valid expert or wizard DLL (Supported on NT only)
  if Win32MajorVersion = VER_PLATFORM_WIN32_NT then
  begin
    DllHandle := LoadLibraryEx(PChar(FileName), 0, DONT_RESOLVE_DLL_REFERENCES {NT only!});
    try
      Result := (GetProcAddress(DllHandle, ExpertEntryPoint) <> nil);
    {$IFDEF GX_VER120_up}
      // As of Delphi 4+ and C++Builder 4+ experts may be "wizards", too.
      if not Result then
        Result := (GetProcAddress(DllHandle, WizardEntryPoint) <> nil);
    {$ENDIF GX_VER120_up}
    finally
      FreeLibrary(DllHandle);
    end;
  end; {NT Check}

  if not Result then
    Exit;
{$ENDIF LoadAndCheckEntryPoint}

  ExpertName := ChangeFileExt(ExtractFileName(FileName), '');

  FExpertsAndState.AddObject(ExpertName, TObject(gesNextTimeEnabled));
  FExpertFilenames.Add(FileName);

  RegIni := TRegIniFile.Create(ConfigInfo.RegKey);
  try
    RegIni.WriteString('Experts', ExpertName, FileName);  // do not localize
  finally
    RegIni.Free;
  end;
end;

procedure TExpertManagerExpert.EnableExpert(Index: Integer);
var
  GxExpertState: TGxExpertState;
begin
  GxExpertState := TGxExpertState(FExpertsAndState.Objects[Index]);

  if GxExpertState = gesNextTimeDisabled then
    FExpertsAndState.Objects[Index] := TObject(gesCurrentlyEnabled)
  else
    FExpertsAndState.Objects[Index] := TObject(gesNextTimeEnabled);

  with TRegIniFile.Create(ConfigInfo.RegKey) do
  try
    WriteString('Experts', FExpertsAndState[Index], FExpertFilenames[Index]);  // do not localize
  finally
    Free;
  end;
end;

function TExpertManagerExpert.DisableExpert(Index: Integer): Boolean;
var
  GxExpertState: TGxExpertState;
begin
  Result := False;

  with TRegistry.Create do
  try
    if OpenKey(ConfigInfo.RegKey + '\Experts', False) then  // do not localize
    begin
      // If the value exists, make sure that it was deleted -> error, otherwise
      if ValueExists(FExpertsAndState[Index]) then
        Result := DeleteValue(FExpertsAndState[Index])
      else
        Result := True;
      CloseKey;
    end;
  finally
    Free;
  end;

  GxExpertState := TGxExpertState(FExpertsAndState.Objects[Index]);

  if GxExpertState = gesNextTimeEnabled then
    FExpertsAndState.Objects[Index] := TObject(gesCurrentlyDisabled)
  else
    FExpertsAndState.Objects[Index] := TObject(gesNextTimeDisabled);
end;

procedure TExpertManagerExpert.SaveDisabledExperts;
var
  RegIni: TRegIniFile;
  i: Integer;
  Count: Integer;
  ExpertString: string;
begin
  // do not localize any of the following lines
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    // Note that we do not have to clear this section,
    // since the "count" of experts is written; this
    // prevents loading of "too many" experts next time round
    Count := 0;
    //{$IFOPT D+} SendDebug('Total experts: '+IntToStr(FExpertsAndState.Count)); {$ENDIF}
    for i := 0 to FExpertsAndState.Count - 1 do
    begin
      if TGxExpertState(FExpertsAndState.Objects[i]) in [gesCurrentlyDisabled, gesNextTimeDisabled] then
      begin
        //{$IFOPT D+} SendDebug('Saving disabled expert: '+FExpertsAndState[i]); {$ENDIF}
        //! StH: This comma below is bad. Very bad. A valid file name may contain
        //! commas... It's still a comma, simply because of backwards compatibility,
        //! but I'd prefer to be using the "|" as a separator.
        //! Note that we are reading with a comma separator below, too.
        //! Two options:
        //!    * Keep it this way and take the risk that reading breaks
        //!      when someone installs an Expert or ExpertName (!) with a
        //!      comma in it
        //!    * Break existing settings (prompt for a one-time conversion process?)
        ExpertString := FExpertsAndState[i] + ',' + FExpertFilenames[i];

        RegIni.WriteString('Expert Manager', Format('Expert%d', [Count]), ExpertString);
        Inc(Count);
      end;
    end;
    RegIni.WriteInteger('Expert Manager', 'Count', Count);
  finally
    RegIni.Free;
  end;
end;

procedure TExpertManagerExpert.Load;
const
  PartSeparator = ',';

  function GetExpertName(const s: string): string;
  var
    CommaPos: Integer;
  begin
    Result := '';
    CommaPos := Pos(PartSeparator, s);
    if CommaPos <> 0 then
      Result := Copy(s, 1, CommaPos - 1);
  end;

  function IsDuplicateExpert(const ExpertValue: string): Boolean;
  var
    i: Integer;
  begin
    Result := False;
    for i := 0 to FExpertsAndState.Count - 1 do
      if CompareText(GetExpertName(ExpertValue), FExpertsAndState[i]) = 0 then
      begin
        Result := True;
        Break;
      end;
  end;

  procedure AddDisabledExpert(Value: string);
  var
    CommaPos: Integer;
  begin
    if not IsDuplicateExpert(Value) then
    begin
      FExpertsAndState.AddObject(GetExpertName(Value), TObject(gesCurrentlyDisabled));

      // Delete everything up to the file name
      CommaPos := Pos(PartSeparator, Value);
      Assert(CommaPos > 0);
      Delete(Value, 1, CommaPos);

      // Delete everything behind that file name (we might store this ',0')
      CommaPos := Pos(PartSeparator, Value);
      if CommaPos > 0 then
        Delete(Value, CommaPos, Length(Value));

      // Finally add the extracted file name to our list of expert DLLs
      FExpertFilenames.Add(Value);
    end;
  end;

var
  RegIni: TRegIniFile;
  i: Integer;
  ValueName: string;
  FileNameValue: string;
begin
  FExpertsAndState.Clear;
  FExpertFilenames.Clear;

  // Load Experts from IDE registry section }
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey);
  try
    RegIni.ReadSectionValues('Experts', FExpertsAndState);  // do not localize
  finally
    RegIni.Free;
  end;

  for i := 0 to FExpertsAndState.Count-1 do
  begin
    ValueName := FExpertsAndState.Names[i];
    FileNameValue := FExpertsAndState.Values[ValueName];
    if FileNameValue <> '' then
    begin
      FExpertsAndState.Strings[i] := ValueName;
      FExpertsAndState.Objects[i] := TObject(gesCurrentlyEnabled);
    end
    else
    begin
      FileNameValue := UnknownExpertFileName;
      FExpertsAndState.Strings[i] := UnknownExpertName;
      FExpertsAndState.Objects[i] := TObject(gesCurrentlyEnabled);
    end;
    FExpertFilenames.Add(FileNameValue);
  end;

  // Now load disabled experts from our own registry location
  // do not localize any of the following lines
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    for i := 0 to RegIni.ReadInteger('Expert Manager', 'Count', 0) - 1 do
    begin
      FileNameValue := RegIni.ReadString('Expert Manager', Format('Expert%d', [i]), UnknownExpert);
      if FileNameValue <> UnknownExpert then
        AddDisabledExpert(FileNameValue);
    end;
  finally
    RegIni.Free;
  end;
end;

function TExpertManagerExpert.IconFileName: string;
begin
  Result := 'ExpMgr';
end;

// *** Code used by ExpMgr executable - DLL exports

var
  fmExperts: TfmExperts = nil;
  ExpMgr: TExpertManagerExpert = nil;

procedure ShowExpertManager; {$IFNDEF GX_BCB} export; {$ENDIF GX_BCB}
begin
  {$IFOPT D+}SendDebug('Showing expert manager'); {$ENDIF}
  ExpMgr := TExpertManagerExpert.Create;
  {$IFOPT D+}SendDebug('Loading expert data'); {$ENDIF}
  ExpMgr.Load;

  {$IFOPT D+}SendDebug('Create Window'); {$ENDIF}
  fmExperts := TfmExperts.CreateWithManager(nil, ExpMgr);
  {$IFOPT D+}SendDebug('Show Window'); {$ENDIF}
  fmExperts.UpdateControlsState;
  fmExperts.ShowModal;
  // This stuff is freed in CloseExpertManager (if it was ever called!)
end;

procedure CloseExpertManager; {$IFNDEF GX_BCB} export; {$ENDIF GX_BCB}
begin
  {$IFOPT D+}SendDebug('Closing expert manager'); {$ENDIF}
  fmExperts.Free;
  fmExperts := nil;

  ExpMgr.Free;
  ExpMgr := nil;
end;

procedure TfmExperts.pmItemsPopup(Sender: TObject);
begin
  if lbExperts.ItemIndex < 0 then
    Abort;
end;

procedure TfmExperts.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if FExpertManager <> nil then
    FExpertManager.SaveDisabledExperts;
end;

procedure TfmExperts.LoadSettings;
begin
  // Load window state
end;

procedure TfmExperts.SaveSettings;
begin
  // Save window state
end;

initialization
  RegisterGX_Expert(TExpertManagerExpert);
end.

