unit GX_Procs;

{$I GX_CondDefine.inc}

//! StH: This unit has been completely prepared for localization
//! StH: This unit is fully compatible with C++ Builder

interface

{$IFDEF VER125}
{$D-} //! StH: This works around a bug in C++ Builder 4.0
{$ENDIF VER125}

{$IFDEF VER110}
  {$D-} //! StH: Work around a compiler bug in C++ Builder 3.0
{$ENDIF VER110}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, ToolIntf, ExptIntf, EditIntf, Grids,
  Outline, HeadList, Buttons, GX_EditRead, Registry,
  {$IFDEF GX_VER120_up}
  ImgList,
  {$ENDIF GX_VER120_up}
  GX_uExperts, Menus;

type
  TProcInfo = class(TObject)
  public
    LineNo: Integer;
    Name: string;
  end;

type
  TfmProcs = class(TForm)
    ilImages: TImageList;
    pnHolder: TPanel;
    hlbProcs: THeaderListbox;
    StatusBar: TStatusBar;
    pnToolBar: TPanel;
    lblSearch: TLabel;
    laSearch: TLabel;
    pnBtnHolder: TPanel;
    sbGoto: TSpeedButton;
    sbHelp: TSpeedButton;
    sbStart: TSpeedButton;
    sbAny: TSpeedButton;
    sbFont: TSpeedButton;
    dlgProcFont: TFontDialog;
    sbCopy: TSpeedButton;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnGotoClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure hlbProcsDrawCell(Sender: TObject; Col, Index: Integer;
      var Rect: TRect; var Text: string; State: TOwnerDrawState;
      var DoDefault: Boolean);
    procedure hlbProcsClick(Sender: TObject);
    procedure sbHelpClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormResize(Sender: TObject);
    procedure sbStartClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure hlbProcsSectionClick(Sender: THeaderListbox;
      Section: THeaderSection);
    procedure sbFontClick(Sender: TObject);
    procedure sbCopyClick(Sender: TObject);
  private
    { Private declarations }
    ProcCount: Integer;
    Loaded: Boolean;
    EditReader: TEditReader;
    Sort: Integer;
    FNoComments: Boolean;
    procedure QuickSort(L, R: Integer);
    function SetIndex(ProcName: string): Integer;
    procedure LoadProcs;
    procedure FillListBox;
    function GetProcInfo(st: string): TProcInfo;
    function GetMethodName(const ProcName: string): string;
  private
    ProcList: TStringList;
    NoUpdate: Boolean;
    GotoLine: Integer;
    FileName: string;
  public
    { Public declarations }
    procedure AddProcedure(const ProcedureName, ProcedureType: string; LineNo: Integer);
    procedure SaveSettings;
    procedure LoadSettings;
    property NoComments: Boolean read FNoComments;
  end;

type
  TProcedureExpert = class(TGX_Expert)
  public
    constructor Create; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    function IconFileName: string; override;
    procedure Click(Sender: TObject); override;
  end;

implementation

{$R *.DFM}
{$R MFImgs.RES}

uses
  GX_UGenFunc, ClipBrd,
{$IFOPT D+}
  GX_DbugIntf, TypInfo,
{$ENDIF D+}
  mPasLex, GX_GExperts,
  GX_uConfigurationInfo;

{ Tell Delphi about exceptions. }

procedure TfmProcs.FormCreate(Sender: TObject);
resourcestring
  SCouldNotLoadImages = 'Could not load images';
var
  FileExt: string;
begin
  Sort := 1;
  FileName := ToolServices.GetCurrentFile;
  FileExt := ExtractUpperFileExt(FileName);
  try
    EditReader := TEditReader.Create(FileName);
    EditReader.BufSize := 30000;
    EditReader.NoTabs := True;
    if FileExt <> '.DPR' then // do not localize
      EditReader.NoComments := NoComments
    else
      EditReader.NoComments := True;
    { TODO -oStefan -c.98 DR1: Activate FreeFileData here }
    // EditReader.FreeFileData;
  except
    on E: Exception do
    begin
      //! FIXME?
      Close;
    end;
  end;

  Loaded := False;
  ProcList := TStringList.Create;
  NoUpdate := False;
  laSearch.Color := clWhite;
  ilImages.ResourceLoad(rtBitmap, 'MFIND_STRIP', clTeal); // do not localize
  if ilImages.Count = 0 then
    raise Exception.Create(SCouldNotLoadImages);
  ProcCount := 0;
  GotoLine := -1;
  hlbProcs.ListBox.ItemHeight := 18;
  CenterForm(Self);
  LoadSettings;
  LoadProcs;
  FillListBox;
  sbGoto.Enabled := (hlbProcs.Items.Count > 0);
  Loaded := True;
  FormResize(Self);
end;

procedure TfmProcs.LoadProcs;
var
  Parser: TmwPasLex;

  function MoveToImplementation: Boolean;
  begin
    if ExtractUpperFileExt(FileName) = '.DPR' then // do not localize
    begin
      Result := True;
      Exit;
    end;
    Result := False;
    while Parser.TokenID <> tkNull do
    begin
      if not (Parser.TokenID in [tkSpace, tkCRLF, tkSemicolon]) then
      //{$IFOPT D+}SendDebug('Type: '+GetEnumName(TypeInfo(TTokenKind), Integer(Parser.TokenID))+'  Token: '+Parser.Token);{$ENDIF}
      if Parser.TokenID = tkImplementation then
        Result := True;
      Parser.Next;
      if Result then Break;
    end;
  end;

  procedure FindProcs;

    function GetProperProcName(ProcType: TTokenKind; IsClass: Boolean): string;
    resourcestring
      SUnknown = 'Unknown';
    begin
      Result := SUnknown;
      if IsClass then
      begin
        if ProcType = tkFunction then
          Result := 'Class Func' // do not localize
        else
          if ProcType = tkProcedure then
            Result := 'Class Proc'; // do not localize
      end
      else
      begin
        case ProcType of
          // do not localize
          tkFunction: Result := 'Function';
          tkProcedure: Result := 'Procedure';
          tkConstructor: Result := 'Constructor';
          tkDestructor: Result := 'Destructor';
        end;
      end;
    end;

  var
    ProcLine: string;
    ProcType: TTokenKind;
    Line: Integer;
    ClassLast: Boolean;
    InParenthesis: Boolean;
    InTypeDeclaration: Boolean;
    FoundNonEmptyType : Boolean;
    IdentifierNeeded: Boolean;
  resourcestring
    SImplementationNotFound = 'implementation section not found (parser error?)';
  begin
    if not MoveToImplementation then
      raise Exception.Create(SImplementationNotFound);
    ClassLast := False;
    InParenthesis := False;
    InTypeDeclaration := False;
    FoundNonEmptyType := False;
    ProcList.Capacity := 200;
    ProcList.BeginUpdate;
    try
      while Parser.TokenID <> tkNull do
      begin
        if not InTypeDeclaration and
           (Parser.TokenID in [tkFunction, tkProcedure, tkConstructor, tkDestructor]) then
        begin
          IdentifierNeeded := True;
          ProcType := Parser.TokenID;
          Line := Parser.LineNumber;
          ProcLine := '';
          while not (Parser.TokenId in [tkNull]) do
          begin
            //{$IFOPT D+}SendDebug('Found Inner Token: '+ Parser.Token+ ' '+BTS(ClassLast));{$ENDIF}
            case Parser.TokenID of
              tkIdentifier, tkRegister:
                IdentifierNeeded := False;

              tkRoundOpen:
                begin
                  // Did we run into an identifier already?
                  // This prevents
                  //    AProcedure = procedure() of object
                  // from being recognised as a procedure
                  if IdentifierNeeded then
                    Break;
                  InParenthesis := True;
                end;

              tkRoundClose:
                InParenthesis := False;

            else
              // nothing
            end; // case

            if (not InParenthesis) and (Parser.TokenID = tkSemiColon) then
              Break;

            if not (Parser.TokenID in [tkCRLF, tkCRLFCo]) then
              ProcLine := ProcLine + Parser.Token;
            Parser.Next;
          end; // while
          if Parser.TokenID = tkSemicolon then
            ProcLine := ProcLine + ';';
          if ClassLast then
            ProcLine := 'class ' + ProcLine; // do not localize
          //{$IFOPT D+}SendDebug('FoundProc: ' + ProcLine);{$ENDIF}
          if not IdentifierNeeded then
            AddProcedure(ProcLine, GetProperProcName(ProcType, ClassLast), Line + 1);
        end;
        if (Parser.TokenID = tkClass) and Parser.IsClass then
        begin
          InTypeDeclaration := True;
          FoundNonEmptyType := False;
        end
        else 
        if InTypeDeclaration and
                (Parser.TokenID in [tkProcedure, tkFunction, tkProperty,
                                    tkPrivate, tkProtected, tkPublic, tkPublished]) then
        begin
          FoundNonEmptyType := True;
        end
        else 
        if InTypeDeclaration and
                ((Parser.TokenID = tkEnd) or
                 ((Parser.TokenID = tkSemiColon) and not FoundNonEmptyType)) then
        begin
          InTypeDeclaration := False;
        end;
        //{$IFOPT D+}SendDebug('Found Token: '+ Parser.Token+ ' '+BTS(ClassLast));{$ENDIF}
        ClassLast := (Parser.TokenID = tkClass);
        if ClassLast then
        begin
          Parser.NextNoJunk;
          //{$IFOPT D+}SendDebug('Found Class Token'+ ' '+BTS(ClassLast));{$ENDIF}
        end
        else
          Parser.Next;
      end;
    finally
      ProcList.EndUpdate;
    end;
  end;

resourcestring
  SParseStatistics = 'Procedures processed in %g seconds';
var
  ParseStartTime: DWORD;
  MemStream: TMemoryStream;
begin
  Parser := TmwPasLex.Create;
  try
    MemStream := TMemoryStream.Create;
    try
      // Since this edit reader is destroyed almost
      // almost immediately, do not call FreeFileData
      with TEditReader.Create(FileName) do
      try
        SaveToStream(MemStream);
      finally
        Free;
      end;
      {$IFOPT D+}SendDebug('Starting Parse'); {$ENDIF}
      Parser.Origin := MemStream.Memory;
      Caption := Caption + ' - ' + ExtractFileName(FileName);
      //Application.ProcessMessages;
      ParseStartTime := GetTickCount;
      Screen.Cursor := crHourGlass;
      try
        FindProcs;
        {$IFOPT D+}SendDebug('QuickSorting'); {$ENDIF}
        QuickSort(0, ProcList.Count - 1);
        ParseStartTime := GetTickCount - ParseStartTime;
      finally
        Screen.Cursor := crDefault;
      end;
      StatusBar.Panels[0].Text := Format(SParseStatistics, [ParseStartTime / 1000]);
      StatusBar.Panels[1].Text := Trim(IntToStr(hlbProcs.Items.Count));
    finally
      MemStream.Free;
    end;
  finally
    Parser.Free;
  end;
end;

procedure TfmProcs.FormDestroy(Sender: TObject);
var
  i: Integer;
begin
  try
    if ProcList <> nil then
    begin
      for i := 0 to ProcList.Count - 1 do
        ProcList.Objects[i].Free;
      ProcList.Free;
      ProcList := nil;
    end;
    EditReader.Free;
    EditReader := nil;
    {$IFOPT D+}SendDebug('Procedure Window Freed'); {$ENDIF}
  except
    on E: Exception do
      ShowExceptionErrorMessage(E);
  end;
end;

procedure TfmProcs.btnCloseClick(Sender: TObject);
begin
  ModalResult := mrCancel;
  Close;
end;

procedure TfmProcs.AddProcedure(const ProcedureName, ProcedureType: string; LineNo: Integer);

  function CleanWhiteSpace(Str: string): string;
  var
    i: Integer;
  begin
    Result := Trim(Str);
    for i := 1 to Length(Result) do
    begin
      if Result[i] = #9 then
        Result[i] := ' ';
    end;
    i := Pos('  ', Result);
    while i > 0 do
    begin
      Delete(Result, i, 1);
      i := Pos('  ', Result);
    end;
  end;

var
  TempStr: string;
  i: Integer;
  ProcedureInfo: TProcInfo;
begin
  // Some fixes from John Hansen <John_Hansen@tcsmgmt.com>
  ProcedureInfo := TProcInfo.Create;
  ProcedureInfo.LineNo := LineNo;
  ProcedureInfo.Name := CleanWhiteSpace(ProcedureName);
  TempStr := ProcedureInfo.Name;
  // Remove the class reserved word
  i := Pos('CLASS ', UpperCase(TempStr)); // do not localize
  if i = 1 then
    Delete(TempStr, 1, Length('CLASS ')); // do not localize
  // Remove 'function' or 'procedure'
  i := Pos(' ', TempStr);
  if i > 0 then
    TempStr := Copy(TempStr, i + 1, Length(TempStr))
  else
    TempStr := TempStr;
  // Remove the paramater list
  i := Pos('(', TempStr);
  if i > 0 then
    TempStr := Copy(TempStr, 1, i - 1);
  // Remove the function return type
  i := Pos(':', TempStr);
  if i > 0 then
    TempStr := Copy(TempStr, 1, i - 1);
  // Check for an implementation procedural type
  if Length(TempStr) = 0 then
  begin
    ProcedureInfo.Free;
    Exit;
  end;
  // Remove any trailing ';'
  if TempStr[Length(TempStr)] = ';' then
    Delete(TempStr, Length(TempStr), 1);
  ProcList.AddObject(#9 + Trim(TempStr) + #9 + ProcedureType + #9 + IntToStr(LineNo), ProcedureInfo);
end;

function TfmProcs.GetMethodName(const ProcName: string): string;
var
  CharPos: Integer;
  TempStr: string;
begin
  Result := ProcName;
  Delete(Result, 1, 1);

  CharPos := Pos(#9, Result);
  if CharPos <> 0 then
    Delete(Result, CharPos, Length(Result));

  CharPos := Pos(' ', Result);
  TempStr := Copy(Result, CharPos + 1, Length(Result));

  CharPos := Pos('.', TempStr);
  if CharPos = 0 then
    Result := TempStr
  else
    TempStr := Copy(TempStr, CharPos + 1, Length(TempStr));

  CharPos := Pos('(', TempStr);
  if CharPos = 0 then
    Result := TempStr
  else
    Result := Copy(TempStr, 1, CharPos - 1);

  Result := Trim(Result);
end;

function TfmProcs.SetIndex(ProcName: string): Integer;
begin
  ProcName := UpperCase(ProcName);
  if Pos('.', ProcName) <> 0 then
    Result := 0
  else
    Result := 1;
  if Pos('CONSTRUCTOR', ProcName) <> 0 then // do not localize
    Result := 2;
  if Pos('DESTRUCTOR', ProcName) <> 0 then // do not localize
    Result := 3;
end;

procedure TfmProcs.btnGotoClick(Sender: TObject);
var
  ProcInfo: TProcInfo;
begin
  if hlbProcs.ItemIndex <> -1 then
  begin
    ProcInfo := GetProcInfo(hlbProcs.Items[hlbprocs.ItemIndex]);
    if ProcInfo <> nil then
    begin
      Assert(EditReader <> nil);
      EditReader.GotoLine(ProcInfo.LineNo);
      EditReader.ShowSource;  //FIXME! This doesn't focus the editor

      { TODO -oStefan -c.98 DR1: Activate FreeFileData here }
      // EditReader.FreeFileData;

      Close;
    end;
  end;
end;

procedure TfmProcs.hlbProcsDrawCell(Sender: TObject; Col, Index: Integer;
  var Rect: TRect; var Text: string; State: TOwnerDrawState;
  var DoDefault: Boolean);
var
  p: Integer;
begin
  if Col = 0 then
  begin
    if odSelected in State then
      Brush.Color := clHighLight
    else
      Brush.Color := clWindow;
    p := SetIndex(hlbProcs.ListBox.Items[index]);
    if (p = 1) and (odSelected in State) then
      p := 4;
    ilImages.Draw(hlbProcs.ListBox.Canvas, Rect.Left + 1, Rect.Top + 1, p);
    DoDefault := false;
  end
  else
    DoDefault := True;
end;

procedure TfmProcs.hlbProcsClick(Sender: TObject);
var
  ProcInfo: TProcInfo;
begin
  ProcInfo := nil;
  if hlbprocs.ItemIndex > -1 then
    ProcInfo := GetProcInfo(hlbProcs.Items[hlbprocs.ItemIndex]);
  if ProcInfo <> nil then
  begin
    StatusBar.Panels[0].Text := ProcInfo.Name;
    StatusBar.Panels[1].Text := Format('%d/%d', [hlbProcs.ItemIndex + 1, hlbProcs.Items.Count]);
    sbGoto.Enabled := (hlbProcs.ItemIndex >= 0);
  end;
end;

function TfmProcs.GetProcInfo(st: string): TProcInfo;
var
  ProcedureIdx: Integer;
begin
  ProcedureIdx := ProcList.IndexOf(st);
  if ProcedureIdx >= 0 then
    Result := TProcInfo(ProcList.Objects[ProcedureIdx])
  else
    Result := nil;
end;

procedure TfmProcs.sbHelpClick(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 4);
end;

procedure TfmProcs.sbCopyClick(Sender: TObject);
var
  i: Integer;
  Procs: TStringList;
  ProcInfo: TProcInfo;
begin
  Procs := TStringList.Create;
  try
    for i := 0 to hlbProcs.Items.Count - 1 do
    begin
      ProcInfo := GetProcInfo(hlbProcs.Items[i]);
      if ProcInfo <> nil then
        Procs.Add(ProcInfo.Name);
    end;
  finally
    if (Procs <> nil) and (Procs.Count > 0) then
      Clipboard.AsText := Procs.Text;
    Procs.Free;
  end;
end;

procedure TfmProcs.FillListBox;
var
  i: Integer;
  ProcName: string;
begin
  hlbProcs.Items.BeginUpdate;
  try
    hlbProcs.Items.Clear;
    if Length(laSearch.Caption) = 0 then
    begin
      hlbProcs.Items.Assign(ProcList);
      if hlbProcs.Items.Count > 0 then
        hlbProcs.ItemIndex := 0;
      Exit;
    end;
    for i := 0 to ProcList.Count - 1 do
    begin
      ProcName := TProcInfo(ProcList.Objects[i]).Name;
      if sbStart.Down and
        (CompareText(laSearch.Caption, Copy(GetMethodName(ProcName), 1, Length(laSearch.Caption))) = 0) then
      begin
        hlbProcs.Items.Add(ProcList.Strings[i])
      end
      else
        if sbAny.Down and (Pos(UpperCase(laSearch.Caption), UpperCase(GetMethodName(ProcName))) <> 0) then
          hlbProcs.Items.Add(ProcList.Strings[i]);
    end;
    if hlbProcs.Items.Count > 0 then
      hlbProcs.ItemIndex := 0;
  finally
    hlbProcs.Items.EndUpdate;
  end;
end;

procedure TfmProcs.FormKeyPress(Sender: TObject; var Key: Char);
begin
  //! the first case should be a better match for
  // identifiers, based on *valid* Pascal identifiers
  // --> IsValidIdent
  case Key of
    '1'..'z':
      begin
        laSearch.Caption := laSearch.Caption + UpperCase(key);
        FillListBox;
      end;
    #8:
      begin
        laSearch.Caption := Copy(laSearch.Caption, 1, Length(laSearch.Caption) - 1);
        FillListBox;
      end;
    #13: btnGotoClick(hlbProcs);
    #27: Close;
  end; //case
end;

procedure TfmProcs.sbFontClick(Sender: TObject);
begin
  dlgProcFont.Font.Assign(hlbProcs.Font);
  if dlgProcFont.Execute then
    hlbProcs.Font.Assign(dlgProcFont.Font);
end;

procedure TfmProcs.FormResize(Sender: TObject);
begin
  with hlbprocs do
    Sections[1].Width := Width - Sections[2].Width - Sections[3].Width - Sections[0].Width;

  with StatusBar do
  begin
    if Width > 80 then
    begin
      Panels[1].Width := 80;
      Panels[0].Width := Width - 80;
    end;
  end;
  laSearch.Width := (pnBtnHolder.Left - laSearch.Left) - 10; // Scott Mattes, 98/11/23, added line
end;

procedure TfmProcs.sbStartClick(Sender: TObject);
begin
  FillListBox;
end;

procedure TfmProcs.SaveSettings;
var
  RegIni: TRegIniFile;
begin
  // do not localize any of the following lines
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    RegIni.WriteBool('ProcViewer', 'SearchAll', sbAny.Down);
    RegIni.WriteInteger('ProcViewer', 'Left', Left);
    RegIni.WriteInteger('ProcViewer', 'Top', Top);
    RegIni.WriteInteger('ProcViewer', 'Width', Width);
    RegIni.WriteInteger('ProcViewer', 'Height', Height);
    RegIni.WriteInteger('ProcViewer', 'SortColumn', Sort);
    SaveFont(RegIni, 'ProcViewer', hlbProcs.Font);
  finally
    RegIni.Free;
  end;
end;

procedure TfmProcs.LoadSettings;
var
  RegIni: TRegIniFile;
begin
  // do not localize any of the following lines
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    //{$IFOPT D+}SendDebug('Loading SearchAll: ' + BTS(RegIni.ReadBool('ProcViewer', 'SearchAll', False))); {$ENDIF}
    if RegIni.ReadBool('ProcViewer', 'SearchAll', True) then
      sbAny.Down := True
    else
      sbStart.Down := True;
    //{$IFOPT D+}SendDebug('sbAny.Down: ' + BTS(sbAny.Down) + ' sbStart.Down: ' + BTS(sbStart.Down)); {$ENDIF}
    Left := RegIni.ReadInteger('ProcViewer', 'Left', Left);
    Top := RegIni.ReadInteger('ProcViewer', 'Top', Top);
    Width := RegIni.ReadInteger('ProcViewer', 'Width', Width);
    Height := RegIni.ReadInteger('ProcViewer', 'Height', Height);
    Sort := RegIni.ReadInteger('ProcViewer', 'SortColumn', Sort);
    LoadFont(RegIni, 'ProcViewer', hlbProcs.Font);
  finally
    RegIni.Free;
  end;
end;

procedure TfmProcs.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveSettings;
end;

procedure TfmProcs.QuickSort(L, R: Integer);

  function GetValue(idx: Integer): string;
  resourcestring
    SInvalidIndex = 'Invalid index number';
  var
    i: Integer;
    TabPos: Integer;
  begin
    if idx >= ProcList.Count then
      raise Exception.Create(SInvalidIndex);
    Result := ProcList.Strings[idx];
    for i := 0 to sort - 1 do
    begin
      TabPos := Pos(#9, Result);
      if TabPos > 0 then
        Delete(Result, 1, TabPos)
      else
        Exit;
    end;
    if Sort = 3 then
    begin
      for i := Length(Result) to 5 do
        Result := ' ' + Result;
    end;
  end;

var
  I, J: Integer;
  P: string;
begin
  if ProcList.Count = 0 then
    Exit;
  repeat
    I := L;
    J := R;
    P := GetValue((L + R) shr 1);
    repeat
      while AnsiCompareText(GetValue(I), P) < 0 do Inc(I);
      while AnsiCompareText(GetValue(J), P) > 0 do Dec(J);
      if I <= J then
      begin
        ProcList.Exchange(I, J);
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then QuickSort(L, J);
    L := I;
  until I >= R;
end;

procedure TfmProcs.hlbProcsSectionClick(Sender: THeaderListbox;
  Section: THeaderSection);
var
  i: Integer;
begin
  i := Section.Index;
  if i <> 0 then
  begin
    Self.Cursor := crHourglass;
    try
      Sort := i;
      QuickSort(0, ProcList.Count - 1);
      FillListBox;
    finally
      Self.Cursor := crDefault;
    end;
  end;
end;

//************************ TProcedureExpert*****************

constructor TProcedureExpert.Create;
begin
  inherited Create;
  ShortCut := Menus.ShortCut(Word('G'), [ssCtrl]);
  HasConfigOptions := False;
  HasMenuItem := True;
end;

function TProcedureExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = '&Procedure List...';
begin
  Result := SMenuCaption;
end;

function TProcedureExpert.GetMenuName: string;
begin
  Result := 'GX_ProcList'; // do not localize
end;

function TProcedureExpert.GetMenuMask: string;
begin // Compatible with TLB/Interface files (though these shouldn't be edited)
  Result := '*.PAS;*.DPR;*.INC;*.TLB;*.OCX;*.OLB;*.DLL;*.EXE'; // do not localize
end;

function TProcedureExpert.GetName: string;
begin
  Result := 'Procedure_Viewer'; // do not localize
end;

function TProcedureExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'Procedure List';
begin
  Result := SDisplayName;
end;


procedure TProcedureExpert.Click(Sender: TObject);

  function IsTypeLibrary(const FileName: string): Boolean;
  var
    FileExt: string;
  begin
    FileExt := ExtractUpperFileExt(FileName);
    Result := ((FileExt = '.TLB')
            or (FileExt = '.OLB')
            or (FileExt = '.OCX')
            or (FileExt = '.DLL')
            or (FileExt = '.EXE')
               );
  end;

var
  FileName : string;
resourcestring
  SPasOrDprOnly = 'This expert is for use in .PAS or .DPR files only';
begin
  FileName := ToolServices.GetCurrentFile;
  if not (IsDprOrPas(FileName) or IsTypeLibrary(FileName)) then
    MessageDlg(SPasOrDprOnly, mtError, [mbOK], 0)
  else
  begin
    try
      {$IFOPT D+}SendDebug('Procedure Expert Activated'); {$ENDIF}
      with TfmProcs.Create(nil) do
      try
        ShowModal;
      finally
        Free;
      end;
    except
      on E: Exception do
        ShowExceptionErrorMessage(E);
    end;
  end;
end;

function TProcedureExpert.IconFileName: string;
begin
  Result := 'ProcView';
end;

initialization
  RegisterGX_Expert(TProcedureExpert);
end.

