unit GX_uClean;

{$I GX_CondDefine.inc}

//! StH: This unit has been completely prepared for localization
//! StH: This unit is fully compatible with C++ Builder

{ TODO -oStefan -c.98 DR1:

    Implement recursive cleaning of directories;

    * TListBox -> TCheckListBox
        - default unchecked
    * FindFirst / FindNext
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ToolIntf, ExptIntf, FileCtrl, GX_GExperts,
  GX_uExperts, Registry, CheckLst;

type
  TCleanExpert = class;

  TfmClean = class(TForm)
    gbxDirectories: TGroupBox;
    lbDirs: TListBox;
    btnAdd: TButton;
    btnRemove: TButton;
    btnCancel: TButton;
    btnClean: TButton;
    laStatus: TLabel;
    btnHelp: TButton;
    gbxExtensions: TGroupBox;
    clbExtensions: TCheckListBox;
    btnAddExt: TButton;
    btnRemoveExt: TButton;
    chkReportErrors: TCheckBox;
    lCleaning: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure clbExtensionsClickCheck(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnCleanClick(Sender: TObject);
    procedure lbDirsDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure btnHelpClick(Sender: TObject);
    procedure lbDirsClick(Sender: TObject);
    procedure btnAddExtClick(Sender: TObject);
    procedure btnRemoveExtClick(Sender: TObject);
    procedure lbDirsKeyPress(Sender: TObject; var Key: Char);
    procedure clbExtensionsClick(Sender: TObject);
    procedure clbExtensionsKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    MaxWidth: Integer;
    CleanExpert: TCleanExpert;
    CleanExtList: TStringList;
  public
    constructor CreateParametrized(OwningExpert: TCleanExpert);
  end;

  TCleanExpert = class(TGX_Expert)
  private
    FReportErrors: Boolean;
    FExtensionList: TStrings;
    FCleanList: TStrings;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    function IconFileName: string; override;
    procedure Click(Sender: TObject); override;
    procedure LoadSettings; override;
    procedure SaveSettings; override;
    property ExtensionList: TStrings read FExtensionList;
    property CleanList: TStrings read FCleanList;
  end;

implementation

{$R *.DFM}

uses
  GX_uConfigurationInfo,
{$IFOPT D+}
  GX_DbugIntf,
{$ENDIF D+}
  GX_uGenFunc;

resourcestring
  SCouldNotDelete = 'Could not delete %s'#13#13+
                    'Continue showing errors?';



function ListFiles(Param: Pointer; const FileName, Unitname, FormName: string): Boolean; stdcall;
var
  Strings: TStrings;
  TempPathString: string;
begin
  Result := True;
  try
    Strings := TfmClean(Param).lbDirs.Items;

    TempPathString := LowerCase(ExtractFilePath(FileName));
    if Strings.IndexOf(TempPathString) < 0 then
      Strings.Add(TempPathString);
  except
    on E: Exception do
      ShowExceptionErrorMessage(E);
  end;
end;

// ************************ TfmClean *********************

constructor TfmClean.CreateParametrized(OwningExpert: TCleanExpert);
begin
  CleanExpert := OwningExpert;

  inherited Create(nil);
end;

procedure TfmClean.FormCreate(Sender: TObject);
const
  CRLF = #13+#10;
resourcestring
  SDefaultCleanExts =
    {$IFDEF GX_VER130_up}
    '.~bpg'+CRLF+'.~cpp'+CRLF+'.~dfm'+CRLF+'.~dpk'+CRLF+
    '.~dsk'+CRLF+'.~h'  +CRLF+'.~hpp'+CRLF+'.~pas'+CRLF+
    {$ELSE}
    '.~bp' +CRLF+'.~cp' +CRLF+'.~df' +CRLF+'.~dp' +CRLF+
    '.~ds' +CRLF+'.~h'  +CRLF+'.~hp' +CRLF+'.~pa' +CRLF+
    {$ENDIF GX_VER130_up}
    '.bak' +CRLF+'.cfg' +CRLF+'.csm' +CRLF+'.dcu' +CRLF+
    '.dof' +CRLF+'.dsk' +CRLF+'.fts' +CRLF+'.gid' +CRLF+
    '.il*' +CRLF+'.kwf' +CRLF+'.md'  +CRLF+'.obj' +CRLF+
    '.tds' +CRLF+'.tmp' +CRLF+'.$*'  +CRLF+'.~*';
var
  i, j: Integer;
begin
  laStatus.Caption := '';

  MaxWidth := lbDirs.Width;

  if CleanExpert.ExtensionList.Count > 0 then
    clbExtensions.Items.Assign(CleanExpert.ExtensionList)
  else
    clbExtensions.Items.Text := SDefaultCleanExts;

  for i := 0 to CleanExtList.Count - 1 do
  begin
    j := clbExtensions.Items.IndexOf(CleanExtList[i]);
    if j >= 0 then
      clbExtensions.Checked[j] := True;
  end;

  ToolServices.EnumProjectUnits(ListFiles, Pointer(Self));
end;

procedure TfmClean.FormDestroy(Sender: TObject);
begin
  CleanExpert.CleanList.Assign(CleanExtList);
  CleanExpert.ExtensionList.Assign(clbExtensions.Items);

  CleanExtList.Free;
  CleanExtList := nil;
end;

procedure TfmClean.clbExtensionsClickCheck(Sender: TObject);
begin
  with Sender as TCheckListBox do
  begin
    if Checked[ItemIndex] then
      CleanExtList.Add(Items[ItemIndex])
    else
      CleanExtList.Delete(CleanExtList.IndexOf(Items[ItemIndex]));
  end;
end;

procedure TfmClean.btnAddClick(Sender: TObject);
var
  Temp: string;
begin
  Temp := '';
  if GetDir(Self, Temp) then
  begin
    Temp := AddSlash(Temp);
    lbDirs.Items.Add(Temp);
  end;
end;


procedure TfmClean.btnRemoveClick(Sender: TObject);
var
  i: Integer;
begin
  i := 0;
  while i <= lbDirs.Items.Count - 1 do
  begin
    if lbDirs.Selected[i] then
      lbDirs.Items.Delete(i)
    else
      Inc(i);
  end;
  btnRemove.Enabled := (lbDirs.ItemIndex > -1) and (lbDirs.Items.Count > 0);
end;

procedure TfmClean.btnAddExtClick(Sender: TObject);
resourcestring
  SAddNewExtension = 'Add file extension';
  SAddNewText = 'Enter the file extension to be cleaned:';
var
  NewExt: string;
  Idx: Integer;
begin
  if InputQuery(SAddNewExtension, SAddNewText, NewExt) then
  begin
    if NewExt[1] = '*' then
      Delete(NewExt, 1, 1);
    if not (NewExt[1] = '.') then
      NewExt := '.' + NewExt;
    Idx := clbExtensions.Items.Add(NewExt);
    clbExtensions.Checked[Idx] := True;
    // JCH 02/17/1999 - Fixes exception when clicking a new extension
    CleanExtList.Add(NewExt);
  end;
end;

procedure TfmClean.btnRemoveExtClick(Sender: TObject);
var
  i: Integer;
begin
  i := clbExtensions.ItemIndex;
  if i < 0 then
    Exit;

  clbExtensions.Items.Delete(i);

  btnRemoveExt.Enabled := (clbExtensions.ItemIndex > -1) and (clbExtensions.Items.Count > 0);
end;

procedure TfmClean.btnCleanClick(Sender: TObject);
var
  Total: Integer;
  TotalFiles: Integer;

    procedure DeleteFoundFile(const FileName: string);
{.$DEFINE SimulateDeleting}
{$IFNDEF SimulateDeleting}
    var
      TempSize: Integer;
{$ENDIF SimulateDeleting}
    begin
      {$IFOPT D+}SendDebug('Deleting file: ' + FileName);{$ENDIF}
{$IFNDEF SimulateDeleting}
      TempSize := GX_uGenFunc.GetFileSize(FileName);
      if DeleteFile(FileName) then
      begin
        Inc(TotalFiles);
        Total := Total + TempSize;
      end
      else
      begin
        if chkReportErrors.Checked then
        begin
          chkReportErrors.Checked := (MessageDlg( Format(SCouldNotDelete,
                                                  [FileName]), mtError, [mbYes, mbNo], 0) = mrYes);
        end;
      end;
{$ENDIF SimulateDeleting}
    end;

var
  i: Integer;
  j: Integer;
  SearchRec: TSearchRec;
  FoundFileExt: string;
  FindResult: Integer;
resourcestring
  SCleaningComplete = 'Cleaning complete. %d files have been deleted.'#13#10+
                      '%s bytes of space have been recovered.';
begin
  Total := 0;
  TotalFiles := 0;
  lCleaning.Visible := True;
  lCleaning.Repaint;
  Self.Cursor := crHourglass;
  try
    for i := 0 to lbDirs.Items.Count-1 do
    begin
      // Ascertain that we have a trailing slash
      lbDirs.Items[i] := AddSlash(lbDirs.Items[i]);

      laStatus.Caption := MinimizeName(lbDirs.Items[i], laStatus.Canvas, laStatus.Width);
      laStatus.Repaint;

      // we explicitly do not search for r/o files since we cannot
      // delete them anyway; alternatively do search r/o and have
      // an error reported?
      FindResult := FindFirst(lbDirs.Items[i] + '*.*', faHidden or faSysFile or faArchive, SearchRec);
      try
        while FindResult = 0 do
        begin
          FoundFileExt := ExtractFileExt(SearchRec.Name);
          for j := 0 to CleanExtList.Count - 1 do
          begin
            if CompareText(FoundFileExt, CleanExtList.Strings[j]) = 0 then
              DeleteFoundFile(lbDirs.Items[i] + SearchRec.Name)
          end;

          FindResult := FindNext(SearchRec);
        end;
      finally
        FindClose(SearchRec);
      end;
    end;
  finally
    Self.Cursor := crDefault;
    MessageDlg(Format(SCleaningComplete, [TotalFiles, FormatFloat('#,;;0', Total)]), mtInformation, [mbOK], 0);
    lCleaning.Visible := False;

    CleanExpert.CleanList.Assign(CleanExtList);
    CleanExpert.ExtensionList.Assign(clbExtensions.Items);

    ModalResult := mrOK;
  end;
end;

procedure TfmClean.lbDirsDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  TempString: string;
begin
  TempString := lbDirs.Items[Index];
  if lbDirs.Canvas.TextWidth(TempString) > MaxWidth then
  begin
    MaxWidth := lbDirs.Canvas.TextWidth(TempString);
    SendMessage(lbDirs.Handle, LB_SETHORIZONTALEXTENT, MaxWidth, 0);
    lbDirs.Invalidate;
  end;
  Rect.Right := Rect.Left + MaxWidth;
  lbDirs.Canvas.FillRect(Rect);
  // new style controls with DT_PATH_ELLIPSIS do not work out properly (buggy) - play it safe.
  DrawText(lbDirs.Canvas.Handle, PChar(TempString), -1, Rect, DT_LEFT);
end;

procedure TfmClean.btnHelpClick(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 14);
end;

procedure TfmClean.lbDirsClick(Sender: TObject);
begin
  btnRemove.Enabled := (lbDirs.ItemIndex > -1);
end;

procedure TfmClean.lbDirsKeyPress(Sender: TObject; var Key: Char);
begin
  btnRemove.Enabled := (lbDirs.ItemIndex > -1);
end;

procedure TfmClean.clbExtensionsClick(Sender: TObject);
begin
  btnRemoveExt.Enabled := (clbExtensions.ItemIndex > -1);
end;

procedure TfmClean.clbExtensionsKeyPress(Sender: TObject; var Key: Char);
begin
  btnRemoveExt.Enabled := (clbExtensions.ItemIndex > -1);
end;

// ************************ TCleanExpert *********************

constructor TCleanExpert.Create;
begin
  inherited Create;

  FCleanList := TStringList.Create;
  FExtensionList := TStringList.Create;

  HasConfigOptions := False;
  LoadSettings;
end;

destructor TCleanExpert.Destroy;
begin
  SaveSettings;

  FCleanList.Free;
  FCleanList := nil;

  FExtensionList.Free;
  FExtensionList := nil;

  inherited Destroy;
end;

function TCleanExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = 'Clea&n Directories...';
begin
  Result := SMenuCaption;
end;

function TCleanExpert.GetMenuName: string;
begin
  Result := 'GX_Clean';
end;

function TCleanExpert.GetMenuMask: string;
begin
  Result := '';
end;

function TCleanExpert.GetName: string;
begin
  Result := 'Clean_Directory';
end;

function TCleanExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'Clean Directories';
begin
  Result := SDisplayName;
end;

procedure TCleanExpert.Click(Sender: TObject);
begin
  with TfmClean.CreateParametrized(Self) do
  try
    chkReportErrors.Checked := FReportErrors;
    ShowModal;
    FReportErrors := chkReportErrors.Checked;
  finally
    Free;
  end;
end;

procedure TCleanExpert.LoadSettings;
var
  RegIni: TRegIniFile;
begin
  inherited;
  // do not localize
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    ReadStringList(RegIni, CleanList, 'CleanList', 'CleanExt');
    ReadStringList(RegIni, ExtensionList, 'CleanList', 'AvailableExt');
    FReportErrors := RegIni.ReadBool('CleanList', 'ReportError', True);
  finally
    RegIni.Free;
  end;
end;

procedure TCleanExpert.SaveSettings;
var
  RegIni: TRegIniFile;
begin
  inherited;
  // do not localize
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    RegIni.WriteBool('CleanList', 'ReportError', FReportErrors);
    WriteStringList(RegIni, ExtensionList, 'CleanList', 'AvailableExt');
    WriteStringList(RegIni, CleanList, 'CleanList', 'CleanExt');
  finally
    RegIni.Free;
  end;
end;

function TCleanExpert.IconFileName: string;
begin
  Result := 'Clean';
end;

initialization
  RegisterGX_Expert(TCleanExpert);
end.

