unit GX_uDepend;

{$I GX_CondDefine.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  {$IFDEF GX_VER120_up}
  ImgList,
  {$ENDIF GX_VER120_up}
  HeadList, ComCtrls, ExtCtrls, GX_EditRead, ToolIntf, ExptIntf, mPasLex,
  Buttons, Registry, Menus, GX_uExperts;

type
  TUnitInfo = class(TObject)
  public
    UnitName: string;
    FileName: string;
  end;

  TfmDepend = class;

  TProjectNotifier = class(TIAddInNotifier)
  private
    fmDepend: TfmDepend;
  public
    procedure FileNotification(NotifyCode: TFileNotification;
      const FileName: string; var Cancel: Boolean); override;
    procedure EventNotification(NotifyCode: TEventNotification;
      var Cancel: Boolean); override;
  end;

  TfmDepend = class(TForm)
    pnlToolbar: TPanel;
    StatusBar: TStatusBar;
    tvUnits: TTreeView;
    Splitter1: TSplitter;
    sbRefresh: TSpeedButton;
    sbAbort: TSpeedButton;
    Panel2: TPanel;
    pcData: TPageControl;
    tshUnitUses: TTabSheet;
    hlbUnitUses: THeaderListbox;
    tshUsedby: TTabSheet;
    hlbUsedBy: THeaderListbox;
    tshIndirect: TTabSheet;
    hlbIndirect: THeaderListbox;
    ilFiles: TImageList;
    pmTreeview: TPopupMenu;
    pmList: TPopupMenu;
    mitOpenUnit: TMenuItem;
    OpenUnit2: TMenuItem;
    N1: TMenuItem;
    Properties1: TMenuItem;
    sbHelp: TSpeedButton;
    MainMenu: TMainMenu;
    File1: TMenuItem;
    Refresh1: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    Help2: TMenuItem;
    N3: TMenuItem;
    About1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure sbRefreshClick(Sender: TObject);
    procedure sbAbortClick(Sender: TObject);
    procedure tvUnitsExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure Panel2Resize(Sender: TObject);
    procedure tvUnitsChange(Sender: TObject; Node: TTreeNode);
    procedure pcDataChange(Sender: TObject);
    procedure mitOpenUnitClick(Sender: TObject);
    procedure OpenUnit2Click(Sender: TObject);
    procedure pmListPopup(Sender: TObject);
    procedure Properties1Click(Sender: TObject);
    procedure sbHelpClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure tvUnitsEditing(Sender: TObject; Node: TTreeNode; var AllowEdit: Boolean);
    procedure pmTreeviewPopup(Sender: TObject);
    procedure hlbSectionClick(Sender: THeaderListbox;
      Section: THeaderSection);
  private
    ProjectNotifier: TProjectNotifier;
    procedure ShowUnitsUsed;
    procedure ShowUsedBy;
    procedure IndirectDepend;
    function GetFileName(const uName: string): string;
    procedure LoadSettings;
    procedure SaveSettings;
    procedure OpenUnit(const UnitName: string);
    //procedure SetListBoxExtent(LB: THeaderListbox);
  private
    RefreshList: Boolean;
    UnitList: TStringList;
    FileList: TStringList;
    Abort: Boolean;
    function FindUnit(const uName: string): Integer;
    procedure BuildUses;
    procedure Clear;
  end;

  TDependExpert = class(TGX_Expert)
  private
    FScanEntireUnit: boolean;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    procedure Configure; override;
    procedure SaveSettings; override;
    procedure LoadSettings; override;
    function IconFileName: string; override;
    procedure Click(Sender: TObject); override;
  end;

var
  fmDepend: TfmDepend = nil;
  DependExpert: TDependExpert = nil;
  RootNode: TTreeNode = nil;

implementation

{$R *.DFM}
{$R VCLOpen.res}

uses
{$IFOPT D+}
  GX_DbugIntf,
{$ENDIF D+}
  GX_uDependProp, GX_GExperts, GX_uConfigurationInfo, GX_uGenFunc, TypInfo, CommCtrl;

procedure TProjectNotifier.EventNotification(
  NotifyCode: TEventNotification; var Cancel: Boolean);
begin
  // nothing
end;

procedure TProjectNotifier.FileNotification(NotifyCode: TFileNotification;
  const FileName: string; var Cancel: Boolean);
begin
  if NotifyCode = fnProjectOpened then
  begin
    fmDepend.Clear;
    if fmDepend.Visible then
      fmDepend.BuildUses
    else
      fmDepend.RefreshList := True;
  end;
end;

procedure TfmDepend.FormCreate(Sender: TObject);
resourcestring
  SCouldNotLoadBitmaps = 'Could not load dependency bitmaps.';
begin
  RefreshList := False;
  with tvUnits do
    SetWindowLong(Handle, GWL_STYLE, GetWindowLong(Handle, GWL_STYLE) or TVS_NOTOOLTIPS);
  UnitList := TStringList.Create;
  FileList := TStringList.Create;
  if not ilFiles.ResourceLoad(rtBitmap, 'VCLOPEN', clFuchsia) then  // do not localize
    MessageDlg(SCouldNotLoadBitmaps, mtWarning, [mbOK], 0);
  CenterForm(Self);
  LoadSettings;
  ProjectNotifier := TProjectNotifier.Create;
  ProjectNotifier.fmDepend := Self;
  ToolServices.AddNotifier(ProjectNotifier);
end;

procedure TfmDepend.FormDestroy(Sender: TObject);
begin
  fmDepend := nil;
  Clear;
  UnitList.Free;
  UnitList := nil;
  FileList.Free;
  FileList := nil;
  SaveSettings;
  ToolServices.RemoveNotifier(ProjectNotifier);
  ProjectNotifier.Free;
  ProjectNotifier := nil;
end;

procedure TfmDepend.Clear;
var
  i: Integer;
begin
  if UnitList <> nil then
  begin
  for i := 0 to UnitList.Count-1 do
    UnitList.Objects[i].Free;
  UnitList.Clear;
  end;

  if FileList <> nil then
  begin
    for i := 0 to FileList.Count-1 do
      FileList.Objects[i].Free;
    FileList.Clear;
  end;
end;

function LoadFileDepend(Param: Pointer; const FileName, Unitname,
  FormName: string): Boolean; stdcall;
var
  Form: TfmDepend;
  EditRead: TEditReader;
  MS: TMemoryStream;
  Parser: TmwPasLex;
  nUses: Integer;
  Index: Integer;
  Node: TTreeNode;
  UList: TStringList;
  uInfo: TUnitInfo;

begin
  Result := True;
  if FileName = ToolServices.GetProjectName then Exit;
  nUses := 0;
  UList := nil;
  //Parser := nil;
  Form := TfmDepend(Param);
  with Form.StatusBar do
  begin
    SimpleText := FileName;
    Repaint;
  end;

  if Form.Abort then
  begin
    Result := False;
    Form.sbAbort.Enabled := False;
    Form.sbRefresh.Enabled := True;
    Exit;
  end;

  if not IsDprOrPas(FileName) then
    Exit;
  if FormName = '' then
    Index := 2
  else
    Index := 3;
  //{$IFOPT D+}SendDebug('Found Project Unit: '+UnitName);{$ENDIF}
  UInfo := TUnitInfo.Create;
  UInfo.UnitName := UnitName;
  UInfo.FileName := FileName;
  Form.FileList.AddObject(UnitName, uInfo);
  MS := TMemoryStream.Create;
  try
    // Since this edit reader is destroyed almost
    // almost immediately, do not call FreeFileData
    EditRead := TEditReader.Create(FileName);
    try
      EditRead.SaveToStream(MS);
    finally
      EditRead.Free;
    end;
    Parser := TmwPasLex.Create;
    try
      //{$IFOPT D+}SendDebug('Parsing '+Unitname);{$ENDIF}
      Parser.Origin := MS.Memory;
      //{$IFOPT D+}SendDebug('First: '+Parser.Token+'('+GetEnumName(TypeInfo(TTokenKind), integer(Parser.TokenID))+')');{$ENDIF}
      while not (Parser.TokenID in [tkUnit, tkNull, tkLibrary, tkProgram]) do
      begin
        Parser.NextNoJunk;
        //{$IFOPT D+}SendDebug('Looking for Unit, found: '+Parser.Token);{$ENDIF}
      end;
      //{$IFOPT D+}SendDebug('L/P/U Found: '+Parser.Token);{$ENDIF}
      if Parser.TokenID in [tkUnit, tkLibrary, tkProgram] then
      begin
        Parser.NextNoJunk;
        //{$IFOPT D+}SendDebug('ID is: '+Parser.Token);{$ENDIF}
        if Parser.TokenID = tkIdentifier then
        begin
          Node := Form.tvUnits.Items.AddChild(RootNode, Parser.Token);
          //{$IFOPT D+}SendDebug('Added '+Parser.Token+' to '+RootNode.Text);{$ENDIF}
          Node.HasChildren := True;
          Node.ImageIndex := Index;
          Node.SelectedIndex := Index;
          UList := TStringList.Create;
          Form.UnitList.AddObject(Parser.Token, UList);
        end;
      end;
      if UList = nil then
        Exit;
      while Parser.TokenID <> tkNull do
      begin
        if Parser.TokenID = tkUses then
        begin
          inc(nUses);
          Parser.NextNoJunk;
          while (Parser.TokenID <> tkSemiColon) and (Parser.TokenID <> tkNull) do
          begin
            if Parser.TokenID = tkIdentifier then
              UList.Add(Parser.Token);
            Parser.NextNoJunk;
            if Parser.TokenID = tkIn then
              while not (Parser.TokenID in [tkSemiColon, tkComma, tkNull]) do
                Parser.NextNoJunk;
          end;
        end;
        if ((DependExpert <> nil) and (not DependExpert.FScanEntireUnit)) and (nUses >= 2) then
          Break;
        Parser.NextNoJunk;
      end;
    finally
      Parser.Free;
    end;
  finally
    MS.Free;
  end;
end;

procedure TfmDepend.BuildUses;
resourcestring
  SParsingUnits = 'Parsing units...';
begin
  tvUnits.Items.BeginUpdate;
  Screen.Cursor := crHourglass;
  sbRefresh.Enabled := False;
  sbAbort.Enabled := True;
  try
    Clear;
    Abort := false;
    { Clear current scroll box }
    hlbIndirect.Clear;
    tvUnits.Items.Clear;
    { Start parsing }
    StatusBar.SimpleText := SParsingUnits;
    StatusBar.Repaint;
    RootNode := tvUnits.Items.Add(nil, ExtractFileName(ToolServices.GetProjectName));
    RootNode.ImageIndex := 4;
    RootNode.SelectedIndex := 4;
    ToolServices.EnumProjectUnits(LoadFileDepend, Pointer(Self));
    RootNode.Expand(False);
    tvUnits.AlphaSort;
  finally
    sbRefresh.Enabled := True;
    sbAbort.Enabled := False;
    Screen.Cursor := crDefault;
    tvUnits.Items.EndUpdate;
  end;
end;

procedure TfmDepend.sbRefreshClick(Sender: TObject);
begin
  BuildUses;
end;

procedure TfmDepend.sbAbortClick(Sender: TObject);
begin
  Abort := True;
end;

function TfmDepend.FindUnit(const uName: string): Integer;
begin
  Result := UnitList.Count-1;
  while Result >= 0 do
  begin
    if CompareText(uName, UnitList.Strings[Result]) = 0 then
      Break;

    Dec(Result);
  end;
end;

function TfmDepend.GetFileName(const uName: string): string;
var
  UInfo: TUnitInfo;
  i: Integer;
begin
  Result := '';
  i := FileList.IndexOf(UpperCase(UName));
  if i >= 0 then
  begin
    UInfo := TUnitInfo(FileList.Objects[i]);
    Result := UInfo.FileName;
  end;
end;

procedure TfmDepend.tvUnitsExpanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
var
  i, j, k: Integer;
  List: TStringList;
  CNode: TTreeNode;
begin
  //First Clear out existing objects
  AllowExpansion := True;
  if Node = RootNode then
    Exit;
  while Node.Count > 0 do
    Node.Item[0].Free;
  i := FindUnit(Node.Text);
  if i >= 0 then
  begin
    List := TStringList(UnitList.Objects[i]);
    for j := 0 to List.Count - 1 do
    begin
      CNode := tvUnits.Items.AddChild(Node, List.Strings[j]);
      CNode.SelectedIndex := 2;
      CNode.ImageIndex := 2;
      k := FindUnit(List.Strings[j]);
      CNode.HasChildren := ((k > -1) and (TStringList(UnitList.Objects[k]).Count > 0));
    end;
    if List.Count = 0 then
      Node.HasChildren := False;
  end
  else
    Node.HasChildren := False;
end;

procedure TfmDepend.Panel2Resize(Sender: TObject);
begin
  with hlbUnitUses do
    Sections[1].Width := Width - Sections[0].Width;
  with hlbUsedBy do
    Sections[1].Width := Width - Sections[0].Width;
  with hlbIndirect do
    Sections[1].Width := Width - Sections[0].Width;
end;

procedure TfmDepend.tvUnitsChange(Sender: TObject; Node: TTreeNode);
begin
  pcDataChange(pcData);
end;

procedure TfmDepend.pcDataChange(Sender: TObject);
begin
  if pcData.ActivePage = tshUnitUses then
    ShowUnitsUsed
  else
  if pcData.ActivePage = tshUsedBy then
    ShowUsedby
  else
  if pcData.ActivePage = tshIndirect then
    IndirectDepend;
  // VCL 4.0 Bug
  {$IFDEF GX_VER120_up}
  pcData.ActivePage.Realign;
  Panel2Resize(Self);
  {$ENDIF GX_VER120_up}
end;

procedure TfmDepend.ShowUnitsUsed;
var
  i, j: Integer;
  List: TStringList;
begin
  hlbUnitUses.Clear;
  if tvUnits.Selected = nil then
    Exit;
  i := FindUnit(tvUnits.Selected.Text);
  if i >= 0 then
  begin
    List := TStringList(UnitList.Objects[i]);
    for j := 0 to List.Count - 1 do
      hlbUnitUses.Items.Add(List.Strings[j] + #9 + GetFileName(List.Strings[j]));
  end;
  //SetListBoxExtent(hlbUnitUses);
end;

(*
procedure TfmDepend.SetListBoxExtent(LB: THeaderListbox);
var
 MaxLen, Len, i: integer;
begin
  Canvas.Font := LB.font;
  try
    MaxLen := 0;
    for i := 0 to LB.Items.Count-1 do begin
      Len := Canvas.TextWidth(LB.Items[i]);
      if Len > MaxLen then
        MaxLen := Len;
    end;
  finally
    Canvas.Font := Font;
  end;
  LB.Perform(LB_SETHORIZONTALEXTENT, MaxLen, 0);
end;
*)

procedure TfmDepend.ShowUsedBy;
var
  i, j: Integer;
  List: TStringList;
begin
  hlbUsedBy.Clear;
  if tvUnits.Selected = nil then
    Exit;
  for i := 0 to UnitList.Count - 1 do
  begin
    List := TStringList(UnitList.Objects[i]);
    for j := 0 to List.Count - 1 do
      if CompareText(tvUnits.Selected.Text, List.Strings[j]) = 0 then
        hlbUsedBy.Items.Add(UnitList.Strings[i] + #9 + GetFileName(UnitList.Strings[i]));
  end;
  //SetListBoxExtent(hlbUsedBy);
end;

procedure TfmDepend.IndirectDepend;
var
  List: TStringList;

  procedure AddUses(const UnitName, Src: string);
  var
    i: Integer;
  begin
    for i := 0 to hlbIndirect.Items.Count-1 do
      if CompareText(UnitName, hlbIndirect.Values[0, i]) = 0 then
      begin
        if not ((Pos(Src, hlbIndirect.Items[i]) = 1) or
           (Pos(Src, ', ' + hlbIndirect.Items[i]) > 0)) then
        begin
          hlbIndirect.Items[i] := hlbIndirect.Items[i] + ', ' + Src;
        end;
        Exit;
      end;
    hlbIndirect.Items.Add(UnitName + #9 + Src);
  end;

  procedure AddItems(UnitName: string);
  var
    i, j: Integer;
    MList: TStringList;
  begin
    i := FindUnit(UnitName);
    if i >= 0 then
    begin
      MList := TStringList(UnitList.Objects[i]);
      for j := 0 to MList.Count-1 do
      begin
        AddUses(MList.Strings[j], UnitName);
        if List.IndexOf(UpperCase(MList.Strings[j])) < 0 then
        begin
          List.Add(UpperCase(MList.Strings[j]));
          AddItems(MList.Strings[j]);
        end;
      end;
    end;
  end;

begin
  if tvUnits.Selected = nil then
    Exit;
  Screen.Cursor := crHourglass;
  List := TStringList.Create;
  try
    AddItems(tvUnits.Selected.Text);
  finally
    List.Free;
    Screen.Cursor := crDefault;
  end;
  //SetListBoxExtent(hlbIndirect);
end;

procedure TfmDepend.SaveSettings;
var
  RegIni: TRegIniFile;
begin
  // do not localize
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    RegIni.WriteInteger('Depend', 'Left', Left);
    RegIni.WriteInteger('Depend', 'Top', Top);
    RegIni.WriteInteger('Depend', 'Width', Width);
    RegIni.WriteInteger('Depend', 'Height', Height);
    RegIni.WriteInteger('Depend', 'Splitter', tvUnits.Width);
  finally
    RegIni.Free;
  end;
end;

procedure TfmDepend.LoadSettings;
var
  RegIni: TRegIniFile;
begin
  // do not localize
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    Left := RegIni.ReadInteger('Depend', 'Left', Left);
    Top := RegIni.ReadInteger('Depend', 'Top', Top);
    Width := RegIni.ReadInteger('Depend', 'Width', Width);
    Height := RegIni.ReadInteger('Depend', 'Height', Height);
    tvUnits.Width := RegIni.ReadInteger('Depend', 'Splitter', tvUnits.Width);
  finally
    RegIni.Free;
  end;
end;

procedure TfmDepend.mitOpenUnitClick(Sender: TObject);
begin
  if tvUnits.Selected <> nil then
    OpenUnit(tvUnits.Selected.Text);
end;

procedure TfmDepend.OpenUnit(const UnitName: string);
var
  i: Integer;
  CurrentFileName: string;
resourcestring
  SFileDoesNotExist = '%s does not exist';
  SOpenError = 'This unit is not in the current project.'#13#10 +
               'A later version of GExperts might be able to open units on the library path.';
begin
  i := FileList.IndexOf(UnitName);
  if i >= 0 then
  begin
    CurrentFileName := TUnitInfo(FileList.Objects[i]).FileName;
    if FileExists(CurrentFileName) then
      ToolServices.OpenFile(CurrentFileName)
    else
      MessageDlg(Format(SFileDoesNotExist, [CurrentFileName]), mtError, [mbOK], 0);
  end
  else
    MessageDlg(SOpenError, mtInformation, [mbOK], 0);
end;

procedure TfmDepend.OpenUnit2Click(Sender: TObject);

  procedure CallOpenUnit(hlb: THeaderListBox);
  begin
    if hlb.ItemIndex >= 0 then
      OpenUnit(hlb.Values[0, hlb.ItemIndex]);
  end;

begin
  if pcData.ActivePage = tshUnitUses then
    CallOpenUnit(hlbUnitUses)
  else
  if pcData.ActivePage = tshUsedBy then
    CallOpenUnit(hlbUsedBy)
  else
  if pcData.ActivePage = tshIndirect then
    CallOpenUnit(hlbIndirect);
end;

procedure TfmDepend.pmListPopup(Sender: TObject);
var
  IsVisible: Boolean;
begin
  IsVisible := (pcData.ActivePage = tshIndirect);

  Properties1.Visible := IsVisible;
  N1.Visible := IsVisible;
end;

procedure TfmDepend.Properties1Click(Sender: TObject);
begin
  if hlbIndirect.ItemIndex < 0 then Exit;
  with TfmDependProp.Create(nil) do
  try
    laFileName.Caption := hlbIndirect.Values[0, hlbIndirect.ItemIndex];
    lbSource.Items.CommaText := hlbIndirect.Values[1, hlbIndirect.ItemIndex];
    ShowModal;
  finally
    Free;
  end;
end;

procedure TfmDepend.sbHelpClick(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 6);
end;

procedure TfmDepend.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfmDepend.About1Click(Sender: TObject);
begin
  ShowGXAboutForm;
end;

procedure TfmDepend.pmTreeviewPopup(Sender: TObject);
begin
  mitOpenUnit.Enabled := (tvUnits.Selected <> RootNode);
end;

procedure TfmDepend.FormActivate(Sender: TObject);
begin
  if RefreshList then
  begin
    RefreshList := False;
    BuildUses;
  end;
end;

procedure TfmDepend.tvUnitsEditing(Sender: TObject; Node: TTreeNode;
  var AllowEdit: Boolean);
begin
  AllowEdit := False;
end;

procedure TfmDepend.hlbSectionClick(Sender: THeaderListbox;
  Section: THeaderSection);
begin
  if Sender.SortColumn = Section.Index then
    Sender.SortAscending := not Sender.SortAscending
  else
  begin
    Sender.SortColumn := Section.Index;
    Sender.SortAscending := True;
  end;
  Sender.Sort(Sender.SortColumn, Sender.SortAscending);
end;

//************************ TDependExpert ********************

constructor TDependExpert.Create;
begin
  inherited Create;
  DependExpert := Self;
  HasConfigOptions := True;
  HasMenuItem := True;
end;

destructor TDependExpert.Destroy;
begin
  DependExpert := nil;

  fmDepend.Free;
  fmDepend := nil;

  inherited Destroy;
end;

function TDependExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = 'Project &Dependencies';
begin
  Result := SMenuCaption;
end;

function TDependExpert.GetMenuName: string;
begin
  Result := 'GX_Depend';
end;

function TDependExpert.GetMenuMask: string;
begin
  Result := '';
end;

function TDependExpert.GetName: string;
begin
  Result := 'Dependency_Expert';
end;

function TDependExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'Project Dependencies';
begin
  Result := SDisplayName;
end;

procedure TDependExpert.Configure;
resourcestring
  SConfigureExplanation = 'Do you want Project Dependencies to scan the entire unit for uses clauses?'#13#13+
                          'This is slower, but will also find additional, unusual uses clauses that have been '+
                          'conditionally defined.';
begin
  FScanEntireUnit := (MessageDlg(SConfigureExplanation, mtConfirmation, [mbYes, mbNo], 0) = mrYes);
end;

procedure TDependExpert.SaveSettings;
var
  RegIni: TRegIniFile;
begin
  inherited SaveSettings;

  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    RegIni.WriteBool('Depend', 'ScanEntireUnit', FScanEntireUnit);
  finally
    RegIni.Free;
  end;
end;

procedure TDependExpert.LoadSettings;
var
  RegIni: TRegIniFile;
begin
  inherited LoadSettings;

  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    FScanEntireUnit := RegIni.ReadBool('Depend', 'ScanEntireUnit', False);
  finally
    RegIni.Free;
  end;
end;

procedure TDependExpert.Click(Sender: TObject);
begin
  if fmDepend = nil then
  begin
    fmDepend := TfmDepend.Create(nil);
    fmDepend.BuildUses;
  end;
  if fmDepend.WindowState = wsMinimized then
    fmDepend.WindowState := wsNormal;
  fmDepend.Show;
end;

function TDependExpert.IconFileName: string;
begin
  Result := 'Depend';
end;

initialization
  RegisterGX_Expert(TDependExpert);
end.

