unit Gx_uEditorEnhance;

{$I GX_CondDefine.Inc}

{$IFNDEF GX_EII}
interface implementation
{$ELSE GX_EII}

interface

uses
  Classes, SysUtils, Windows, Forms, Messages, Controls,
  ExtCtrls, Menus, Buttons, Dialogs, StdCtrls,
  ToolIntf, ExptIntf,
  EIManager, EIPanel, EINotifiers;

type
  TEditorEnhancements = class(TComponent)
  private
    FToolBarAlign: TAlign;
    FIdeManager: TIdeManager;
    FFormChangeNotifier: TEIMessageNotifier;
    FDialogExit: TNotifyEvent;
    FToolbarButtonsList: TList;
    FToolBarList: TList;
    FToolbarNotifier: TIAddInNotifier;
    FNewEditWindow: TNotifyEvent;
    FButtons: Boolean;
    FButtonsFlat: Boolean;
    FMultiLine: Boolean;
    FHotTrack: Boolean;
    FToolBarVisible: Boolean;
    FFontControl: Boolean;
    FEditorContextMenus: Boolean;
    FEditorContextMenusAtBottom: Boolean;

    FDoHandleEnvDialog: Boolean;
    FHookedEnvDialogInstance: TForm;
    FEnabled: Boolean;

    FHookedDialogBounds: TRect;

    procedure LoadSettings;

    procedure NewEditWindowClick(Sender: TObject);
    procedure HookViewNewEditorItem(DoHook: Boolean);
    procedure LogFormChange(Sender: TObject);
    procedure DialogOnDestroyHook(Sender: TObject);
    procedure DialogOnResizeHook(Sender: TObject);
    procedure AddToolbar;
    procedure RemoveToolbar;
    procedure SetEnabled(const Value: Boolean);
    procedure Install;
    procedure Remove;
  protected
    procedure Notification(Component: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SaveSettings;
    procedure ExecuteExpertByNo(ButtonNo: Integer);
    // Toolbar code
    procedure ApplyToolbarSettings;
    property ToolbarButtonsList: TList read FToolbarButtonsList write FToolbarButtonsList;

    procedure ShowConfigurationDialog(ActiveTab: string);
    property HookedEnvDialogInstance: TForm read FHookedEnvDialogInstance write FHookedEnvDialogInstance;
    property DoHandleEnvDialog: Boolean read FDoHandleEnvDialog write FDoHandleEnvDialog;
  public
    property Enabled: Boolean read FEnabled write SetEnabled;
    property IdeManager: TIdeManager read FIdeManager;
    // Tab properties
    property MultiLine: Boolean read FMultiLine write FMultiLine;
    property HotTrack: Boolean read FHotTrack write FHotTrack;
    property Buttons: Boolean read FButtons write FButtons;
    property ButtonsFlat: Boolean read FButtonsFlat write FButtonsFlat;
    property FontControl: Boolean read FFontControl write FFontControl;
    // Context menu properties
    property EditorContextMenus: Boolean read FEditorContextMenus write FEditorContextMenus;
    property EditorContextMenusAtBottom: Boolean read FEditorContextMenusAtBottom write FEditorContextMenusAtBottom;
    // Toolbar properties
    property ToolbarAlign: TAlign read FToolBarAlign write FToolBarAlign;
    property ToolBarVisible: Boolean read FToolBarVisible write FToolBarVisible;
  end;

function EditorEnhancements: TEditorEnhancements;
procedure FreeEditorEnhancements;

implementation

uses
  Registry, ComCtrls,
  {$IFOPT D+} GX_DbugIntf, {$ENDIF D+}
  GX_uConfigurationInfo, GX_uConfigure,
  GX_GExperts,
  Gx_uToolbar, Gx_uToolbarButtons,
  GX_uGenFunc;

type
  TFormChangeNotifier = class(TEIMessageNotifier)
  private
    FEditManager: TEditorEnhancements;
  public
    constructor Create(EditManager: TEditorEnhancements);
    procedure BeforeEditorMessageProcess(Manager: TEIManager;
      OriginalMessage: TMessage; var Msg: TMessage); override;
    procedure OnActiveFormChanged(AForm: TCustomForm); override;
    procedure OnApplicationActivate(Activated: Boolean); override;
  end;

{ TFormChangeNotifier }

constructor TFormChangeNotifier.Create(EditManager: TEditorEnhancements);
begin
  inherited Create;

  Assert(EditManager <> nil);
  FEditManager := EditManager;
end;

procedure TFormChangeNotifier.BeforeEditorMessageProcess(Manager: TEIManager;
  OriginalMessage: TMessage; var Msg: TMessage);
begin
  case OriginalMessage.Msg of
    CN_KEYUP:
      begin
        if TWMKey(Msg).CharCode = 0 then
          Msg.Result := 1;
      end;
  end;
end;

procedure TFormChangeNotifier.OnActiveFormChanged(AForm: TCustomForm);
begin
  FEditManager.LogFormChange(AForm);
end;

procedure TFormChangeNotifier.OnApplicationActivate(Activated: Boolean);
begin
  // Nothing
end;

type
  TToolBarNotifier = class(TIAddInNotifier)
  private
    FEditManager: TEditorEnhancements;
  public
    constructor Create(EditManager: TEditorEnhancements);
    procedure FileNotification(NotifyCode: TFileNotification;
      const FileName: string; var Cancel: Boolean); override;
    procedure EventNotification(NotifyCode: TEventNotification;
      var Cancel: Boolean); override;
  end;

{ ToolBarNotifier }

constructor TToolBarNotifier.Create(EditManager: TEditorEnhancements);
begin
  inherited Create;

  Assert(EditManager <> nil);
  FEditManager := EditManager;
end;

procedure TToolBarNotifier.EventNotification(NotifyCode: TEventNotification; var Cancel: Boolean);
begin
  // nothing
end;

procedure TToolBarNotifier.FileNotification(NotifyCode: TFileNotification;
  const FileName: string; var Cancel: Boolean);
begin
  case NotifyCode of
    fnFileOpened,
    fnProjectOpened,
    fnDefaultDesktopLoad,
    fnProjectDesktopLoad:  FEditManager.AddToolbar;
  else
    // nothing
  end;
end;

{ TEditorEnhancements }

constructor TEditorEnhancements.Create(AOwner: TComponent);
begin
  //! StH: There have been some rather inexplicable problems (AVs)
  // if AOwner <> nil; simply take no risks - we don't need an owner

  Assert(AOwner = nil);

  inherited Create(AOwner);

  Name := 'GX_EditEnhance';

  FToolBarAlign := alTop;
  FHookedEnvDialogInstance := nil;
  FEditorContextMenus := False;
  FFontControl := False;
  FMultiLine := False;
  FEditorContextMenusAtBottom := True;
  {$IFDEF GX_VER120_up}
  // Delphi 4+ default to hot tracking on
  FHotTrack := True;
  {$ELSE}
  FHotTrack := False;
  {$ENDIF GX_VER120_up}

  FButtons := False;
  FButtonsFlat := False;
  FToolBarVisible := True;

  FToolbarButtonsList := TList.Create;

  LoadSettings;
end;

destructor TEditorEnhancements.Destroy;
begin
  {$IFOPT D+}SendDebug('Destroying Editor Enhancements'); {$ENDIF}
  Remove;

  FToolbarButtonsList.Free;
  FToolbarButtonsList := nil;

  {$IFOPT D+}SendDebug('Editor Enhancements Destroyed'); {$ENDIF}

  inherited Destroy;
end;

procedure TEditorEnhancements.Install;
begin
  if ToolServices = nil then
    Exit;

  // Initialize the IDE Manager
  FIdeManager := InstallIDEIntegrationManager('GExperts');

  FToolbarList := TList.Create;

  // Notifiers
  FToolbarNotifier := TToolBarNotifier.Create(Self);
  ToolServices.AddNotifier(FToolbarNotifier);

  FFormChangeNotifier := TFormChangeNotifier.Create(Self);
  Assert(FIdeManager <> nil);
  FIdeManager.AddNotifier(FFormChangeNotifier);

  AddToolbar;
  HookViewNewEditorItem(True);
end;

procedure TEditorEnhancements.Remove;

  function IdeManagerExists: Boolean;
  begin
    Result := (FindIDEIntegrationManager <> nil);
  end;

begin
  if ToolServices = nil then
    Exit;

  HookViewNewEditorItem(False);
  Application.ProcessMessages;

  if IdeManagerExists and (FFormChangeNotifier <> nil) then
  begin
    {$IFOPT D+}SendDebug('Removing EII Notifier'); {$ENDIF}
    Assert(FIdeManager <> nil);
    FIdeManager.RemoveNotifier(FFormChangeNotifier);
  end;
  FFormChangeNotifier.Free;
  FFormChangeNotifier := nil;

  if FToolbarNotifier <> nil then
  begin
    ToolServices.RemoveNotifier(FToolbarNotifier);
    FToolbarNotifier.Free;
    FToolbarNotifier := nil;
  end;

  RemoveToolbar;

  FToolBarList.Free;
  FToolBarList := nil;
end;

procedure TEditorEnhancements.LogFormChange(Sender: TObject);
var
  ActiveForm: TForm;

    procedure HandleTEditWindow;
    var
      Editor: TCustomControl;
    begin
      AddToolbar;
      Editor := ActiveForm.FindComponent('Editor') as TCustomControl;
      if Editor <> nil then
        Editor.SetFocus;
    end;

    procedure HandleTEnvDialog;
    begin
      ActiveForm.SetBounds(-1000, -1000, 1, 1);
      FHookedEnvDialogInstance := ActiveForm;
      Application.ProcessMessages;
    end;

    procedure HandleTViewDialog;
    begin
      try
        ActiveForm.BorderStyle := bsSizeable;
      except
        on E: Exception do
        begin
          // Need to swallow visible warning
        end;
      end;
      ActiveForm.BoundsRect := FHookedDialogBounds;

      FDialogExit := ActiveForm.OnDestroy;
      ActiveForm.OnDestroy := DialogOnDestroyHook;
      ActiveForm.OnResize := DialogOnResizeHook;

      ShowWindow(ActiveForm.Handle, SW_SHOWNA);

      Application.ProcessMessages;
    end;

begin
  ActiveForm := Screen.ActiveForm;
  if ActiveForm = nil then
    Exit;

  {$IFOPT D+} SendDebug('FormChange Window Class:' + Screen.ActiveForm.ClassName + ' (' + Screen.ActiveForm.Name + ')'); {$ENDIF}

  if CompareText(Screen.ActiveForm.Classname, 'TEditWindow') = 0 then
    HandleTEditWindow
  else
  if (CompareText(Screen.ActiveForm.ClassName, 'TEnvDialog') = 0) and FDoHandleEnvDialog then
    HandleTEnvDialog
  else
  if (CompareText(Screen.ActiveForm.Classname, 'TViewDialog') = 0) and False {DExpert.Resize} then
    HandleTViewDialog;
end;

procedure TEditorEnhancements.DialogOnResizeHook(Sender: TObject);
var
  Form: TForm;
  ListBox: TListBox;
  HelpButton, CancelButton, OKButton: TButton;
  Edit: TEdit;
begin
  Form := Sender as TForm;
  if Form = nil then
    Exit;

  ListBox := Form.FindComponent('ListBox') as TListBox;
  HelpButton := Form.FindComponent('HelpButton') as TButton;
  CancelButton := Form.FindComponent('CancelButton') as TButton;
  OKButton := Form.FindComponent('OKButton') as TButton;
  Edit := Form.FindComponent('Edit') as TEdit;

  if (ListBox = nil) or (OKButton = nil) or (HelpButton = nil) or
     (CancelButton = nil) or (Edit = nil) then
  begin
    Exit;
  end;

  if Form.Width >= 120 then
  begin
    OKButton.Left := Form.ClientWidth - OKButton.Width - 6;
    HelpButton.Left := OKButton.Left;
    CancelButton.Left := OKButton.Left;
    ListBox.Width := OKButton.Left - 6 - ListBox.Left;
    ListBox.Height := Form.ClientHeight - ListBox.Top - 4;
    Edit.Width := ListBox.Width;
  end;
end;

procedure TEditorEnhancements.DialogOnDestroyHook(Sender: TObject);
var
  SendingForm: TForm;
begin
  if Sender is TForm then
  begin
    SendingForm := TForm(Sender);

    FHookedDialogBounds := SendingForm.BoundsRect;
    if Assigned(FDialogExit) then
    begin
      SendingForm.OnDestroy := FDialogExit;
      FDialogExit(Sender);
    end;
  end;
end;

procedure TEditorEnhancements.HookViewNewEditorItem(DoHook: Boolean);
var
  Menu: TMenuItem;
  App: TCustomForm;
begin
  App := GetIdeMainForm;
  if App = nil then
    Exit;

  Menu := App.FindComponent('ViewNewEditorItem') as TMenuItem;
  if Menu <> nil then
  begin
    if DoHook then
    begin
      FNewEditWindow := Menu.OnClick;
      Menu.OnClick := NewEditWindowClick;
    end
    else
    begin
      if Assigned(FNewEditWindow) then
        Menu.OnClick := FNewEditWindow;
      FNewEditWindow := nil;
    end;
  end;
end;

procedure TEditorEnhancements.NewEditWindowClick(Sender: TObject);
begin
  if Assigned(FNewEditWindow) then
    FNewEditWindow(Sender);

  AddToolbar;
end;

procedure TEditorEnhancements.AddToolbar;
var
  i: Integer;
  EditForm: TForm;
  ToolBar: TGXToolBar;
  EIManager: TEIManager;
  EIPanel: TEIPanel;
begin
  Assert(FIdeManager <> nil);
  for i := 0 to FIdeManager.EIManagerCount - 1 do
  begin
    EIManager := FIdeManager.EIManagers[i];
    EditForm := EIManager.Editor;
    if EditForm.FindComponent('GX_Toolbar') = nil then
    begin
      Toolbar := TGXToolBar.CreateManaged(EditForm, Self);
      FToolBarList.Add(ToolBar);
      ToolBar.FreeNotification(Self);

      EIPanel := FIdeManager.RequestIntegration(EIManager, ToolBar);
      // We do not want to have any buttons on
      // the integration panel - and we do not
      // want any caption on our panel
      EIPanel.TitleBarButtonVisible := [];
      EIPanel.TitleBarButtonEnabled := [];
      EIPanel.TitleBarVisible := False;

      //
      EIPanel.Align := FToolbarAlign;
      if FToolBarAlign in [alTop, alBottom] then
        EIPanel.Height := 33
      else
        EIPanel.Width := 33;

      // Hide the splitter bar
      EIPanel.SplitterVisible := False;
      EIPanel.BevelInner := bvNone;
      EIPanel.BevelOuter := bvNone;

      Toolbar.Align := alClient;
      Toolbar.Name := 'GX_Toolbar';
      Toolbar.Caption := '';
      ToolBar.Parent := EIPanel;
      ToolBar.BringToFront;
      ToolBar.CreateToolbarButtons;

      // Finally make the EIPanel visible depending on settings
      EIPanel.Visible := ToolBarVisible;

      ToolBar.SetEditorControls;
    end;
  end;
end;

procedure TEditorEnhancements.RemoveToolbar;
var
  EIPanel: TEIPanel;
  GExpertsToolbar: TGXToolBar;
begin
  if Application.Terminated then
    Exit;

  if FToolBarList = nil then
    Exit;

  while FToolBarList.Count > 0 do
  begin
    GExpertsToolbar := FToolBarList.Items[0];

    EIPanel := GExpertsToolbar.Parent as TEIPanel;
    // Note, notification removes the item from the FToolBarList
    GExpertsToolbar.Free;
    { TODO -oStefan -Issue: Should we Free the EIPanel, too? }
    EIPanel.Visible := False;
  end;
end;

procedure TEditorEnhancements.Notification(Component: TComponent; Operation: TOperation);
var
  i: Integer;
begin
  if Operation = opRemove then
  begin
    i := FToolBarList.IndexOf(Component);
    if i >= 0 then
      FToolBarList.Delete(i);
  end;

  inherited Notification(Component, Operation);
end;

resourcestring
  SEditorExpertMissing = 'Editor expert %d not found.';

procedure TEditorEnhancements.ExecuteExpertByNo(ButtonNo: Integer);
var
  i: Integer;
begin
  for i := 0 to GExpertsInst.EditorExpertCount - 1 do
  begin
    if GExpertsInst.EditorExpertList[i].ButtonNo = ButtonNo then
    begin
      GExpertsInst.EditorExpertList[i].Execute;
      Exit;
    end;
  end;
  MessageDlg(Format(SEditorExpertMissing, [ButtonNo]), mtError, [mbOK], 0);
end;

procedure TEditorEnhancements.ShowConfigurationDialog(ActiveTab: string);
var
  Dlg: TfmConfiguration;
  TabComp: TComponent;
  i: Integer;
begin
  Dlg := TfmConfiguration.Create(nil);
  try
    with Dlg do
    begin
      TabComp := FindComponent(ActiveTab);
      if TabComp is TTabSheet then
        pcConfig.ActivePage := TTabSheet(TabComp);
      // Hide non-essential tab sheets
      for i := 0 to pcConfig.PageCount - 1 do
        pcConfig.Pages[i].TabVisible := (pcConfig.Pages[i].Name  = 'tshEditor') or
          (pcConfig.Pages[i].Name  = 'tshToolbar');
    end;
    Dlg.ShowModal;
  finally
    Dlg.Free;
  end;
end;

procedure TEditorEnhancements.ApplyToolbarSettings;
var
  i: Integer;
  EIPanel: TEIPanel;
  GExpertsToolbar: TGXToolBar;
begin
  if FToolBarList = nil then
    Exit;

  for i := 0 to FToolBarList.Count - 1 do
  begin
    GExpertsToolbar := FToolBarList.Items[i];

    EIPanel := GExpertsToolbar.Parent as TEIPanel;
    EIPanel.Visible := False;
    EIPanel.Align := FToolBarAlign;

    if FToolBarAlign in [alTop, alBottom] then
      EIPanel.Height := 33
    else
      EIPanel.Width := 33;

    EIPanel.Visible := ToolBarVisible;

    GExpertsToolbar.CreateToolbarButtons;
    GExpertsToolbar.SetEditorControls;
  end;
end;

procedure TEditorEnhancements.SetEnabled(const Value: Boolean);
begin
  if FEnabled = Value then
    Exit;

  FEnabled := Value;
  if FEnabled then
    Install
  else
    Remove;
end;

procedure TEditorEnhancements.SaveSettings;
var
  RegIni: TRegIniFile;
  i: Integer;
begin
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    RegIni.WriteInteger('ToolBar', 'Count', FToolbarButtonsList.Count);

    for i := 0 to FToolbarButtonsList.Count - 1 do
      RegIni.WriteInteger('ToolBar', 'Button' + IntToStr(i), Integer(FToolbarButtonsList.Items[i]));

    RegIni.WriteBool('EditEnhance', 'MultiLine', MultiLine);
    RegIni.WriteBool('EditEnhance', 'HotTrack', HotTrack);
    RegIni.WriteBool('EditEnhance', 'Buttons', Buttons);
    RegIni.WriteBool('EditEnhance', 'ButtonsFlat', ButtonsFlat);
    RegIni.WriteBool('EditEnhance', 'ToolBarVisible', ToolBarVisible);
    RegIni.WriteBool('EditEnhance', 'FontControl', FontControl);
    RegIni.WriteBool('EditEnhance', 'EditMenus', FEditorContextMenus);
    {$IFNDEF GX_VER110_up}
    RegIni.WriteBool('EditEnhance', 'BTMenus', FEditorContextMenusAtBottom);
    {$ENDIF GX_VER110_up}
    RegIni.WriteInteger('EditEnhance', 'ToolbarAlign', Ord(FToolBarAlign));
  finally
    RegIni.Free;
  end;
end;

procedure TEditorEnhancements.LoadSettings;
var
  RegIni: TRegIniFile;
  i: Integer;
  ToolbarButtonCount: Integer;
begin
  Assert(ConfigInfo <> nil);

  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    ToolbarButtonCount := RegIni.ReadInteger('ToolBar', 'Count', 0);
    for i := 0 to ToolbarButtonCount-1 do
      FToolbarButtonsList.Add(Pointer(RegIni.ReadInteger('ToolBar', 'Button' + IntToStr(i), SeparatorMenuItemMarker)));

    if ToolbarButtonCount = 0 then
    begin
      // Add default buttons
      { TODO -oStefan -cIssue : Hard-coded toolbar button indices }
      for i := 8 to 12 do
        FToolbarButtonsList.Add(Pointer(i));
    end;

    MultiLine := RegIni.ReadBool('EditEnhance', 'MultiLine', MultiLine);
    HotTrack := RegIni.ReadBool('EditEnhance', 'HotTrack', HotTrack);
    Buttons := RegIni.ReadBool('EditEnhance', 'Buttons', Buttons);
    ButtonsFlat := RegIni.ReadBool('EditEnhance', 'ButtonsFlat', ButtonsFlat);
    ToolBarVisible := RegIni.ReadBool('EditEnhance', 'ToolBarVisible', ToolBarVisible);
    FontControl := RegIni.ReadBool('EditEnhance', 'FontControl', FontControl);
    {$IFNDEF GX_VER130_up}
    FEditorContextMenus := RegIni.ReadBool('EditEnhance', 'EditMenus', FEditorContextMenus);
    {$ENDIF GX_VER130_up}
    {$IFNDEF GX_VER110_up}
    FEditorContextMenusAtBottom := RegIni.ReadBool('EditEnhance', 'BTMenus', FEditorContextMenusAtBottom);
    {$ENDIF GX_VER110_up}
    FToolBarAlign := TAlign(RegIni.ReadInteger('EditEnhance', 'ToolbarAlign', Ord(FToolBarAlign)));
  finally
    RegIni.Free;
  end;
end;

var
  PrivateEditorEnhancements: TEditorEnhancements;
  CanCreate: Boolean = True;

function EditorEnhancements: TEditorEnhancements;
begin
  Assert(CanCreate);

  if PrivateEditorEnhancements = nil then
    PrivateEditorEnhancements := TEditorEnhancements.Create(nil);

  Result := PrivateEditorEnhancements;
end;

procedure FreeEditorEnhancements;
begin
  CanCreate := False;

  PrivateEditorEnhancements.Free;
  PrivateEditorEnhancements := nil;
end;

initialization

finalization
  FreeEditorEnhancements;

{$ENDIF GX_EII}

end.
