unit GX_uGenFunc;

{$I GX_CondDefine.inc}

//! StH: This unit has been completely prepared for localization
//! StH: This unit is fully compatible with C++ Builder

interface

uses
  SysUtils, Forms, Classes, Registry, Graphics,
  Controls, Windows, Dialogs, ComCtrls,
  {$IFNDEF GX_BCB}
  LibIntf,  // If you get errors here, you didn't read SourceCode.txt
            // See GX_uResults for a way to use LibIntf in BCB
  {$ENDIF GX_BCB}
  {$IFDEF DELPHIDEBUGGINGPACKAGE}
  Debugger,
  {$ENDIF DELPHIDEBUGGINGPACKAGE}
  {$IFDEF SYNTAXMEMO}
  SyntaxEd, SynParse,
  {$ENDIF SYNTAXMEMO}
  EditIntf, ToolIntf, ExptIntf;

{$IFDEF VER100}
const // Not defined in Delphi 3
  TVS_NOTOOLTIPS = $80;
{$ENDIF VER100}

type
  TUnitInfo = class(TObject)
  public
    UnitName: string;
    FormName: string;
    FileName: string;
  end;

  TCompInfo = class(TObject)
  public
    CompName: string;
    CompType: string;
  end;

{$IFNDEF GX_BCB}
procedure SelectComponent(Name: string);
{$ENDIF GX_BCB}
procedure GetCompInfoList(List: TStringList);
procedure GetUnitInfoList(List: TStringList);
//procedure OutputComponentList(Form: TForm);
function GetFileSize(const FileName: string): Integer;
function Min(v1, v2: Integer): Integer;
function Max(v1, v2: Integer): Integer;
function CaseInsensitivePos(const SubString, S: string): Integer;
function GetFileName(FileName: string): string;
procedure CenterForm(Form: TCustomForm);
function StripWhiteSpace(const S: string): string;
//function NextChar(s: string; pos: Integer; pat: string): Boolean;
function AddSlash(const Directory: string): string;
function IsStandAlone: Boolean;

{$IFDEF SYNTAXMEMO}
type
  GXSyntaxParser = (gxpPlaceHolder, gxpNone, gxpPAS, gxpCPP, gxpHTML, gxpSQL);
  GXSyntaxParsers = set of GXSyntaxParser;

function GetSyntaxParser(SyntaxParser: GXSyntaxParser): TSyntaxMemoParser;

{$R Parsers.res}
{$ENDIF SYNTAXMEMO}

{$IFNDEF ACEREPORTER}
procedure MessageNoAceReporter;
{$ENDIF ACEREPORTER}

function GetPropAsString(CompIntf: TIComponentInterface; Index: Integer): string;
function GetPropIndex(CompIntf: TIComponentInterface; const Name: string): Integer;
procedure SetPropAsString(CompIntf: TIComponentInterface; Index: Integer; const Value: string);
function GetProjectDir: string;
function GetProjectName: string;
//function IsCodeRushInstalled: Boolean;
function ExtractPureFilename(const FileName: string): string;
function GetDelphiBackgroundColor: TColor;
function GetDir(ParentForm: TForm; var Dir: string): Boolean;
procedure SaveFont(RegIni: TRegIniFile; const Key: string; Font: TFont);
procedure LoadFont(RegIni: TRegIniFile; const Key: string; Font: TFont);
procedure ReadStringList(RegIni: TRegIniFile; List: TStrings; const Key, SubKey: string);
procedure WriteStringList(RegIni: TRegIniFile; List: TStrings; const Key, Subkey: string);
procedure AssertIsDprOrPas(const FileName: string);
function IsDprOrPas(const FileName: string): Boolean;
function IsDfm(const FileName: string): Boolean;
function IsKnownSourceFile(const FileName: string): Boolean;
function IsCppBuilderIde: Boolean;
function ExtractUpperFileExt(const FileName: string): string;
{$IFOPT D+}
function BooleanText(B: Boolean): string;
{$ENDIF D+}
function DLLName: string;
function GetCurrentEditView(EditorIntf: TIEditorInterface): Integer;
function BuiltWithPackages: Boolean;
procedure ShowNoPackagesError;
//procedure PackTable(Table: TTable);
function GetIdeMainForm: TCustomForm;
function GetComponentPaletteTabControl: TTabControl;

// Shows the exception error message
procedure ShowExceptionErrorMessage(E: Exception);
// Use this function for silent processing
procedure ProcessException(E: Exception);
// This function will report the exception with the message (and E.Message)
procedure ProcessExceptionMsg(E: Exception; const Msg: string);

implementation

uses           
  {$IFOPT D+}
  GX_DbugIntf,
  {$ENDIF D+}
  DdeMan, BrowseDr,
  {$IFNDEF GX_NOBDE}
  DB, DBTables, BDE,
  {$ENDIF GX_NOBDE}
  GX_GExperts, GX_VerDepConst, GX_uConfigurationInfo;

const
  Unknown = 'Unknown Expert,Unknown,0'; // do not localize
  {$IFDEF SYNTAXMEMO}
var
  Parsers: array[GXSyntaxParser] of TSyntaxMemoParser;
  {$ENDIF SYNTAXMEMO}

procedure ReadStringList(RegIni: TRegIniFile; List: TStrings; const Key, SubKey: string);
var
  i: Integer;
  ListString: string;
begin
  for i := 0 to RegIni.ReadInteger(Key, 'Count', 0) - 1 do // do not localize
  begin
    ListString := RegIni.ReadString(Key, SubKey + IntToStr(i), Unknown);
    if ListString <> Unknown then
      List.Add(ListString);
  end;
end;

procedure WriteStringList(RegIni: TRegIniFile; List: TStrings; const Key, Subkey: string);
var
  i: Integer;
begin
  with RegIni do
  begin
    for i := 0 to List.Count - 1 do
      WriteString(Key, SubKey + IntToStr(i), List.Strings[i]);
    WriteInteger(Key, 'Count', List.Count); // do not localize
  end;
end;

function GetDir(ParentForm: TForm; var Dir: string): Boolean;
resourcestring
  SSelDir = 'Select a directory';
var
  Dlg: TBrowseDirectoryDlg;
begin
  Dlg := TBrowseDirectoryDlg.Create(nil);
  try
    with Dlg do
    begin
      Parent := ParentForm;
      Root := idDesktopExpanded;
      Selection := Dir;
      Title := SSelDir;

      // Windows NT at least with IE 4.0 has problems
      // when demanding that the status bar be shown.
      // Simply turn off the staus bar if that happens

      // If you get an error here, you must upgrade to Delphi 3.02 before compiling!
      if (Win32Platform <> VER_PLATFORM_WIN32_NT) and (GetComCtlVersion > ComCtlVersionIE4) then
      begin
        Options := Options + [bfDirectoriesOnly, bfEditBox];
        ShowSelectionInStatus := True;
        StatusText := SSelDir + '...'; // do not localize
      end
      else
      begin
        Options := Options + [bfDirectoriesOnly];
        ShowSelectionInStatus := False;
      end;

      Center := True;
      Caption := SSelDir;
      Result := Execute;
      if Result then
        Dir := Dlg.Selection;
    end;
  finally
    Dlg.Free;
  end;
end;

procedure SaveFont(RegIni: TRegIniFile; const Key: string; Font: TFont);
begin
  with RegIni do
  begin
    // Do not localize any of the following strings
    WriteString(Key, 'Name', Font.Name);
    WriteInteger(key, 'Size', Font.Size);
    WriteBool(Key, 'Bold', (fsBold in Font.Style));
    WriteBool(Key, 'Italic', (fsItalic in Font.Style));
    WriteBool(Key, 'Underline', (fsUnderline in Font.Style));
  end;
end;

procedure LoadFont(RegIni: TRegIniFile; const Key: string; Font: TFont);
begin
  with RegIni do
  begin
    // Do not localize any of the following strings
    Font.Name := ReadString(Key, 'Name', Font.Name);
    Font.Size := ReadInteger(key, 'Size', Font.Size);
    if ReadBool(Key, 'Bold', (fsBold in Font.Style)) then
      Font.Style := Font.Style + [fsBold];
    if ReadBool(Key, 'Italic', (fsItalic in Font.Style)) then
      Font.Style := Font.Style + [fsItalic];
    if ReadBool(Key, 'Underline', (fsUnderline in Font.Style)) then
      Font.Style := Font.Style + [fsUnderLine];
  end;
end;

{$IFNDEF ACEREPORTER}
procedure MessageNoAceReporter;
resourcestring
  SMessageNoAceReporter = 'Feature not available - GExperts has been compiled without ACE Reporter support.';
begin
  MessageDlg(SMessageNoAceReporter, mtInformation, [mbOK], 0);
end;
{$ENDIF ACEREPORTER}


function GetDelphiBackgroundColor: TColor;
const
  {$IFNDEF GX_VER110_up}
  RegAddKey = '';
  {$ELSE}
  RegAddKey = '\Editor\Highlight'; // do not localize
  {$ENDIF GX_VER110_up}
var
  RegIniFile: TRegIniFile;
  {$IFNDEF GX_VER110_up}
  StringSetting: string;
  CommaPos: Integer;
  {$ENDIF GX_VER110_up}
begin
  Assert(ToolServices <> nil);
  RegIniFile := TRegIniFile.Create(ToolServices.GetBaseRegistryKey + RegAddKey);
  try
    try
      {$IFNDEF GX_VER110_up}
      StringSetting := RegIniFile.ReadString('Highlight', 'Whitespace', '16');
      // Extract numerical value for StringSetting
      CommaPos := Pos(',', StringSetting);
      if CommaPos > 0 then
      begin
        Delete(StringSetting, 1, CommaPos);
        CommaPos := Pos(',', StringSetting);
        if CommaPos > 0 then
          StringSetting := Copy(StringSetting, 1, CommaPos - 1);
      end;
      Result := TColor(StrToInt(StringSetting));
      {$ELSE}
      if RegIniFile.ReadInteger('Whitespace', 'Background Color', 15) = 0 then
        Result := clBlack
      else
        Result := TColor(RegIniFile.ReadInteger('Whitespace', 'Background Color', 15));
      {$ENDIF GX_VER110_up}
    except
      on E: Exception do
      begin
        Result := clWhite;
      end;
    end;
  finally
    RegIniFile.Free;
  end;
  {$IFOPT D+}SendDebug('Delphi Background Color is: ' + ColorToString(Result) + '  '); {$ENDIF}
end;

function ExtractPureFilename(const FileName: string): string;
var
  ExtPos: Integer;
begin
  Result := ExtractFileName(FileName);
  if Result = '' then Exit;
  ExtPos := CaseInsensitivePos(ExtractFileExt(Result), Result);
  if ExtPos > 0 then
    Delete(Result, ExtPos, Length(Result));
end;

function LoadUnitList(Param: Pointer; const FileName, Unitname,
  FormName: string): Boolean; stdcall;
var
  List: TStringList;
  UInfo: TUnitInfo;
begin
  Result := True;
  List := TStringList(Param);
  if UnitName <> '' then
  begin
    UInfo := TUnitInfo.Create;
    UInfo.UnitName := UnitName;
    UInfo.FileName := FileName;
    UInfo.FormName := FormName;
    List.AddObject(UnitName, UInfo);
  end;
end;

procedure GetUnitInfoList(List: TStringList);
begin
  ToolServices.EnumProjectUnits(LoadUnitList, List);
end;

procedure GetCompInfoList(List: TStringList);
var
  CompIntf, FormCompIntf: TIComponentInterface;
  ModIntf: TIModuleInterface;
  FormIntf: TIFormInterface;
  CompInfo: TCompInfo;
  Control: TControl;
  UnitName, FileName, FileExt: string;
  i: Integer;
begin
  // ModIntf:=nil;
  FormCompIntf := nil;
  FormIntf := nil;

  FileName := UpperCase(ToolServices.GetCurrentFile);
  if IsDfm(FileName) then
  begin
    // Get module interface
    FileExt := ExtractFileExt(FileName);
    i := Pos(FileExt, FileName);
    if i > 0 then
    begin
      //! StH: FIXME
      { TODO -oStefan -cC++Builder : DFM -> PAS assumption is not valid for C++Builder }
      UnitName := Copy(FileName, 1, i) + 'PAS';
    end
    else
      UnitName := FileName;
  end
  else
    UnitName := FileName;

  ModIntf := ToolServices.GetModuleInterface(UnitName);
  if ModIntf = nil then
    Exit;

  try
    FormIntf := ModIntf.GetFormInterface;
    if FormIntf = nil then
      Exit;

    FormCompIntf := FormIntf.GetFormComponent;
    if FormCompIntf <> nil then
      for i := 0 to FormCompIntf.GetComponentCount - 1 do
      begin
        CompIntf := FormCompIntf.GetComponent(i);
        try
          if CompIntF <> nil then
          begin
            CompInfo := TCompInfo.Create;
            Control := TControl(CompIntf.GetComponentHandle);
            CompInfo.CompName := Control.Name;
            CompInfo.CompType := Control.ClassName;
            List.AddObject(CompInfo.CompName, CompInfo);
          end;
        finally
          CompIntf.Free;
        end;
      end;
  finally
    if (FormCompIntf <> nil) and (FormCompIntf.GetComponentHandle <> nil) then
      FormCompIntf.Free;

    ModIntf.Free;
    FormIntf.Free;
  end;
end;

{$IFNDEF GX_BCB}
// This function is defined elsewhere for C++Builder
procedure SelectComponent(Name: string);
var
  CompIntf, FormCompIntf: TIComponentInterface;
  ModIntf: TIModuleInterface;
  FormIntf: TIFormInterface;
  Form: TIForm;
  Control: TControl;
  UnitName, FileName, FileExt: string;
  i: Integer;

  procedure ShowForm;
  begin
    if ModIntf <> nil then
    begin
      ModIntf.ShowForm;
      if FormCompIntf <> nil then
        TForm(FormCompIntf.GetComponentHandle).SendToBack;
    end;
  end;

begin
  ModIntf := nil;
  FormCompIntf := nil;
  FormIntf := nil;

  FileName := UpperCase(ToolServices.GetCurrentFile);
  if IsDfm(FileName) then
  begin
    // Get module interface
    FileExt := ExtractFileExt(FileName);
    i := Pos(FileExt, FileName);
    if i > 0 then
      UnitName := Copy(FileName, 1, i) + 'PAS'
    else
      UnitName := FileName;
  end
  else
    UnitName := FileName;

  ModIntf := ToolServices.GetModuleInterface(UnitName);
  if ModIntf = nil then
    Exit;
  try
    FormIntf := ModIntf.GetFormInterface;
    if FormIntf = nil then
      Exit;

    FormCompIntf := FormIntf.GetFormComponent;
    if FormCompIntf <> nil then
      for i := 0 to FormCompIntf.GetComponentCount - 1 do
      begin
        CompIntf := FormCompIntf.GetComponent(i);
        try
          if CompIntF <> nil then
          begin
            Control := TControl(CompIntf.GetComponentHandle);
            if CompareText(Control.Name, Name) = 0 then
            begin
              Form := CompLib.GetActiveForm;
              if Form = nil then
                ShowForm
              else
              begin
                if CompareText(Form.GetFormName, TForm(FormCompIntf.GetComponentHandle).Name) <> 0 then
                  ShowForm;
              end;
              CompIntf.Select;
              Exit;
            end;
          end;
        finally
          CompIntf.Free;
        end;
      end;
  finally
    if (FormCompIntf <> nil) and (FormCompIntf.GetComponentHandle <> nil) then
      FormCompIntf.Free;

    ModIntf.Free;
    FormIntf.Free;
  end;
end;
{$ENDIF GX_BCB}

(*
function IsCodeRushInstalled: Boolean;
var
  Reg: TRegistry;
  List: TStringList;
  i: Integer;
begin
  IsCodeRushInstalled := False;
  List := TStringList.Create;
  Reg := TRegistry.Create;
  try
    if not Reg.OpenKey('Software\Borland\Delphi\4.0\Known Packages', False) then exit;
    Reg.GetValueNames(List);
    for i := 0 to List.Count - 1 do
      if uppercase(ExtractFileName(List.Strings[i])) = 'CODERUSH3.DPL' then
      begin
        IsCodeRushInstalled := True;
        Break;
      end;
  finally
    Reg.Free;
    List.Free;
  end;
end;

procedure OutputComponentList(Form: TForm);
var
  i: Integer;
begin
  for i := 0 to Form.ComponentCount - 1 do
    SendDebugEx(Form.Components[i].Name, mtInformation);
end;
*)

function AddSlash(const Directory: string): string;
begin
  if Length(Trim(Directory)) = 0 then
    Result := ''
  else
    if Directory[Length(Directory)] <> '\' then
      Result := Directory + '\'
    else
      Result := Directory;
end;

function IsStandAlone: Boolean;
begin
  Result := (ToolServices = nil);
end;

{$IFOPT D+}
function BooleanText(B: Boolean): string;
const
  BoolText: array[Boolean] of string =
    ('False', 'True'); // Do not localize
begin
  Result := BoolText[B];
end;
{$ENDIF}

{$IFDEF SYNTAXMEMO}
function GetSyntaxParser(SyntaxParser: GXSyntaxParser): TSyntaxMemoParser;

  procedure CreateCompiledParser(AParser: GXSyntaxParser; const ResourceName: string);
  var
    RS: TResourceStream;
  begin
    Parsers[AParser] := TSyntaxMemoParser.Create(nil);
    Parsers[AParser].UI_Styles.GutterWidth := 0;
    RS := TResourceStream.Create(HInstance, ResourceName, RT_RCDATA);
    try
      Parsers[AParser].CompileFromStream(RS);
    finally
      RS.Free;
    end;
  end;

begin
  // First try to get a cached copy and use that one;
  // if none exists, create a parser instance
  Result := Parsers[SyntaxParser];
  if Result <> nil then
    Exit;

  case SyntaxParser of
    gxpPAS:
      begin
        CreateCompiledParser(gxpPAS, 'PASPARSER');
        {$IFDEF GX_VER120_up}
          // Has the user customized the coloring? (TSyntaxMemo.SetToDelphi bug workaround)
          with TRegIniFile.Create(ConfigInfo.RegKey + '\Editor\Highlight\Assembler') do
            if ValueExists('Bold') then
              SetToDelphi(Parsers[gxpPAS], MajorVersionNumberChar);
        {$ENDIF GX_VER120_up}

        {$IFDEF VER100}
        SetToDelphi(Parsers[gxpPAS], MajorVersionNumberChar);
        {$ENDIF VER100}
      end;

    gxpCPP:
      begin
        CreateCompiledParser(gxpCPP, 'CPARSER');

        {$IFDEF GX_DELPHI}
        SetToDelphi(Parsers[gxpCPP], MajorVersionNumberChar);
        {$ENDIF GX_DELPHI}

        {$IFDEF GX_BCB}
        SetToBcb(Parsers[gxpCPP], MajorVersionNumberChar);
        {$ENDIF GX_BCB}
      end;

    gxpHTML:
      begin
        CreateCompiledParser(gxpHTML, 'HTMLPARSER');
      end;

    gxpSQL:
      begin
        CreateCompiledParser(gxpSQL, 'SQLPARSER');
      end;

  else
    Assert(False, 'Internal error getting syntax parser');
  end;

  Result := Parsers[SyntaxParser];
  Assert(Result <> nil);
end;

procedure FreeSyntaxParsers;
var
  i: GXSyntaxParser;
begin
  for i := Low(GXSyntaxParser) to High(GXSyntaxParser) do
  begin
    Parsers[i].Free;
    Parsers[i] := nil;
  end;
end;
{$ENDIF SYNTAXMEMO}

function GetFileSize(const FileName: string): Integer;
var
  F: file of Byte;
  OldFileMode: Byte;
begin
  AssignFile(F, FileName);
  try
    OldFileMode := FileMode;
    FileMode := 0; // read-only

    Reset(F);
    try
      Result := FileSize(F);
    finally
      CloseFile(F);
      FileMode := OldFileMode;
    end;
  except
    on E: Exception do
      Result := 0;
  end;
end;

function BuiltWithPackages: Boolean;
var
  IsNotInInstance: Boolean;
begin
  Result := True;

  {$IFNDEF GX_NotPackageBuilt}
  IsNotInInstance := (FindClassHInstance(TForm) <> HINSTANCE); // VCL
  Result := Result and IsNotInInstance;

  IsNotInInstance := (FindClassHInstance(TDdeServerConv) <> HINSTANCE); // VCLX
  Result := Result and IsNotInInstance;

  {$IFNDEF GX_NOBDE}
  IsNotInInstance := (FindClassHInstance(TDataSet) <> HINSTANCE); // VCLDB
  Result := Result and IsNotInInstance;

  IsNotInInstance := (FindClassHInstance(TTable) <> HINSTANCE); // VCLBDE/VCLDB
  Result := Result and IsNotInInstance;
  {$ENDIF GX_NOBDE}

  {$ENDIF GX_NotPackageBuilt}
end;

procedure ShowNoPackagesError;
resourcestring
  SGxBuildError = 'GExperts build error';
  SNotBuiltWithRequiredPackages =
 'GExperts has not been built with the required runtime packages '#13+
 #13+
 {$UNDEF PackageListDefined}
 {$IFDEF VER130}
   '  VCL50, VCLX50, VCLDB50, DSNIDE50, and VCLBDE50'#13+
   {$DEFINE PackageListDefined}
 {$ENDIF VER130}
 {$IFDEF VER125}
   '  VCL40, VCLX40, and VCLDB40'#13+
   {$DEFINE PackageListDefined}
 {$ENDIF VER125}
 {$IFDEF VER120}
   '  VCL40, VCLX40, and VCLDB40'#13+
   {$DEFINE PackageListDefined}
 {$ENDIF VER120}
 {$IFDEF VER110}
   '  VCL35, VCLX35, and VCLDB35'#13+
   {$DEFINE PackageListDefined}
 {$ENDIF VER110}
 {$IFDEF VER100}
   '  VCL30, VCLX30, and VCLDB30'#13+
   {$DEFINE PackageListDefined}
 {$ENDIF VER100}
 {$IFNDEF PackageListDefined}
   Package list not specified
 {$ENDIF PackageListDefined}
 {$UNDEF PackageListDefined}
 #13+
 'Please add these packages to the list of used runtime packages, '+
 'check the "Build with runtime packages" box, and rebuild GExperts.'#13+
 #13+
 'GExperts will not be installed into the IDE until this has been done.';
begin
  Windows.MessageBox(0,
                     PChar(SNotBuiltWithRequiredPackages),
                     PChar(SGxBuildError),
                     MB_OK or MB_ICONERROR);
end;

function GetProjectDir: string;
begin
  Assert(ToolServices <> nil);
  Result := ExtractFilePath(ToolServices.GetProjectName);
  if Length(Result) = 0 then
    raise ERangeError.Create('String error');
  if Result[Length(Result)] = '\' then
    Delete(Result, Length(Result), 1);
end;

function GetProjectName: string;
const
  DprExtension = '.DPR';
var
  DprPos: Integer;
begin
  Assert(ToolServices <> nil);
  Result := UpperCase(ExtractFileName(ToolServices.GetProjectName));
  DprPos := Pos(DprExtension, Result);
  if DprPos > 0 then
    Delete(Result, DprPos, Length(DprExtension));
end;

(*
function NextChar(s: string; pos: Integer; pat: string): Boolean;
begin
  Result := False;
  if pos > length(s) then exit;
  while (pos <= length(s)) and (s[pos] = ' ') do
    inc(pos);
  if pos > length(s) then exit;
  Result := (npos(s[pos], pat) > 0);
end;
*)

function GetFileName(FileName: string): string;
var
  FileExt: string;
  ExtPos: Integer;
begin
  FileName := ExtractFileName(FileName);
  FileExt := ExtractFileExt(FileName);
  ExtPos := Pos(FileExt, FileName);
  if ExtPos > 0 then
    Result := Copy(FileName, 1, ExtPos - 1)
  else
    Result := FileName;
end;

procedure CenterForm(Form: TCustomForm);
var
  Rect: TRect;
begin
  if Form = nil then Exit;

  SystemParametersInfo(SPI_GETWORKAREA, 0, @Rect, 0);

  with Form do
  begin
    SetBounds((Rect.Right - Rect.Left - Width) div 2,
      (Rect.Bottom - Rect.Top - Height) div 2,
               {Form.} Width,
               {Form.} Height);
  end;
end;

function StripWhiteSpace(const S: string): string;
var
  i: integer;
begin
  Result := S;

  i := Length(Result);
  while i > 0 do
  begin
    case Result[i] of

      #10, #13: Delete(Result, i, 1);

      #32:
        begin
          // this requires special casing
          // we only want to remove doubled blanks
          if (i > 1) and (Result[i - 1] = #32) then
            Delete(Result, i, 1);
        end;
    end;
    Dec(i);
  end;
end;

function CaseInsensitivePos(const SubString, S: string): Integer;
begin
  Result := Pos(UpperCase(SubString), UpperCase(S));
end;

function Min(v1, v2: Integer): Integer;
begin
  if v1 < v2 then
    Result := v1
  else
    Result := v2;
end;

function Max(v1, v2: Integer): Integer;
begin
  if v1 > v2 then
    Result := v1
  else
    Result := v2;
end;

{ Get a property value and return it as a string. }

function GetPropAsString(CompIntf: TIComponentInterface; Index: Integer): string;
var
  VariantValue: Variant;
  OrdinalValue: LongInt;
  MethodValue: TMethod;
  FloatValue: Extended;
begin
  case CompIntf.GetPropType(Index) of
    ptString, ptLString, ptLWString:
      begin
        CompIntf.GetPropValue(Index, Result);
      end;
    ptInteger, ptChar, ptEnumeration, ptSet, ptClass, ptWChar:
      begin
        CompIntf.GetPropValue(Index, OrdinalValue);
        case CompIntf.GetPropType(Index) of
          ptInteger, ptEnumeration:
            Result := IntToStr(OrdinalValue);
          ptSet, ptClass:
            Result := '$' + IntToHex(OrdinalValue, 8); //! StH: BCB - 0x as hex prefix?!
          ptChar:
            Result := Char(OrdinalValue);
          ptWChar:
            Result := WideChar(OrdinalValue);
        end;
      end;
    ptFloat:
      begin
        CompIntf.GetPropValue(Index, FloatValue);
        Result := FloatToStr(FloatValue);
      end;
    ptMethod:
      begin
        CompIntf.GetPropValue(Index, MethodValue);
        Result := Format('Method(%p)', [MethodValue.Code]); //! StH: To do - localize?
      end;
    ptVariant:
      begin
        CompIntf.GetPropValue(Index, VariantValue);
        Result := VariantValue;
      end;
  else
    Result := '';
  end;
end;

{ Set a property value from a string }

procedure SetPropAsString(CompIntf: TIComponentInterface; Index: Integer; const Value: string);
var
  FloatValue: Extended;
  IntegerValue: Integer;
begin
  try
    case CompIntf.GetPropType(Index) of
      ptString, ptLString, ptLWString:
        begin
          CompIntf.SetProp(Index, Value);
        end;
      ptInteger, ptEnumeration:
        begin
          IntegerValue := StrToInt(Value);
          CompIntf.SetProp(Index, IntegerValue);
        end;
      ptFloat:
        begin
          FloatValue := StrToFloat(Value);
          CompIntf.SetProp(Index, FloatValue);
        end;
    end;
  except
    on E: Exception do
    begin
      // nothing - swallow each exception
    end;
  end;
end;

function GetPropIndex(CompIntf: TIComponentInterface; const Name: string): Integer;
begin
  Result := CompIntf.GetPropCount - 1;
  while Result >= 0 do
  begin
    if CompareText(CompIntf.GetPropname(Result), Name) = 0 then
      Break;
    Dec(Result);
  end;
end;

procedure AssertIsDprOrPas(const FileName: string);
resourcestring
  SExpertForPasOrDprOnly = 'This expert is for use in .PAS or .DPR files only';
begin
  if not IsDprOrPas(FileName) then
    raise Exception.Create(SExpertForPasOrDprOnly);
end;

function IsDprOrPas(const FileName: string): Boolean;
var
  FileExt: string;
begin
  FileExt := ExtractUpperFileExt(FileName);
  Result := ((FileExt = '.PAS') or (FileExt = '.DPR'));
end;

function IsDfm(const FileName: string): Boolean;
begin
  Result := (ExtractUpperFileExt(FileName) = '.DFM');
end;

function IsCpp(const FileName: string): Boolean;
begin
  Result := (ExtractUpperFileExt(FileName) = '.CPP');
end;

function IsKnownSourceFile(const FileName: string): Boolean;
begin
  Result := IsDprOrPas(FileName) or IsCpp(FileName);
end;

function IsCppBuilderIde: Boolean;
begin
  Result := False;
  if ToolServices <> nil then
    Result := (Pos('C++BUILDER', UpperCase(ToolServices.GetBaseRegistryKey)) <> 0);
end;

function ExtractUpperFileExt(const FileName: string): string;
begin
  Result := UpperCase(ExtractFileExt(FileName));
end;

function DLLName: string;
var
  Buf: array[0..MAX_PATH + 1] of Char;
begin
  GetModuleFileName(HINSTANCE, Buf, SizeOf(Buf) - 1);
  Result := StrPas(Buf);
end;

function GetCurrentEditView(EditorIntf: TIEditorInterface): Integer;
var
  EditorFileName: string;
  WindowIterator: HWND;
  Buffer: array[0..255] of Char;
  APos: PChar;
begin
  Assert(EditorIntf <> nil);
  Result := -1;
  if EditorIntf.GetViewCount = 1 then
  begin
    // If there is only one edit view, then it is clear that
    // the first one is the current edit view
    Result := 0;
  end
  else
  begin
    EditorFileName := UpperCase(ExtractFileName(EditorIntf.FileName));
    WindowIterator := GetWindow(GetDesktopWindow, GW_CHILD);
    // Iterate over all windows whose owner is the Desktop
    WindowIterator := GetWindow(WindowIterator, GW_HWNDNEXT);

    // If we find a window with the class name "TEditWindow"
    // and the filenames match then this is the currently active
    // edit view
    while (WindowIterator <> 0) and IsWindow(WindowIterator) do
    begin
      if GetClassName(WindowIterator, Buffer, SizeOf(Buffer) - 1) = 0 then
        RaiseLastWin32Error;
      if StrPos(Buffer, 'TEditWindow') <> nil then
      begin
        if GetWindowText(WindowIterator, Buffer, SizeOf(Buffer) - 1) = 0 then
          RaiseLastWin32Error;
        StrUpper(Buffer);
        if StrPos(Buffer, PChar(EditorFileName)) <> nil then
        begin
          // In the case where there are multiple edit windows
          // open the first one you come to in the iteration process
          // should always be the top-most (or most recently active) edit
          // window - JCH
          // Scan window caption from the end; if we started at the
          // beginning, we might run into the colon of C:\MyFile
          APos := StrRScan(Buffer, ':');
          Inc(APos);
          Result := StrToIntDef(StrPas(APos), -1);
          // Subtract 1 since we need 0..GetViewCount-1 rather than 1..GetViewCount
          if Result <> -1 then
            Dec(Result);
          Break;
        end;
      end;
      WindowIterator := GetWindow(WindowIterator, GW_HWNDNEXT);
    end;
  end;
end;

function GetIdeMainForm: TCustomForm;
begin
  Result := Application.FindComponent('AppBuilder') as TCustomForm;
  {$IFOPT D+} if Result = nil then SendDebug('Unable to find AppBuilder!'); {$ENDIF}
end;

function GetComponentPaletteTabControl: TTabControl;
var
  MainForm: TCustomForm;
begin
  Result := nil;
  MainForm := GetIdeMainForm;
  if MainForm <> nil then
    Result := TTabControl(MainForm.FindComponent('TabControl'));
  {$IFOPT D+}if Result = nil then SendDebug('Unable to find TabControl!'); {$ENDIF}
end;

procedure ShowExceptionErrorMessage(E: Exception);
begin
  ProcessException(E);
  MessageDlg(E.Message, mtError, [mbOK], 0);
end;

procedure ProcessExceptionMsg(E: Exception; const Msg: string);
begin
  MessageDlg(Msg + E.Message, mtError, [mbOK], 0);
end;

procedure ProcessException(E: Exception);
begin
end;

initialization
  {$IFDEF SYNTAXMEMO}
  FillChar(Parsers, SizeOf(Parsers), 0);
finalization
  FreeSyntaxParsers;
  {$ENDIF SYNTAXMEMO}
end.


