unit GX_uGrid;

{$I GX_CondDefine.inc}

interface

//! StH: This unit has been completely prepared for localization
//! StH: This unit is fully compatible with C++ Builder

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, Grids, ExtCtrls, EditIntf, ToolIntf, ExptIntf, SortGrid, GX_uExperts;

type
  TfmCompGrid = class(TForm)
    pnlToolbar: TPanel;
    StringGrid: TSortGrid;
    sbSave: TSpeedButton;
    sbHelp: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure StringGridSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: string);
    procedure sbSaveClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure StringGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure sbHelpClick(Sender: TObject);
  private
    { Private declarations }
    FormIntf: TIFormInterface;
    ModIntf: TIModuleInterface;
    FModified: Boolean;
    procedure LoadComponents;
  public
    { Public declarations }
    property Modified: Boolean read FModified write FModified;
  end;

  TGridExpert = class(TGX_Expert)
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    function IconFileName: string; override;
    procedure Click(Sender: TObject); override;
  end;


implementation

{$R *.DFM}

uses
  GX_uGenFunc, GX_GExperts, GX_uConfigurationInfo;

procedure TfmCompGrid.FormCreate(Sender: TObject);
resourcestring
  SCellComponent = 'Component';
begin
  FormIntf := nil;
  ModIntf := nil;
  FModified := false;
  with StringGrid do
  begin
    Cells[0, 0] := SCellComponent;
    Cells[1, 0] := 'Parent';       // do not localize
    Cells[2, 0] := 'Tag';          // do not localize
    Cells[3, 0] := 'HelpContext';  // do not localize
  end;
  LoadComponents;
end;

procedure TfmCompGrid.LoadComponents;
var
  CompIntf, FormCompIntf: TIComponentInterface;

  function GetParentName: string;
  resourcestring
    SNoParent = 'No Parent';
  var
    Comp: TControl;
    Parent: TControl;
  begin
    Result := SNoParent;
    if CompIntf = nil then
      Exit;
    if TComponent(CompIntf.GetComponentHandle) is TControl then
    begin
      Comp := TControl(CompIntf.GetComponentHandle);
      if Comp <> nil then
      begin
        if Comp.Parent <> nil then
        begin
          Parent := TControl(Comp.Parent);
          Result := Parent.Name;
        end;
      end;
    end;
  end;

resourcestring
  SCouldNotGetModuleIntf = 'Could not get module interface for %s.';
  SCouldNotGetFormIntf = 'Could not get form interface.';
  SNotAvailable = 'N/A';
var
  UnitName: string;
  CurrentFileName: string;
  i, p: Integer;
  ComponentTag: string;
  ComponentHelpContext: string;
  ComponentName: string;
  ParentName: string;
  FileExt: string;
begin
  try
    CurrentFileName := UpperCase(ToolServices.GetCurrentFile);
    if not IsDfm(CurrentFileName) then
    begin
      MessageDlg('This Expert is for use in .DFM files only', mtError, [mbOK], 0);
      Exit;
    end;

    { Get module interface }
    FileExt := ExtractFileExt(CurrentFileName);
    i := Pos(FileExt, CurrentFileName);
    if i > 0 then
    begin
      UnitName := Copy(CurrentFileName, 1, i) + 'PAS';
      ModIntf := ToolServices.GetModuleInterface(UnitName);
      if ModIntf = nil then
      begin
        UnitName := Copy(CurrentFileName, 1, i) + 'CPP'; //! StH: Or is that HPP?
        ModIntf := ToolServices.GetModuleInterface(UnitName);
      end;
    end
    else
    begin
      UnitName := CurrentFileName;
      ModIntf := ToolServices.GetModuleInterface(UnitName);
    end;

    if ModIntf = nil then
    begin
      MessageDlg(Format(SCouldNotGetModuleIntf, [UnitName]), mtError, [mbOK], 0);
      Exit;
    end;

    FormIntf := ModIntf.GetFormInterface;
    if FormIntf = nil then
    begin
      MessageDlg(SCouldNotGetFormIntf, mtError, [mbOK], 0);
      Exit;
    end;

    FormCompIntf := FormIntf.GetFormComponent;
    try
      if FormCompIntf <> nil then
        for i := 0 to FormCompIntf.GetComponentCount - 1 do
        begin
          CompIntf := FormCompIntf.GetComponent(i);
          try
            if CompIntF <> nil then
            begin
              ParentName := GetParentName;
              SetLength(ComponentName, 255);
              p := GetPropIndex(CompIntf, 'TAG');  // do not localize
              if p < 0 then
                ComponentTag := SNotAvailable
              else
                ComponentTag := GetPropAsString(CompIntf, p);
              p := GetPropIndex(CompIntf, 'HELPCONTEXT');  // do not localize
              if p < 0 then
                ComponentHelpContext := SNotAvailable
              else
                ComponentHelpContext := GetPropAsString(CompIntf, p);
              if Length(Trim(ComponentHelpContext)) = 0 then
                ComponentHelpContext := '0';
              p := GetPropIndex(CompIntf, 'NAME');  // do not localize
              if p < 0 then
                ComponentName := SNotAvailable
              else
                ComponentName := GetPropAsString(CompIntf, p);
              with StringGrid do
              begin
                Cells[0, RowCount - 1] := ComponentName;
                Cells[1, RowCount - 1] := ParentName;
                Cells[2, RowCount - 1] := ComponentTag;
                Cells[3, RowCount - 1] := ComponentHelpContext;
                Objects[0, RowCount - 1] := Pointer(i);
                RowCount := RowCount + 1;
              end;
            end;
          finally
            CompIntf.Free;
          end;
        end;
    finally
      if (FormCompIntf <> nil) and (FormCompIntf.GetComponentHandle <> nil) then
        FormCompIntf.Free;  //! ????? Why do we need   FormCompIntf.GetComponentHandle <> nil
    end;
    if StringGrid.RowCount > 2 then
      StringGrid.RowCount := StringGrid.RowCount - 1;
  except
    on E: Exception do
      ShowExceptionErrorMessage(E);
  end;
end;

procedure TfmCompGrid.FormDestroy(Sender: TObject);
begin
  FormIntf.Free;
  FormIntf := nil;

  ModIntf.Free;
  ModIntf := nil;
end;

procedure TfmCompGrid.StringGridSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: string);
begin
  FModified := True;
end;

procedure TfmCompGrid.sbSaveClick(Sender: TObject);
var
  n, p, i: Integer;
  CompIntf: TIComponentInterface;
  FormCompIntf: TIComponentInterface;
begin
  FormCompIntf := FormIntf.GetFormComponent;
  try
    for i := 1 to StringGrid.RowCount - 1 do
    begin
      n := Integer(StringGrid.Objects[0, i]);
      CompIntf := FormCompIntf.GetComponent(n);
      if CompIntf <> nil then
      begin
        p := GetPropIndex(CompIntf, 'TAG');
        if p >= 0 then
          SetPropAsString(CompIntf, p, StringGrid.Cells[2, i]);
        p := GetPropIndex(CompIntf, 'HELPCONTEXT');
        if p >= 0 then
          SetPropAsString(CompIntf, p, StringGrid.Cells[3, i]);
      end;
    end;
    FModified := False;
  finally
    FormCompIntf.Free;
  end;
end;

procedure TfmCompGrid.FormClose(Sender: TObject; var Action: TCloseAction);
resourcestring
  SSaveChanges = 'You have unsaved changes, save them before closing?';
begin
  if FModified then
    if MessageDlg(SSaveChanges, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      sbSaveClick(sbSave);
end;

procedure TfmCompGrid.StringGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  if (ACol = 3) and (StringGrid.Cells[3, ARow] = 'N/A') then
    CanSelect := False;
end;

//************************ TGridExpert ********************

constructor TGridExpert.Create;
begin
  inherited Create;
  ShortCut := 0;
  HasConfigOptions := False;
  HasMenuItem := True;
end;

destructor TGridExpert.Destroy;
begin
  inherited Destroy;
end;

function TGridExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = 'Component &Grid...';
begin
  Result := SMenuCaption;
end;

function TGridExpert.GetMenuName: string;
begin
  Result := 'GX_Grid';
end;

function TGridExpert.GetMenuMask: string;
begin
  Result := '.DFM';
end;

function TGridExpert.GetName: string;
begin
  Result := 'Component_Grid_Expert';
end;

function TGridExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'Component Grid';
begin
  Result := SDisplayName;
end;

procedure TGridExpert.Click(Sender: TObject);
begin
  with TfmCompGrid.Create(nil) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

function TGridExpert.IconFileName: string;
begin
  Result := 'Grid';
end;

procedure TfmCompGrid.sbHelpClick(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 22);
end;

initialization
  RegisterGX_Expert(TGridExpert);
end.

