unit GX_uLayout;

{
  Name: GX_uLayout
  Purpose: positions app, edit and object inspector windows according to pre-
           determined, or user specified, locations

  Maintenance History:
  --------------------
  1998/12/15 Scott Mattes
             added the TfmLayout.btnCustomClick event and code modified from
             the LoadCustomLayout embedded procedure of the TLayoutExpert.Click
             event to save the user's current settings.
}

{$I GX_CondDefine.inc}

interface

//! StH: This unit has been completely prepared for localization
//! StH: This unit is fully compatible with C++ Builder

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, GX_uExperts, ToolIntf, ExptIntf, Registry;

type
  TfmLayout = class(TForm)
    gbxLayout: TGroupBox;
    Panel17: TPanel;
    Shape1: TShape;
    Shape2: TShape;
    Shape3: TShape;
    Shape4: TShape;
    Label11: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Panel18: TPanel;
    Shape5: TShape;
    Shape6: TShape;
    Shape7: TShape;
    Shape8: TShape;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    rblayout1: TRadioButton;
    rblayout2: TRadioButton;
    rbCustom: TRadioButton;
    btnCustom: TButton;
    Button1: TButton;
    Button2: TButton;
    btnHelp: TButton;
    procedure btnCustomClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  TLayoutType = (ltLayout1, ltLayout2, ltCustom);

  TLayoutExpert = class(TGX_Expert)
  private
    FLayoutType: TLayoutType;
  public
    constructor Create; override;
    destructor Destroy; override;
    function GetMenuCaption: string; override;
    function GetMenuName: string; override;
    function GetMenuMask: string; override;
    function GetName: string; override;
    function GetDisplayName: string; override;
    function IconFileName: string; override;
    procedure Click(Sender: TObject); override;
    procedure LoadSettings; override;
    procedure SaveSettings; override;
    procedure Configure; override;
    property LayoutType: TLayoutType read FLayoutType write FLayoutType;
  end;

implementation

{$R *.DFM}

uses
  Menus, GX_uGenFunc, GX_GExperts, GX_uConfigurationInfo;

procedure TfmLayout.btnCustomClick(Sender: TObject);
var
  RegIni: TRegIniFile;
  i: Integer;

  procedure SaveWindow(Form: TForm);
  begin
    // Don't save the state of the two GExperts configuration dialogs
    if (Form.Name = 'fmConfiguration') or (Form.Name = 'fmLayout') then
      Exit;
    // Don't save project forms or invisible forms
    if (Form.Designer <> nil) or (not Form.Visible) then
      Exit;
    RegIni.WriteInteger(Form.Name, 'Left', Form.Left);
    RegIni.WriteInteger(Form.Name, 'Top', Form.Top);
    RegIni.WriteInteger(Form.Name, 'Height', Form.Height);
    RegIni.WriteInteger(Form.Name, 'Width', Form.Width);
  end;

begin
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts\Layout');
  try
    TRegistry(RegIni).DeleteKey(ConfigInfo.RegKey + '\GExperts\Layout');
    RegIni.OpenKey(ConfigInfo.RegKey + '\GExperts\Layout', True);
    for i := 0 to Screen.FormCount - 1 do
      SaveWindow(TForm(Screen.Forms[i]));
  finally
    RegIni.Free;
  end;
end;

//************************ TMsgExpExpert ********************

constructor TLayoutExpert.Create;
begin
  inherited Create;
  {$IFDEF GX_VER130_up}
  DefaultActive := False; // Delphi 5+ provide more powerful saved desktops
  {$ENDIF GX_VER130_up}
  ShortCut := Menus.ShortCut(Word('L'), [ssCtrl, ssShift]);
  HasConfigOptions := True;
  HasMenuItem := True;
end;

destructor TLayoutExpert.Destroy;
begin
  inherited Destroy;
end;

function TLayoutExpert.GetMenuCaption: string;
resourcestring
  SMenuCaption = 'Perfect La&yout';
begin
  Result := SMenuCaption;
end;

function TLayoutExpert.GetMenuName: string;
begin
  Result := 'GX_Layout';  // do not localize
end;

function TLayoutExpert.GetMenuMask: string;
begin
  Result := '';
end;

function TLayoutExpert.GetName: string;
begin
  Result := 'Layout_Expert';  // do not localize
end;

function TLayoutExpert.GetDisplayName: string;
resourcestring
  SDisplayName = 'Perfect Layout';
begin
  Result := SDisplayName;
end;

procedure TLayoutExpert.Click(Sender: TObject);

  procedure LoadCustomLayout;
  var
    RegIni: TRegIniFile;
    i: Integer;

    procedure LoadWindow(Form: TForm);
    begin
      Form.WindowState := wsNormal;
      // do not localize any of the below
      Form.Left := RegIni.ReadInteger(Form.Name, 'Left', Form.Left);
      Form.Top := RegIni.ReadInteger(Form.Name, 'Top', Form.Top);
      Form.Height := RegIni.ReadInteger(Form.Name, 'Height', Form.Height);
      Form.Width := RegIni.ReadInteger(Form.Name, 'Width', Form.Width);
    end;

  begin
    RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts\Layout');
    try
      for i := 0 to Screen.FormCount - 1 do
        LoadWindow(TForm(Screen.Forms[i]));
    finally
      RegIni.Free;
    end;
  end;


  function FindForm(Name: string): TForm;
  var
    App: TCustomForm;
  begin
    Result := Application.FindComponent(Name) as TForm;
    if Result = nil then
    begin
      App := GetIdeMainForm;
      if App <> nil then
        Result := App.FindComponent(Name) as TForm;
    end;
  end;

resourcestring
  SCouldNotFindAppBuilder = 'Could not find IDE AppBuilder window.';
var
  OI, Watch, Editor: TCustomForm;
  App: TCustomForm;
  Left, Width, Top, Bottom: Integer;
  R: TRect;
begin
  if LayoutType = ltCustom then
  begin
    LoadCustomLayout;
    Exit;
  end;
  SystemParametersInfo(SPI_GETWORKAREA, 0, @R, 0);

  App := GetIdeMainForm;
  if App = nil then
  begin
    MessageDlg(SCouldNotFindAppBuilder, mtError, [mbOK], 0);
    Exit;
  end;
  App.WindowState := wsNormal;
  App.SetBounds(R.Left, R.Top, R.Right-R.Left, App.Height);
  Top := R.Top + App.height;
  Bottom := R.Bottom - R.Top;
  Watch := FindForm('WatchWindow');  // do not localize
  if Watch <> nil then
  begin
    if Watch.Visible {$IFDEF GX_VER120_up} and Watch.Floating {$ENDIF} then
    begin
      Watch.WindowState := wsNormal;
      Watch.SetBounds(R.Left, R.Bottom - Watch.Height, R.Right - R.Left, Watch.height);
      Bottom := Watch.Top;
    end;
  end;
  OI := FindForm('PropertyInspector'); // do not localize
  if OI <> nil then
  begin
    // In case the OI is docked:
    if GetParentForm(OI) <> nil then
      OI := GetParentForm(OI);
    OI.Top := Top;
    OI.Height := Bottom - Top;
    if LayoutType = ltLayout1 then
    begin
      OI.Left := R.Left;
      Left := OI.Left + OI.Width;
    end
    else
    begin
      OI.Left := R.Right - R.Left - OI.Width;
      Left := R.Left;
    end;
    Width := R.Right - R.Left - OI.Width;
  end
  else
  begin
    Left := R.Left;
    Width := R.Right - R.Left;
  end;
  Editor := FindForm('EditWindow_0'); // do not localize
  if Editor <> nil then
  begin
    Editor.WindowState := wsNormal;
    Editor.SetBounds(Left, Top, Width, Bottom - Top);
  end;
end;

procedure TLayoutExpert.LoadSettings;
var
  RegIni: TRegIniFile;
begin
  inherited LoadSettings;

  // do not localize any of the below items
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    FLayoutType := TLayoutType(RegIni.ReadInteger('Misc', 'Layout', 0));
  finally
    RegIni.Free;
  end;
end;

procedure TLayoutExpert.SaveSettings;
var
  RegIni: TRegIniFile;
begin
  inherited SaveSettings;

  // do not localize any of the below items
  RegIni := TRegIniFile.Create(ConfigInfo.RegKey + '\GExperts');
  try
    RegIni.WriteInteger('Misc', 'Layout', ord(FLayoutType));
  finally
    RegIni.Free;
  end;
end;

procedure TLayoutExpert.Configure;
var
  Dlg: TfmLayout;
begin
  Dlg := TfmLayout.Create(nil);
  try
    Dlg.rbLayout1.Checked := (FLayoutType = ltLayout1);
    Dlg.rbLayout2.Checked := (FLayoutType = ltLayout2);
    Dlg.rbCustom.Checked := (FLayoutType = ltCustom);
    if Dlg.ShowModal = mrOK then
    begin
      if Dlg.rbLayout1.Checked then
        FLayoutType := ltLayout1
      else
      if Dlg.rbLayout2.Checked then
        FLayoutType := ltLayout2
      else
      if Dlg.rbCustom.Checked then
        FLayoutType := ltCustom;
    end;
  finally
    Dlg.Free;
  end;
end;

function TLayoutExpert.IconFileName: string;
begin
  Result := 'Layout';
end;

procedure TfmLayout.btnHelpClick(Sender: TObject);
begin
  WinHelp(Self.Handle, PChar(ConfigInfo.HelpFile), HELP_CONTEXT, 25);
end;

initialization
  RegisterGX_Expert(TLayoutExpert);

end.

