{ This unit is used to perform string substitution for string variables
  The following variables are supported:
   %PROJECTDIR% - Current project directory
   %PROJECTNAME% - Project file name
   %UNIT% - Current unit in the code editor
   %PROCNAME% - Current procedure -- only for Object Pascal
   %DATE%, %DATETIME%, %YEAR%, %MONTH%, %MONTHSHORTNAME%
   %MONTHLONGNAME%, %DAY%, %DAYSHORTNAME%, %DAYLONGNAME%
}

unit GX_uSubst;

interface

//! StH: This unit has been completely prepared for localization
//! StH: This unit is compatible with C++ Builder
//!      --> BUT there is only a Pascal parser available; hence do
//!          pass in DoParseSource = False if the current file is
//!          not Object Pascal source code

uses
  Classes, SysUtils, ToolIntf, ExptIntf;

function ReplaceStrings(const RawString: string; DoParseSource: Boolean): string;

implementation

uses
  GX_uConfigurationInfo, GX_uGenFunc, mPasLex, Gx_EditRead;

resourcestring
  SUnknownNameResult = 'Not available';

function GetProcName: string;
var
  Parser: TmwPasLex;
  EditRead: TEditReader;
  MemStream: TMemoryStream;
begin
  Result := SUnknownNameResult;
  MemStream := TMemoryStream.Create;
  try
    // Since this edit reader is destroyed almost
    // almost immediately, do not call FreeFileData
    EditRead := TEditReader.Create(ToolServices.GetCurrentFile);
    try
      EditRead.SaveToStreamFromPos(MemStream);
    finally
      EditRead.Free;
    end;
    Parser := TmwPasLex.Create;
    try
      Parser.Origin := MemStream.Memory;
      while not (Parser.TokenID in [tkNull, tkProcedure, tkFunction, tkConstructor, tkDestructor]) do
        Parser.NextNoJunk;
      if Parser.TokenID in [tkProcedure, tkFunction, tkConstructor, tkDestructor] then
      begin
        Parser.NextNoJunk;
        if Parser.TokenID = tkIdentifier then
          Result := Parser.Token;
        Parser.NextNoJunk;
        if Parser.TokenID = tkPoint then
        begin
          Parser.NextNoJunk;
          Result := Result + '.' + Parser.Token;
        end;
      end;
    finally
      Parser.Free;
    end;
  finally
    MemStream.Free;
  end;
end;

function GetUnitName: string;
var
  Parser: TmwPasLex;
  EditRead: TEditReader;
  MemStream: TMemoryStream;
begin
  Result := SUnknownNameResult;
  MemStream := TMemoryStream.Create;
  try
    // Since this edit reader is destroyed almost
    // almost immediately, do not call FreeFileData
    EditRead := TEditReader.Create(ToolServices.GetCurrentFile);
    try
      EditRead.SaveToStream(MemStream);
    finally
      EditRead.Free;
    end;
    Parser := TmwPasLex.Create;
    try
      Parser.Origin := MemStream.Memory;
      while (Parser.TokenID <> tkNull) and (Parser.TokenID <> tkUnit) do
        Parser.NextNoJunk;
      if Parser.TokenID = tkUnit then
      begin
        Parser.NextNoJunk;
        if Parser.TokenID = tkIdentifier then
          Result := Parser.Token;
      end;
    finally
      Parser.Free;
    end;
  finally
    MemStream.Free;
  end;
end;

function GetReplaceString(StVar: string; DoParseSource: Boolean): string;
begin
  // do not localize any of the below items
  Result := '';
  StVar := UpperCase(StVar);
  if StVar = '%PROJECTDIR%' then
    Result := GetProjectDir
  else
  if StVar = '%PROJECTNAME%' then
    Result := GetProjectName
  else
  if StVar = '%DATETIME%' then
    Result := DateTimetoStr(Date + Time)
  else
  if StVar = '%DATE%' then
    Result := DatetoStr(Date)
  else
  if StVar = '%YEAR%' then
    Result := FormatDateTime('yyyy', Date)  //! Y2K lesson <g>
  else
  if StVar = '%MONTH%' then
    Result := FormatDateTime('mm', Date)
  else
  if StVar = '%MONTHSHORTNAME%' then
    Result := FormatDateTime('mmm', Date)
  else
  if StVar = '%MONTHLONGNAME%' then
    Result := FormatDateTime('mmmm', Date)
  else
  if StVar = '%DAY%' then
    Result := FormatDateTime('dd', Date)
  else
  if StVar = '%DAYSHORTNAME%' then
    Result := FormatDateTime('ddd', Date)
  else
  if StVar = '%DAYLONGNAME%' then
    Result := FormatDateTime('dddd', Date)
  else
  if StVar = '%UNIT%' then
  begin
    if DoParseSource then
      Result := GetUnitName
    else
      Result := SUnknownNameResult;
  end
  else
  if StVar = '%PROCNAME%' then
  begin
    if DoParseSource then
      Result := GetProcName
    else
      Result := SUnknownNameResult;
  end;
end;

function ReplaceStrings(const RawString: string; DoParseSource: Boolean): string;
resourcestring
  SReplaceError = 'Error!';
var
  i: Integer;
  t: Integer;
begin
  i := 1;
  while i <= Length(RawString) do
  begin
    if RawString[i] <> '%' then
      Result := Result + RawString[i]
    else
    begin
      if i < Length(RawString) then
        if RawString[i + 1] = '%' then
        begin
          Result := Result + '%';
          Inc(i, 2);
          Continue;
        end;
      t := i + 1;
      while (t <= Length(RawString)) and (RawString[t] <> '%') do
        Inc(t);
      if t <= Length(RawString) then
        Result := Result + GetReplaceString(Copy(RawString, i, t - i + 1), DoParseSource)
      else
        Result := Result + SReplaceError;
      i := t;
    end;
    Inc(i);
  end;
end;

end.

